#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DroneFlightDirector_test.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import unittest

import RTC
import OpenRTM_aist
import DroneFlightDirector

import sys
from collections import namedtuple

from mock import Mock
from mock import patch, MagicMock


EXECUTE_RATE = 4
INPORT_SET_NUM = 3
DRONE_NUM = 3 #制御するドローン数


class TestDroneFlightDirector(unittest.TestCase):
    def setUp(self):
        self.m = DroneFlightDirector.DroneFlightDirector(0)

        self.m._Drone_Num = DRONE_NUM
        self.m._execute_rate = EXECUTE_RATE
        self.m._MinAlt[0] = 2.0
        self.m._MaxAlt[0] = 15.0

        self.xx_logger = Mock()
        DroneFlightDirector.logger.warning = self.xx_logger.warning
        DroneFlightDirector.logger.error = self.xx_logger.error
        DroneFlightDirector.logger.info = self.xx_logger.info
        DroneFlightDirector.logger.debug = self.xx_logger.debug

    def test_read_inport(self):
        xx1 = Mock()
        xx2 = Mock()
        xx3 = Mock()
        self.m._inDrCtrlIn[0].isNew = xx1.isNew
        self.m._inDrCtrlIn[0].read = xx1.read
        self.m._inDrCtrlIn[1].isNew = xx2.isNew
        self.m._inDrCtrlIn[1].read = xx2.read
        self.m._inDrCtrlIn[2].isNew = xx3.isNew
        self.m._inDrCtrlIn[2].read = xx3.read

        init_drone_ctrl = RTC.DroneCtrl(RTC.Time(0,0),
                                    0,
                                    RTC.TimedString(RTC.Time(0,0), ""),
                                    RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                0.0, 0, RTC.GPS_FIX_NORMAL),
                                    RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                0.0, 0, RTC.GPS_FIX_NORMAL),
                                    RTC.TimedFloatSeq(RTC.Time(0,0),[0,0,0]),
                                    RTC.TimedFloat(RTC.Time(0,0),0),
                                    RTC.TimedString(RTC.Time(0,0),""))
        drone_ctrl1 = RTC.DroneCtrl(RTC.Time(0,0),
                                    100,
                                    RTC.TimedString(RTC.Time(0,0), ""),
                                    RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                37.0, 138.0, 215.0, 0.0, 0.0, 0.0, 0.0,
                                                0.0, 0, RTC.GPS_FIX_NORMAL),
                                    RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                37.1, 138.1, 10.0, 0.0, 0.0, 0.0, 0.0,
                                                0.0, 0, RTC.GPS_FIX_NORMAL),
                                    RTC.TimedFloatSeq(RTC.Time(0,0),[3,2,1]),
                                    RTC.TimedFloat(RTC.Time(0,0),45),
                                    RTC.TimedString(RTC.Time(0,0),"Hovering"))

        drone_ctrl2 = RTC.DroneCtrl(RTC.Time(0,0),
                                   100,
                                   RTC.TimedString(RTC.Time(0,0), ""),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               37.5, 138.5, 215.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               37.6, 138.6, 10.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.TimedFloatSeq(RTC.Time(0,0),[4,3,2]),
                                   RTC.TimedFloat(RTC.Time(0,0),50),
                                   RTC.TimedString(RTC.Time(0,0),"Alt_CHG"))

        drone_ctrl3 = RTC.DroneCtrl(RTC.Time(0,0),
                                   100,
                                   RTC.TimedString(RTC.Time(0,0), ""),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               38.0, 139.0, 215.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               38.1, 139.1, 10.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.TimedFloatSeq(RTC.Time(0,0),[5,4,3]),
                                   RTC.TimedFloat(RTC.Time(0,0),55),
                                   RTC.TimedString(RTC.Time(0,0),"In_Move"))

        id_result = [0] * INPORT_SET_NUM
        home_lat_result = [0] * INPORT_SET_NUM
        home_lon_result = [0] * INPORT_SET_NUM
        home_alt_result = [0] * INPORT_SET_NUM
        drone_lat_result = [0] * INPORT_SET_NUM
        drone_lon_result = [0] * INPORT_SET_NUM
        drone_alt_result = [0] * INPORT_SET_NUM
        accel_x_result = [0] * INPORT_SET_NUM
        accel_y_result = [0] * INPORT_SET_NUM
        accel_z_result = [0] * INPORT_SET_NUM
        mag_result = [0] * INPORT_SET_NUM
        state_result = [0] * INPORT_SET_NUM

        self.m._d_inDrCtrl[0] = init_drone_ctrl
        self.m._d_inDrCtrl[1] = init_drone_ctrl
        self.m._d_inDrCtrl[2] = init_drone_ctrl
        xx1.read.return_value = drone_ctrl1
        xx2.read.return_value = drone_ctrl2
        xx3.read.return_value = drone_ctrl3

        in_is_new = [False,True]
        id_result[0] = [0, 100]
        home_lat_result[0] = [0, 37.0]
        home_lon_result[0] = [0, 138.0]
        home_alt_result[0] = [0, 215.0]
        drone_lat_result[0] = [0, 37.1]
        drone_lon_result[0] = [0, 138.1]
        drone_alt_result[0] = [0, 10.0]
        accel_x_result[0] = [0, 3]
        accel_y_result[0] = [0, 2]
        accel_z_result[0] = [0, 1]
        mag_result[0] = [0, 45]
        state_result[0] = ["", "Hovering"]
        id_result[1] = [0, 100]
        home_lat_result[1] = [0, 37.5]
        home_lon_result[1] = [0, 138.5]
        home_alt_result[1] = [0, 215.0]
        drone_lat_result[1] = [0, 37.6]
        drone_lon_result[1] = [0, 138.6]
        drone_alt_result[1] = [0, 10.0]
        accel_x_result[1] = [0, 4]
        accel_y_result[1] = [0, 3]
        accel_z_result[1] = [0, 2]
        mag_result[1] = [0, 50]
        state_result[1] = ["", "Alt_CHG"]
        id_result[2] = [0, 100]
        home_lat_result[2] = [0, 38.0]
        home_lon_result[2] = [0, 139.0]
        home_alt_result[2] = [0, 215.0]
        drone_lat_result[2] = [0, 38.1]
        drone_lon_result[2] = [0, 139.1]
        drone_alt_result[2] = [0, 10.0]
        accel_x_result[2] = [0, 5]
        accel_y_result[2] = [0, 4]
        accel_z_result[2] = [0, 3]
        mag_result[2] = [0, 55]
        state_result[2] = ["", "In_Move"]

        for i,d in enumerate(in_is_new):
            xx1.isNew.return_value = in_is_new[i]
            xx2.isNew.return_value = in_is_new[i]
            xx3.isNew.return_value = in_is_new[i]
            self.m._read_inport()
            for drone_num in range(DRONE_NUM):
                self.assertEqual(self.m._d_inDrCtrl[drone_num].id, id_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].homeGPS.latitude, home_lat_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].homeGPS.longitude, home_lon_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].homeGPS.altitude, home_alt_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].droneGPS.latitude, drone_lat_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].droneGPS.longitude, drone_lon_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].droneGPS.altitude, drone_alt_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].accel.data[0], accel_x_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].accel.data[1], accel_y_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].accel.data[2], accel_z_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].mag.data, mag_result[drone_num][i])
                self.assertEqual(self.m._d_inDrCtrl[drone_num].state.data, state_result[drone_num][i])

    def test_is_rcv_inport_valid_val(self):

        in_state = [0] * INPORT_SET_NUM
        in_home = [0] * INPORT_SET_NUM
        in_drone = [0] * INPORT_SET_NUM
        in_mag = [0] * INPORT_SET_NUM

        in_all_armed = [True, True, True, True, True, True, True, True, True, True,
                        True, True, True, True, True, True, False, False, False]

        in_state[0] = ["Armed", "Takeoff", "In_Move", "Alt_CHG", "Hovering", "Yaw_CHG", "Armed",
                       "Disarm", "RTL", "Takeoff", "In_Move", "Takeoff", "Alt_CHG",
                       "Disarm", "Hovering", "Disarm", "In_Move", "Disarm", "RTL"]
        in_state[1] = ["Armed", "Takeoff", "In_Move", "Alt_CHG", "Hovering", "Yaw_CHG", "Armed",
                       "Armed", "Hovering", "Disarm", "RTL", "In_Move", "Armed",
                       "Disarm", "RTL", "RTL", "In_Move", "Disarm", "RTL"]
        in_state[2] = ["Armed", "Takeoff", "In_Move", "Alt_CHG", "Hovering", "Yaw_CHG", "Armed",
                       "Armed", "Takeoff", "In_Move", "Alt_CHG", "Disarm", "RTL",
                       "Alt_CHG", "Disarm", "RTL", "Hovering", "In_Move", "Alt_CHG"]

        in_drone[0] = [True, True, True, False, False, True, False,
                       False, False, True, True, True, True,
                       False, True, True, True, False, False]
        in_drone[1] = [True, True, False, True, True, False, True,
                       True, True, False, False, True, True,
                       False, False, True, True, False, False]
        in_drone[2] = [True, False, True, True, True, True, True,
                       True, True, True, True, False, False,
                       True, False, True, True, True, True]

        result = [True, False, False, False, False, False, False,
                  True, True, True, True, True, True,
                  True, True, False, False, False, False]

        test_case = 0
        while test_case < len(result):
            self.m._all_armed = in_all_armed[test_case]
            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]
                self.m._drone_gps[drone_num] = in_drone[drone_num][test_case]
                drone_num += 1

            self.assertEqual(self.m._is_rcv_inport_valid_val(), result[test_case])
            test_case += 1

    def test_is_all_drone_states_armed(self):

        in_state = [0] * INPORT_SET_NUM

        in_state[0] = ["Armed", "Takeoff", "Armed", "Armed", "Hovering", "Armed", "Takeoff", "Disarm"]
        in_state[1] = ["Armed", "Armed", "In_Move", "Armed", "Hovering", "Yaw_CHG", "Armed", "Takeoff"]
        in_state[2] = ["Armed", "Armed", "Armed", "Alt_CHG", "Armed", "Yaw_CHG", "Takeoff", "In_Move"]

        result = [True, False, False, False, False, False, False, False]

        test_case = 0
        while test_case < len(result):
            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]
                drone_num += 1

            self.m._is_all_drone_states_armed()

            self.assertEqual(self.m._all_armed, result[test_case])

            test_case += 1

    def test_set_leader(self):

        in_state = [0] * INPORT_SET_NUM
        in_accel = [0] * INPORT_SET_NUM

        in_state[0] = ["Hovering", "Hovering", "Hovering", "Disarm", "RTL"]
        in_state[1] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering"]
        in_state[2] = ["Hovering", "Hovering", "Disarm", "RTL", "Disarm"]

        in_accel[0] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_accel[1] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_accel[2] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_leader_set_first = [True, False, False, False, False]

        in_rtl_num = [0, 0, 0, 1, 2]


        leader_first_result = [False, False, False, False, False]
        rtm_num_result = [0, 0, 1, 2, 2]


        test_case = 0
        while test_case < len(leader_first_result):

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].accel.data = in_accel[drone_num][test_case]
                drone_num += 1

            self.m._leader_set_first = in_leader_set_first[test_case]
            self.m._rtl_num = in_rtl_num[test_case]

            self.m._set_leader()

            self.assertEqual(self.m._leader_set_first, leader_first_result[test_case])
            self.assertEqual(self.m._rtl_num, rtm_num_result[test_case])

            test_case += 1

    def test_calc_leader_pos(self):

        in_state = [0] * INPORT_SET_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_takeoff_alt = [0] * INPORT_SET_NUM
        in_mag = [0] * INPORT_SET_NUM
        in_accel = [0] * INPORT_SET_NUM

        in_avg_drone_alt = [5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0]
        in_state[0] = ["Hovering", "RTL", "Disarm", "Hovering", "Disarm", "RTL", "Hovering", "RTL", "Disarm"]
        in_state[1] = ["Hovering", "Hovering", "RTL", "Hovering", "Hovering", "Disarm", "Hovering", "Hovering", "Disarm"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering"]
        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_alt[0] = [0.0, 0.0, 0.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0]
        in_drone_alt[1] = [0.0, 0.0, 0.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0]
        in_drone_alt[2] = [0.0, 0.0, 0.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0]
        in_takeoff_alt[0] = [5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0]
        in_takeoff_alt[1] = [7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0]
        in_takeoff_alt[2] = [9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0]
        in_mag[0] = [320, 320, 320, 160, 160, 160, 270, 270, 270]
        in_mag[1] = [10, 10, 10, 190, 190, 190, 80, 80, 80]
        in_mag[2] = [120, 120, 120, 185, 185, 185, 100, 100, 100]
        in_rtl_num = [0, 1, 2, 0, 1, 2, 0, 1, 2]
        in_accel[0] = [[3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0],
                       [3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0]]
        in_accel[1] = [[2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0],
                       [2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0]]
        in_accel[2] = [[1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0],
                       [1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0]]

        avg_drone_lat_result = [37.530, 37.535, 37.540, 37.530, 37.535, 37.540, 37.530, 37.535, 37.540]
        avg_drone_lon_result = [139.940, 139.945, 139.950, 139.940, 139.945, 139.950, 139.940, 139.945, 139.950]
        avg_drone_alt_result = [7.0, 8.0, 9.0, 10.0, 11.0, 12.0, 10.0, 11.0, 12.0]
        avg_mag_result = [30, 65, 120, 178, 187, 185, 30, 90, 100]


        avg_speed_result = [2.799, 2.699, 3.162, 2.799, 2.699, 3.162, 2.799, 2.699, 3.162]

        test_case = 0
        while test_case < len(avg_drone_lat_result):
            self.m._avg_drone_pos.altitude = in_avg_drone_alt[test_case]
            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.latitude = in_drone_lat[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.longitude = in_drone_lon[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.altitude = in_drone_alt[drone_num][test_case]
                self.m._takeoff_alt[drone_num] = in_takeoff_alt[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].mag.data = in_mag[drone_num][test_case]

                self.m._d_inDrCtrl[drone_num].accel.data = in_accel[drone_num][test_case]
                drone_num += 1

            self.m._rtl_num = in_rtl_num[test_case]

            self.m._calc_leader_pos()

            self.assertEqual(self.m._avg_drone_pos.latitude, avg_drone_lat_result[test_case])
            self.assertEqual(self.m._avg_drone_pos.longitude, avg_drone_lon_result[test_case])
            self.assertEqual(self.m._avg_drone_pos.altitude, avg_drone_alt_result[test_case])
            self.assertEqual(self.m._avg_mag, avg_mag_result[test_case])
            self.assertEqual(round(self.m._avg_speed,3), avg_speed_result[test_case])

            test_case += 1


    def test_set_leader_pos(self):

        in_avg_lat = [37.530]
        in_avg_lon = [139.940]
        in_avg_alt = [7.0]

        leader_lat_result = [37.530]
        leader_lon_result = [139.940]
        leader_alt_result = [7.0]

        leader_target_lat_result = [37.530]
        leader_target_lon_result = [139.940]
        leader_target_alt_result = [7.0]

        test_case = 0
        while test_case < len(leader_lat_result):
            self.m._avg_drone_pos.latitude = in_avg_lat[test_case]
            self.m._avg_drone_pos.longitude = in_avg_lon[test_case]
            self.m._avg_drone_pos.altitude = in_avg_alt[test_case]

            self.m._set_leader_pos()

            self.assertEqual(self.m._leader_pos.latitude, leader_lat_result[test_case])
            self.assertEqual(self.m._leader_pos.longitude, leader_lon_result[test_case])
            self.assertEqual(self.m._leader_pos.altitude, leader_alt_result[test_case])
            self.assertEqual(self.m._leader_target.latitude, leader_target_lat_result[test_case])
            self.assertEqual(self.m._leader_target.longitude, leader_target_lon_result[test_case])
            self.assertEqual(self.m._leader_target.altitude, leader_target_alt_result[test_case])

            test_case += 1

    def test_drone_diff(self):

        in_state = [0] * INPORT_SET_NUM
        in_diff_lat = [0] * DRONE_NUM
        in_diff_lon = [0] * DRONE_NUM
        in_diff_alt = [0] * DRONE_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_takeoff_alt = [0] * INPORT_SET_NUM
        in_mag = [0] * INPORT_SET_NUM
        diff_lat_result = [0] * DRONE_NUM
        diff_lon_result = [0] * DRONE_NUM
        diff_alt_result = [0] * DRONE_NUM



        in_state[0] = ["Hovering", "RTL", "Disarm", "Disarm", "RTL"]
        in_state[1] = ["Hovering", "Hovering", "RTL", "Hovering", "Disarm"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering"]
        in_diff_lat[0] = [-0.15, -0.15, -0.15, -0.15, -0.15]
        in_diff_lat[1] = [0.15, 0.15, 0.15, 0.15, 0.15]
        in_diff_lat[2] = [0.25, 0.25, 0.25, 0.25, 0.25]
        in_diff_lon[0] = [-0.15, -0.15, -0.15, -0.15, -0.15]
        in_diff_lon[1] = [0.15, 0.15, 0.15, 0.15, 0.15]
        in_diff_lon[2] = [0.25, 0.25, 0.25, 0.25, 0.25]
        in_diff_alt[0] = [-1.0, -1.0, -1.0, -1.0, -1.0]
        in_diff_alt[1] = [2.0, 2.0, 2.0, 2.0, 2.0]
        in_diff_alt[2] = [5.0, 5.0, 5.0, 5.0, 5.0]

        in_leader_lat = [37.530, 37.535, 37.540, 37.535, 37.540]
        in_leader_lon = [139.940, 139.945, 139.950, 139.945, 139.950]
        in_leader_alt = [7.0, 8.0, 9.0, 11.0, 12.0]

        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_alt[0] = [0.0, 0.0, 0.0, 8.0, 8.0]
        in_drone_alt[1] = [0.0, 0.0, 0.0, 10.0, 10.0]
        in_drone_alt[2] = [0.0, 0.0, 0.0, 12.0, 12.0]
        in_takeoff_alt[0] = [5.0, 5.0, 5.0, 5.0, 5.0]
        in_takeoff_alt[1] = [7.0, 7.0, 7.0, 7.0, 7.0]
        in_takeoff_alt[2] = [9.0, 9.0, 9.0, 9.0, 9.0]


        diff_lat_result[0] = [-0.01, -0.15, -0.15, -0.15, -0.15]
        diff_lat_result[1] = [0.0, -0.005, 0.15, -0.005, 0.15]
        diff_lat_result[2] = [0.01, 0.005, 0.0, 0.005, 0.0]
        diff_lon_result[0] = [-0.01, -0.15, -0.15, -0.15, -0.15]
        diff_lon_result[1] = [0.0, -0.005, 0.15, -0.005, 0.15]
        diff_lon_result[2] = [0.01, 0.005, 0.0, 0.005, 0.0]
        diff_alt_result[0] = [-2.0, -1.0, -1.0, -1.0, -1.0]
        diff_alt_result[1] = [0.0, -1.0, 2.0, -1.0, 2.0]
        diff_alt_result[2] = [2.0, 1.0, 0.0, 1.0, 0.0]

        test_case = 0
        while test_case < len(diff_lat_result[0]):

            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]
            self.m._leader_pos.altitude = in_leader_alt[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].latitude = in_diff_lat[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].longitude = in_diff_lon[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].altitude = in_diff_alt[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.latitude = in_drone_lat[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.longitude = in_drone_lon[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.altitude = in_drone_alt[drone_num][test_case]
                self.m._takeoff_alt[drone_num] = in_takeoff_alt[drone_num][test_case]
                drone_num += 1


            self.m._drone_diff()

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.assertEqual(round(self.m._drone_diff_pos[drone_num].latitude,3), diff_lat_result[drone_num][test_case])
                self.assertEqual(round(self.m._drone_diff_pos[drone_num].longitude,3), diff_lon_result[drone_num][test_case])
                self.assertEqual(self.m._drone_diff_pos[drone_num].altitude, diff_alt_result[drone_num][test_case])
                drone_num += 1

            test_case += 1

    def test_select_pivot(self):

        in_drone_diff_lat = [0] * INPORT_SET_NUM
        in_drone_diff_lon = [0] * INPORT_SET_NUM
        pivot_result = [0] * 2

        in_leader_diff_lat = [0, 0, 0]
        in_leader_diff_lon = [0, 0, 0]
        in_drone_diff_lat[0] = [0.00012, 0.00004, 0.00007]
        in_drone_diff_lat[1] = [-0.00003, -0.00002, -0.00007]
        in_drone_diff_lat[2] = [-0.00006, -0.00003, -0.00001]
        in_drone_diff_lon[0] = [0.00003, 0.00001, -0.00002]
        in_drone_diff_lon[1] = [0.00002, 0.00004, 0.00005]
        in_drone_diff_lon[2] = [-0.00006, -0.00008, -0.00003]

        pivot_result[0] = [0, 2, 1]
        pivot_result[1] = [2, 1, 0]

       

        test_case = 0
        while test_case < len(pivot_result[0]):

            self.m._leader_diff_pos.latitude = in_leader_diff_lat[test_case]
            self.m._leader_diff_pos.longitude = in_leader_diff_lon[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._drone_diff_pos[drone_num].latitude = in_drone_diff_lat[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].longitude = in_drone_diff_lon[drone_num][test_case]

                drone_num += 1

            self.m._select_pivot()

            self.assertEqual(self.m._pivot[0], pivot_result[0][test_case])
            self.assertEqual(self.m._pivot[1], pivot_result[1][test_case])

            test_case += 1

    def test_decide_left_right_pivot(self):

        in_pivot = [0] * 2
        in_drone_diff_lat = [0] * INPORT_SET_NUM
        in_drone_diff_lon = [0] * INPORT_SET_NUM

        in_pivot[0] = [0, 2, 1]
        in_pivot[1] = [2, 1, 0]
        in_leader_diff_lat = [0, 0, 0]
        in_leader_diff_lon = [0, 0, 0]
        in_drone_diff_lat[0] = [0.00012, 0.00004, 0.00007]
        in_drone_diff_lat[1] = [-0.00003, -0.00002, -0.00007]
        in_drone_diff_lat[2] = [-0.00006, -0.00003, -0.00001]
        in_drone_diff_lon[0] = [0.00003, 0.00001, -0.00002]
        in_drone_diff_lon[1] = [0.00002, 0.00004, 0.00005]
        in_drone_diff_lon[2] = [-0.00006, -0.00008, -0.00003]
        in_avg_mag = [150, 340, 240]

        right_pivot_result = [2, 1, 0]
        left_pivot_result = [0, 2, 1]
       

        test_case = 0
        while test_case < len(right_pivot_result):
            self.m._pivot[0] = in_pivot[0][test_case]
            self.m._pivot[1] = in_pivot[1][test_case]
            self.m._leader_diff_pos.latitude = in_leader_diff_lat[test_case]
            self.m._leader_diff_pos.longitude = in_leader_diff_lon[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._drone_diff_pos[drone_num].latitude = in_drone_diff_lat[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].longitude = in_drone_diff_lon[drone_num][test_case]

                drone_num += 1

            self.m._avg_mag = in_avg_mag[test_case]

            self.m._decide_left_right_pivot()

            self.assertEqual(self.m._right_pivot, right_pivot_result[test_case])
            self.assertEqual(self.m._left_pivot, left_pivot_result[test_case])

            test_case += 1


    def test_diff_target_angle(self):
        in_avg_mag = [30, 30, 280, 280]
        in_target_lat = [37.520, 37.580, 37.520, 37.580]
        in_target_lon = [139.910, 139.810, 139.910, 139.810]
        in_pos_lat = [37.550, 37.500, 37.550, 37.500]
        in_pos_lon = [139.940, 139.890, 139.940, 139.890]

        target = namedtuple("target", ["latitude", "longitude"])
        pos = namedtuple("pos", ["latitude", "longitude"])

        deg_result = [-165.0, -75.0, -55.0, 35.0]
       

        test_case = 0
        while test_case < len(deg_result):

            self.m._avg_mag = in_avg_mag[test_case]

            t = target(in_target_lat[test_case], in_target_lon[test_case])
            p = pos(in_pos_lat[test_case], in_pos_lon[test_case])
            deg = round(self.m._diff_target_angle(t, p), 1)

            self.assertEqual(deg, deg_result[test_case])

            test_case += 1
    
    
    def test_set_target_gps(self):
        xx = Mock()
        self.m._takeoff_landing = xx._takeoff_landing
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down
        self.m._turn_dir = xx._turn_dir

        in_state = [0] * INPORT_SET_NUM
        in_home_lat = [0] * INPORT_SET_NUM
        in_home_lon = [0] * INPORT_SET_NUM
        in_home_alt = [0] * INPORT_SET_NUM
        in_takeoff_alt = [0] * INPORT_SET_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_accel = [0] * INPORT_SET_NUM

        target_lat_result = [0] * INPORT_SET_NUM
        target_lon_result = [0] * INPORT_SET_NUM
        target_alt_result = [0] * INPORT_SET_NUM


        in_diff_lat = [0] * DRONE_NUM
        in_diff_lon = [0] * DRONE_NUM
        in_diff_alt = [0] * DRONE_NUM



        in_takeoff_landing = [1, -1, 0, 0, 0, 0, 0, 0, 0, 0,
                              0,  0, 0, 0, 0, 0, 0, 0]

        in_landing_flag = [False, True, False, False, False, False, False, False, False, False,
                           False, False, False, False, False, False, False, False]

        in_state[0] = ["Armed", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "RTL",
                       "RTL", "RTL", "Disarm", "Disarm", "Disarm", "Disarm", "RTL", "RTL",
                       "RTL", "Hovering"]
        in_state[1] = ["Armed", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                       "Hovering", "Hovering", "Hovering", "Hovering", "Disarm", "Disarm", "Disarm", "RTL",
                       "RTL", "Hovering"]
        in_state[2] = ["Armed", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                       "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                       "Hovering", "Hovering"]

        in_home_lat[0] = [37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 
                          37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500]
        in_home_lat[1] = [37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510,
                          37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510]
        in_home_lat[2] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520,
                          37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520]

        in_home_lon[0] = [139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900,
                          139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900]
        in_home_lon[1] = [139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910,
                          139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910] 
        in_home_lon[2] = [139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920,
                          139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920]
        in_home_alt[0] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_home_alt[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_home_alt[2] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]

        in_takeoff_alt[0] = [5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0,
                             5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0]    
        in_takeoff_alt[1] = [7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0,
                             7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0]
        in_takeoff_alt[2] = [9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                             9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0]

        in_leader_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 
                         37.530, 37.530, 37.530, 37.530, 37.530, 
                         37.530, 37.530, 37.530, 37.530, 37.530,
                         37.530]
        in_leader_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 
                         139.950, 139.950, 139.950, 139.950, 139.950, 
                         139.950, 139.950, 139.950, 139.950, 139.950,
                         139.950]
        in_leader_alt = [7.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0,
                         10.0, 10.0, 10.0, 10.0, 10.0,
                         10.0, 10.0, 10.0, 10.0, 10.0,
                         10.0]
 
        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520,
                           37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530,
                           37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540,
                           37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930,
                           139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940,
                           139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980,
                           139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980]
        in_drone_alt[0] = [0.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
                           8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0]
        in_drone_alt[1] = [0.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0,
                           10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0]
        in_drone_alt[2] = [0.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0,
                           12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0]

        in_front_back = [0, 0, 1, 0, 0, 0, 0, 1, 0, 0,
                         0, 0, 1, 0, 0, 0, 0, 0]
        in_right_left = [0, 0, 0, -1, 0, 0, 0, 0, -1, 0,
                         0, 0, 0, -1, 0, 0, 0, 0]
        in_up_down = [0, 0, 0, 0, 1, 0, 0, 0, 0, 1,
                      0, 0, 0, 0, 1, 0, 0, 0]

        in_turn_dir = [0, 0, 0, 0, 0, 1, -1, 0, 0, 0,
                       1, -1, 0, 0, 0, 1, -1, 0]

        #in_avg_mag = [350, 350, 350, 160, 160, 160]

        in_mode = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                   0, 0, 0, 0, 0, 0, 0, 0]
        in_accel[0] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_accel[1] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_accel[2] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]

        in_last_front_back = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                              0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_last_right_left = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                              0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_last_up_down = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]

        in_diff_lat[0] = [-0.01, -0.01, -0.01, -0.01, -0.01, -0.01, -0.01,
                           -0.01, -0.01, -0.01, -0.01, -0.01,
                           -0.01, -0.01, -0.01, -0.01, -0.01, 
                          -0.01 ]
        in_diff_lat[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0 ]
        in_diff_lat[2] = [0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
                          0.01, 0.01, 0.01, 0.01, 0.01,
                           0.01, 0.01, 0.01, 0.01, 0.01,
                          0.01]
        in_diff_lon[0] = [-0.02, -0.02, -0.02, -0.02, -0.02, -0.02, -0.02,
                           -0.02, -0.02, -0.02, -0.02, -0.02,
                           -0.02, -0.02, -0.02, -0.02, -0.02,
                          -0.02]
        in_diff_lon[1] = [-0.01, -0.01, -0.01, -0.01, -0.01, -0.01, -0.01,
                          -0.01, -0.01, -0.01, -0.01, -0.01,
                           -0.01, -0.01, -0.01, -0.01, -0.01,
                          -0.01 ]
        in_diff_lon[2] = [0.03, 0.03, 0.03, 0.03, 0.03, 0.03, 0.03,
                          0.03, 0.03, 0.03, 0.03, 0.03,
                           0.03, 0.03, 0.03, 0.03, 0.03,
                          0.03 ]
        in_diff_alt[0] = [-2.0, -2.0, -2.0, -2.0, -2.0, -2.0, -2.0, 
                           -2.0, -2.0, -2.0, -2.0, -2.0,
                           -2.0, -2.0, -2.0, -2.0, -2.0,
                          -2.0 ]
        in_diff_alt[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
                          0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0 ]
        in_diff_alt[2] = [2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 
                          2.0, 2.0, 2.0, 2.0, 2.0,
                           2.0, 2.0, 2.0, 2.0, 2.0,
                          2.0 ]
        """
        in_diff_lat[0] = [-0.01, -0.01, -0.01, -0.01, -0.01, -0.01, -0.01,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0, 
                          -0.01]
        in_diff_lat[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          -0.005, -0.005, -0.005, -0.005, -0.005,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0 ]
        in_diff_lat[2] = [0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
                          0.005, 0.005, 0.005, 0.005, 0.005,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          0.01]
        in_diff_lon[0] = [-0.02, -0.02, -0.02, -0.02, -0.02, -0.02, -0.02,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          -0.02]
        in_diff_lon[1] = [-0.01, -0.01, -0.01, -0.01, -0.01, -0.01, -0.01,
                          -0.02, -0.02, -0.02, -0.02, -0.02,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          -0.01]
        in_diff_lon[2] = [0.03,11 0.03, 0.03, 0.03, 0.03, 0.03, 0.03,
                          0.02,11 0.02, 0.02, 0.02, 0.02,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          0.03]
        in_diff_alt[0] = [-2.0, -2.0, -2.0, -2.0, -2.0, -2.0, -2.0, 
                           0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          -2.0 ]
        in_diff_alt[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
                          -1.0, -1.0, -1.0, -1.0, -1.0,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0 ]
        in_diff_alt[2] = [2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 
                          1.0, 1.0, 1.0, 1.0, 1.0,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                          2.0 ]
        """
        in_rtl_num = [0, 0, 0, 0, 0, 0, 0,
                      1, 1, 1, 1, 1,
                      2, 2, 2, 2, 2,
                      0]



        virtual_leader_pos_lat_result = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.535, 37.535, 37.535,
                                37.535, 37.535, 37.540, 37.540, 37.540, 37.540, 37.540, 37.530]

        virtual_leader_pos_lon_result = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.960, 139.960, 139.960,
                                139.960, 139.960, 139.980, 139.980, 139.980, 139.980, 139.980, 139.950]

        virtual_leader_pos_alt_result = [7.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 11.0, 11.0, 11.0,
                                11.0, 11.0, 12.0, 12.0, 12.0, 12.0, 12.0, 10.0]

        virtual_leader_target_lat_result = [37.530, 37.530, 37.53041, 37.530, 37.530, 37.53001, 37.53001, 37.53541, 37.535, 37.535,
                                37.53501, 37.53501, 37.54041, 37.540, 37.540, 37.53998, 37.54003, 37.530]

        virtual_leader_target_lon_result = [139.950, 139.950, 139.950, 139.94968, 139.950, 139.94999, 139.94999, 139.960, 139.95968, 139.960,
                                139.960, 139.95999, 139.980, 139.97968, 139.980, 139.980, 139.97998, 139.950]

        virtual_leader_target_alt_result = [7.0, 10.0, 10.0, 10.0, 11.0, 10.0, 10.0, 11.0, 11.0, 12.0,
                                11.0, 11.0, 12.0, 12.0, 13.0, 12.0, 12.0, 10.0]

        virtual_leader_mag_result = [350.0, 350.0, 350.0, 350.0, 350.0, 350.0, 350.0, 350.0, 350.0, 350.0,
                                350.0, 350.0, 350.0, 350.0, 350.0, 350.0, 350.0, 350.0]


        target_lat_result[0] = [37.500, 37.500, 37.52041, 37.520, 37.520, 37.52002, 37.520, 37.500, 37.500, 37.500,
                                37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.520]

        target_lat_result[1] = [37.510, 37.510, 37.53041, 37.530, 37.530, 37.53002, 37.530, 37.53041, 37.530, 37.530,
                                37.53002, 37.530, 37.510, 37.510, 37.510, 37.510, 37.510, 37.530]

        target_lat_result[2] = [37.520, 37.520, 37.54041, 37.540, 37.540, 37.540, 37.54002, 37.54041, 37.540, 37.540,
                                37.540, 37.54002, 37.54041, 37.540, 37.540, 37.53998, 37.54003, 37.540]

        target_lon_result[0] = [139.900, 139.900, 139.930, 139.92968, 139.930, 139.92999, 139.930, 139.900, 139.900, 139.900,
                                139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.930]

        target_lon_result[1] = [139.910, 139.910, 139.940, 139.93968, 139.940, 139.93999, 139.93999, 139.940, 139.93968, 139.940,
                                139.93999, 139.940, 139.910, 139.910, 139.910, 139.910, 139.910, 139.940]

        target_lon_result[2] = [139.920, 139.920, 139.980, 139.97968, 139.980, 139.980, 139.97999, 139.980, 139.97968, 139.980,
                                139.980, 139.97999, 139.980, 139.97968, 139.980, 139.980, 139.97998, 139.980]

        target_alt_result[0] = [5.0, 0.0, 8.0, 8.0, 9.0, 8.0, 8.0, 0.0, 0.0, 0.0,
                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 8.0]

        target_alt_result[1] = [7.0, 0.0, 10.0, 10.0, 11.0, 10.0, 10.0, 10.0, 10.0, 11.0,
                                10.0, 10.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.0]

        target_alt_result[2] = [9.0, 0.0, 12.0, 12.0, 13.0, 12.0, 12.0, 12.0, 12.0, 13.0,
                                12.0, 12.0, 12.0, 12.0, 13.0, 12.0, 12.0, 12.0]


        test_case = 0


        while test_case < len(target_lat_result[0]):
            xx._takeoff_landing.return_value = in_takeoff_landing[test_case]
            xx._front_back.return_value =in_front_back[test_case]
            xx._right_left.return_value =in_right_left[test_case]
            xx._up_down.return_value =in_up_down[test_case]
            xx._turn_dir.return_value = in_turn_dir[test_case]

            self.m._landing_flag = in_landing_flag[test_case]

            self.m._rtl_num = in_rtl_num[test_case]

            self.m._leader_set_first = True
            #self.m._leader_home.latitude = in_leader_lat[0]
            #self.m._leader_home.longitude = in_leader_lon[0]
            #self.m._leader_home.altitude = in_leader_alt[0]
            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]
            self.m._leader_pos.altitude = in_leader_alt[test_case]
            self.m._leader_target.latitude = in_leader_lat[test_case]
            self.m._leader_target.longitude = in_leader_lon[test_case]
            self.m._leader_target.altitude = in_leader_alt[test_case]
            self.m._avg_drone_pos.altitude = in_leader_alt[test_case]

            self.m._mode = in_mode[test_case]

            self.m._last_front_back = in_last_front_back[test_case]
            self.m._last_right_left = in_last_right_left[test_case]
            self.m._last_up_down = in_last_up_down[test_case]
            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].homeGPS.latitude = in_home_lat[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].homeGPS.longitude = in_home_lon[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].homeGPS.altitude = in_home_alt[drone_num][test_case]
                self.m._takeoff_alt[drone_num] = in_takeoff_alt[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.latitude = in_drone_lat[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.longitude = in_drone_lon[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.altitude = in_drone_alt[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].mag.data = 350

                self.m._drone_diff_pos[drone_num].latitude = in_diff_lat[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].longitude = in_diff_lon[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].altitude = in_diff_alt[drone_num][test_case]

                #target_gps.py[get_target_cal] if drone_target.altitude > 0.0を満たすため
                self.m._target[drone_num].altitude = 1.0
                self.m._turn_first = True
                self.m._avg_mag = 350
                self.m._d_inDrCtrl[drone_num].accel.data = in_accel[drone_num][test_case]

                drone_num += 1

            self.m._home = self.m._is_drone_on_home()
            self.m._set_target_gps()

            self.assertEqual(round(self.m._d_outVirtualLeader.droneGPS.latitude,5),
                             virtual_leader_pos_lat_result[test_case])
            self.assertEqual(round(self.m._d_outVirtualLeader.droneGPS.longitude,5),
                             virtual_leader_pos_lon_result[test_case])
            self.assertEqual(self.m._d_outVirtualLeader.droneGPS.altitude,
                             virtual_leader_pos_alt_result[test_case])
            self.assertEqual(round(self.m._d_outVirtualLeader.targetGPS.latitude,5),
                             virtual_leader_target_lat_result[test_case])
            self.assertEqual(round(self.m._d_outVirtualLeader.targetGPS.longitude,5),
                             virtual_leader_target_lon_result[test_case])
            self.assertEqual(self.m._d_outVirtualLeader.targetGPS.altitude,
                             virtual_leader_target_alt_result[test_case])
            self.assertEqual(self.m._d_outVirtualLeader.mag.data,
                             virtual_leader_mag_result[test_case])

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.assertEqual(round(self.m._d_outTarget[drone_num].pos.latitude,5), target_lat_result[drone_num][test_case])
                self.assertEqual(round(self.m._d_outTarget[drone_num].pos.longitude,5), target_lon_result[drone_num][test_case])
                self.assertEqual(self.m._d_outTarget[drone_num].pos.altitude, target_alt_result[drone_num][test_case])
                drone_num += 1

            test_case += 1

    def test_set_leader_alt(self):

        in_state = [0] * INPORT_SET_NUM
        in_diff_drone_alt = [0] * INPORT_SET_NUM

        in_up_down = [0,
                      1, 1, 1, 1, 1, 1, 1,
                      1, 1, 1, 1, 1, 1, 1,
                      -1, -1, -1, -1, -1, -1, -1,
                      -1, -1, -1, -1, -1, -1, -1,
                      1, 1, 1, 1, 1, 1, 1,
                      1, 1, 1, 1, 1, 1, 1,
                      -1, -1, -1, -1, -1, -1, -1,
                      -1, -1, -1, -1, -1, -1, -1,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,

                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0]
        in_state[0] = ["Armed",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Yaw_CHG", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Yaw_CHG", "Disarm", "RTL",
                       "Alt_CHG", "Alt_CHG", "Alt_CHG", "RTL", "Alt_CHG", "Disarm", "RTL",
                       "Alt_CHG", "Alt_CHG", "Alt_CHG", "RTL", "Alt_CHG", "Disarm", "RTL",
                       "Alt_CHG", "Alt_CHG", "Alt_CHG", "RTL", "Alt_CHG", "Disarm", "RTL",
                       "Alt_CHG", "Alt_CHG", "Alt_CHG", "RTL", "Alt_CHG", "Disarm", "RTL",
                       "Alt_CHG", "Alt_CHG", "Alt_CHG", "RTL", "Alt_CHG", "Disarm", "RTL",
                       "Alt_CHG", "Alt_CHG", "Alt_CHG", "RTL", "Alt_CHG", "Disarm", "RTL",

                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL",
                       "Armed", "Takeoff", "In_Move", "RTL", "Hovering", "Disarm", "RTL"]
        in_state[1] = ["Armed",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Yaw_CHG", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Yaw_CHG", "RTL", "In_Move", "RTL",
                       "Alt_CHG", "Alt_CHG", "Disarm", "Alt_CHG", "RTL", "Alt_CHG", "RTL",
                       "Alt_CHG", "Alt_CHG", "Disarm", "Alt_CHG", "RTL", "Alt_CHG", "RTL",
                       "Alt_CHG", "Alt_CHG", "Disarm", "Alt_CHG", "RTL", "Alt_CHG", "RTL",
                       "Alt_CHG", "Alt_CHG", "Disarm", "Alt_CHG", "RTL", "Alt_CHG", "RTL",
                       "Alt_CHG", "Alt_CHG", "Disarm", "Alt_CHG", "RTL", "Alt_CHG", "RTL",
                       "Alt_CHG", "Alt_CHG", "Disarm", "Alt_CHG", "RTL", "Alt_CHG", "RTL",

                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL",
                       "Armed", "Takeoff", "Disarm", "Hovering", "RTL", "In_Move", "RTL"]
        in_state[2] = ["Armed",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Yaw_CHG", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Yaw_CHG", "Disarm", "RTL", "Takeoff",
                       "Alt_CHG", "RTL", "Alt_CHG", "Alt_CHG", "Disarm", "RTL", "Alt_CHG",
                       "Alt_CHG", "RTL", "Alt_CHG", "Alt_CHG", "Disarm", "RTL", "Alt_CHG",
                       "Alt_CHG", "RTL", "Alt_CHG", "Alt_CHG", "Disarm", "RTL", "Alt_CHG",
                       "Alt_CHG", "RTL", "Alt_CHG", "Alt_CHG", "Disarm", "RTL", "Alt_CHG",
                       "Alt_CHG", "RTL", "Alt_CHG", "Alt_CHG", "Disarm", "RTL", "Alt_CHG",
                       "Alt_CHG", "RTL", "Alt_CHG", "Alt_CHG", "Disarm", "RTL", "Alt_CHG",

                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff",
                       "Armed", "RTL", "In_Move", "Hovering", "Disarm", "RTL", "Takeoff"]
        in_diff_drone_alt[0] = 2.0
        in_diff_drone_alt[1] = 0
        in_diff_drone_alt[2] = -2.0

        in_rtl_num = [0,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,

                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2,
                      0, 1, 1, 1, 2, 2, 2]

        in_avg_drone_alt = [10.0,
                            10.0, 11.0, 10.0, 9.0, 12.0, 10.0, 8.0,
                            11.0, 12.0, 11.0, 10.0, 13.0, 11.0, 9.0,
                            10.0, 11.0, 10.0, 9.0, 12.0, 10.0, 8.0,
                            9.0, 10.0, 9.0, 8.0, 11.0, 9.0, 7.0,
                            10.0, 11.0, 10.0, 9.0, 12.0, 10.0, 8.0,
                            11.0, 12.0, 11.0, 10.0, 13.0, 11.0, 9.0,
                            10.0, 11.0, 10.0, 9.0, 12.0, 10.0, 8.0,
                            9.0, 10.0, 9.0, 8.0, 11.0, 9.0, 7.0,
                            10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5,
                            9.5, 9.5, 9.5, 9.5, 9.5, 9.5, 9.5,

                            10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                            9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                            10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                            9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                            10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                            9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                            10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                            9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                            10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                            9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                            10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                            9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                            10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                            9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3]
        in_leader_pos_alt = [10.0]
        in_leader_target_alt = [11.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,

                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0]
        in_last_up_down = [0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           1, 1, 1, 1, 1, 1, 1,
                           1, 1, 1, 1, 1, 1, 1,
                           -1, -1, -1, -1, -1, -1, -1,
                           -1, -1, -1, -1, -1, -1, -1,
                           1, 1, 1, 1, 1, 1, 1,
                           -1, -1, -1, -1, -1, -1, -1,

                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0,
                           0, 0, 0, 0, 0 ,0, 0]
        in_alt_adjust_cnt = [30,
                             30, 30, 30 ,30, 30, 30, 30,
                             30, 30, 30 ,30, 30, 30, 30,
                             30, 30, 30 ,30, 30, 30, 30,
                             30, 30, 30 ,30, 30, 30, 30,
                             30, 30, 30 ,30, 30, 30, 30,
                             30, 30, 30 ,30, 30, 30, 30,
                             30, 30, 30 ,30, 30, 30, 30,
                             30, 30, 30 ,30, 30, 30, 30,
                             30, 30, 30 ,30, 30, 30, 30,
                             30, 30, 30 ,30, 30, 30, 30,

                             -1, -1, -1, -1, -1, -1, -1,
                             -1, -1, -1, -1, -1, -1, -1,
                             0, 0, 0, 0, 0, 0, 0,
                             0, 0, 0, 0, 0, 0, 0,
                             1, 1, 1, 1, 1, 1, 1,
                             1, 1, 1, 1, 1, 1, 1,
                             10, 10, 10, 10, 10, 10, 10,
                             10, 10, 10, 10, 10, 10, 10,
                             29, 29, 29, 29, 29, 29, 29,
                             29, 29, 29, 29, 29, 29, 29,
                             30, 30, 30, 30, 30, 30, 30,
                             30, 30, 30, 30, 30, 30, 30,
                             31, 31, 31, 31, 31, 31, 31,
                             31, 31, 31, 31, 31, 31, 31,]
        in_execute_rate = 10.0


        leader_target_alt_result = [11.0,
                                    10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0,
                                    11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                    10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0,
                                    9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                    10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0,
                                    11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                    10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0,
                                    9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                    10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5,
                                    9.5, 9.5, 9.5, 9.5, 9.5, 9.5, 9.5,

                                    11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                    9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                                    10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                                    9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                                    10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                                    9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                                    10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                                    9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                                    10.7, 10.7, 10.7, 10.7, 10.7, 10.7, 10.7,
                                    9.3, 9.3, 9.3, 9.3, 9.3, 9.3, 9.3,
                                    11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                    9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,

                                    11.0, 11.0, 11.0, 11.0, 11.0, 11.0, 11.0,
                                    9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0]


        self.m._leader_pos.altitude = in_leader_pos_alt[0]
        self.m._execute_rate = in_execute_rate
        for drone_num in range(DRONE_NUM):
            self.m._drone_diff_pos[drone_num].altitude = in_diff_drone_alt[drone_num]

        for test_case in range(len(leader_target_alt_result)):
            self.m._avg_drone_pos.altitude = in_avg_drone_alt[test_case]
            self.m._leader_target.altitude = in_leader_target_alt[test_case]

            self.m._last_up_down = in_last_up_down[test_case]
            self.m._alt_adjust_cnt = in_alt_adjust_cnt[test_case]

            self.m._rtl_num = in_rtl_num[test_case]

            for drone_num in range(DRONE_NUM):
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]

            self.m._set_leader_alt(in_up_down[test_case])

            self.assertEqual(self.m._leader_target.altitude, leader_target_alt_result[test_case])
        

    def test_trans_target_speed(self):

        in_state = [0] * INPORT_SET_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_target_lat = [0] * INPORT_SET_NUM
        in_target_lon = [0] * INPORT_SET_NUM
        in_target_alt = [0] * INPORT_SET_NUM

        target_speed_result = [0] * INPORT_SET_NUM


        in_speed = [3.0, 3.0, 3.0, 3.0]
        in_leader_lat = [37.530, 37.530, 37.530, 37.530]
        in_leader_lon = [139.940, 139.940, 139.940, 139.940]
        in_leader_target_lat = [37.535, 37.535, 37.535, 37.535]
        in_leader_target_lon = [139.950, 139.950, 139.950, 139.950]
        in_state[0] = ["Hovering", "Disarm", "RTL", "RTL"]
        in_state[1] = ["Hovering", "Hovering", "RTL", "Disarm"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering"]
        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.930, 139.930, 139.930, 139.930]
        in_drone_lon[2] = [139.960, 139.960, 139.960, 139.960]
        in_target_lat[0] = [37.525, 37.525, 37.525, 37.525]
        in_target_lat[1] = [37.534, 37.534, 37.534, 37.534]
        in_target_lat[2] = [37.547, 37.547, 37.547, 37.547]
        in_target_lon[0] = [139.940, 139.940, 139.940, 139.940]
        in_target_lon[1] = [139.940, 139.940, 139.940, 139.940]
        in_target_lon[2] = [139.969, 139.969, 139.969, 139.969]

        

        target_speed_result[0] = [3.0, 3.0, 3.0, 3.0]
        target_speed_result[1] = [2.843, 2.843, 2.843, 2.843]
        target_speed_result[2] = [3.196, 3.196, 3.196, 3.196]



        test_case = 0
        while test_case < len(target_speed_result[0]):
            self.m._SpeedParam = in_speed[test_case]
            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]
            self.m._leader_target.latitude = in_leader_target_lat[test_case]
            self.m._leader_target.longitude = in_leader_target_lon[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.latitude = in_drone_lat[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.longitude = in_drone_lon[drone_num][test_case]
                self.m._d_outTarget[drone_num].pos.latitude = in_target_lat[drone_num][test_case]
                self.m._d_outTarget[drone_num].pos.longitude = in_target_lon[drone_num][test_case]
                drone_num += 1


            self.m._trans_target_speed()

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.assertEqual(round(self.m._d_outTarget[drone_num].speed.data,3), target_speed_result[drone_num][test_case])
                drone_num += 1

            test_case += 1

    def test_turn_target_speed(self):

        init_target_speed = [0] * INPORT_SET_NUM

        in_drone_diff_lat = [0] * INPORT_SET_NUM
        in_drone_diff_lon = [0] * INPORT_SET_NUM
        in_drone_diff_alt = [0] * INPORT_SET_NUM
        in_state = [0] * INPORT_SET_NUM

        target_speed_result = [0] * INPORT_SET_NUM


        init_target_speed[0] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        init_target_speed[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        init_target_speed[2] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]


        in_right_pivot = [0, 1, 2, 0, 0, 0]
        in_left_pivot = [2, 2, 2, 2, 1, 0]
        in_drone_diff_lat[0] = [-0.00001, -0.00001, -0.00001, -0.00001, -0.00001, -0.00001]
        in_drone_diff_lat[1] = [0.00002, 0.00002, 0.00002, 0.00002, 0.00002, 0.00002]
        in_drone_diff_lat[2] = [-0.00003, -0.00003, -0.00003, -0.00003, -0.00003, -0.00003]
        in_drone_diff_lon[0] = [-0.00004, -0.00004, -0.00004, -0.00004, -0.00004, -0.00004]
        in_drone_diff_lon[1] = [0.00003, 0.00003, 0.00003, 0.00003, 0.00003, 0.00003]
        in_drone_diff_lon[2] = [0.00002, 0.00002, 0.00002, 0.00002, 0.00002, 0.00002]
        in_turn_pivot = [2, 2, 2, 0, 0, 0]
        in_speed = [3.0, 3.0, 3.0, 3.0, 3.0, 3.0]
        in_state[0] = ["Hovering", "Disarm", "RTL", "Hovering", "Hovering", "Hovering"]
        in_state[1] = ["Hovering", "Hovering", "RTL", "Hovering", "Hovering", "Disarm"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Disarm", "RTL"]


        target_speed_result[0] = [3.0,    0.0, 0.0,    0.0, 0.0, 0.0]
        target_speed_result[1] = [2.4047, 3.0, 0.0, 3.6111, 3.0, 0.0]
        target_speed_result[2] = [0.0,    0.0, 0.0,    3.0, 0.0, 0.0]


        test_case = 0
        while test_case < len(target_speed_result[0]):
            self.m._turn_pivot = in_turn_pivot[test_case]
            self.m._right_pivot = in_right_pivot[test_case]
            self.m._left_pivot = in_left_pivot[test_case]
            self.m._SpeedParam = in_speed[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._drone_diff_pos[drone_num].latitude = in_drone_diff_lat[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].longitude = in_drone_diff_lon[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]

                self.m._d_outTarget[drone_num].speed.data = init_target_speed[drone_num][test_case]

                drone_num += 1

            self.m._turn_target_speed()

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.assertEqual(round(self.m._d_outTarget[drone_num].speed.data,4), target_speed_result[drone_num][test_case])
                drone_num += 1

            test_case += 1


    def test_set_target_yaw(self):
        xx = Mock()
        self.m._turn_dir = xx._turn_dir

        init_target_yaw = [0] * INPORT_SET_NUM

        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_target_lat = [0] * INPORT_SET_NUM
        in_target_lon = [0] * INPORT_SET_NUM
        in_target_alt = [0] * INPORT_SET_NUM
        in_state = [0] * INPORT_SET_NUM

        target_yaw_result = [0] * INPORT_SET_NUM


        init_target_yaw[0] = [[0, 315], [0, 315], [0, 315], [0, 315], [0, 135], [0, 135], [0, 135], [0, 135], 
                      [0, 315], [0, 135], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315],
                      [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135]]
        init_target_yaw[1] = [[0, 315], [0, 315], [0, 315], [0, 315], [0, 135], [0, 135], [0, 135], [0, 135], 
                      [0, 315], [0, 135], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315],
                      [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135]]
        init_target_yaw[2] = [[0, 315], [0, 315], [0, 315], [0, 315], [0, 135], [0, 135], [0, 135], [0, 135], 
                      [0, 315], [0, 135], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315],
                      [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135]]

        in_turn_dir = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
                       -1, -1, -1, 1, 1, 1, -1, -1, -1, 1, 1, 1,
                       0, 0]

        in_mode = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
                   0, 0]

        in_move_mode = ["Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Relative", "Relative",
                        "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", 
                        "Relative", "Relative", "Relative", "Relative", "Relative", "Relative",
                        "Absolute", "Relative"]



        in_leader_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530,
                         37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530,
                         37.530, 37.530, 37.530, 37.530]
        in_leader_lon = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940,
                         139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940,
                         139.940, 139.940, 139.940, 139.940]
        in_leader_target_lat = [37.535, 37.525, 37.530, 37.530, 37.535, 37.525, 37.530, 37.530, 37.535, 37.530,
                                37.513, 37.532, 37.527, 37.513, 37.532, 37.527, 37.513, 37.532, 37.527, 37.513,
                                37.532, 37.527, 37.530, 37.530]
        in_leader_target_lon = [139.940, 139.940, 139.945, 139.935, 139.940, 139.940, 139.945, 139.935, 139.940, 139.945,
                                139.942, 139.937, 139.932, 139.942, 139.937, 139.932, 139.942, 139.937, 139.932, 139.942,
                                139.937, 139.932, 139.945, 139.935]

        in_right_pivot = [2, 2, 2, 2, 0, 0, 0, 0, 2, 0, 
                          2, 2, 2, 2, 2, 2, 0, 1, 2, 0, 1, 2, 2, 0]
        in_left_pivot = [0, 0, 0, 0, 2, 2, 2, 2, 0, 2,
                         0, 1, 2, 0, 1, 2, 2, 2, 2, 2, 2, 2, 0, 2]

        in_turn_end = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                         2, 2, 2, 0, 1, 2, 0, 1, 2, 2, 2, 2, 0, 0]

        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520,
                           37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530,
                           37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540,
                           37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930,
                           139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940,
                           139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950,
                           139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950]


        in_target_lat[0] = [37.525, 37.515, 37.520, 37.520, 37.525, 37.515, 37.520, 37.520, 37.520, 37.520,
                            37.520, 37.520, 37.520, 37.524, 37.524, 37.524, 37.518, 37.518, 37.518, 37.520, 37.520, 37.520,
                            37.520, 37.520]
        in_target_lat[1] = [37.535, 37.525, 37.530, 37.530, 37.535, 37.525, 37.530, 37.530, 37.530, 37.530,
                            37.531, 37.531, 37.531, 37.532, 37.532, 37.532, 37.529, 37.529, 37.529, 37.528, 37.528, 37.528,
                            37.530, 37.530]
        in_target_lat[2] = [37.545, 37.535, 37.540, 37.540, 37.545, 37.535, 37.540, 37.540, 37.540, 37.540,
                            37.542, 37.542, 37.542, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.536, 37.536, 37.536,
                            37.540, 37.540]
        in_target_lon[0] = [139.930, 139.930, 139.935, 139.925, 139.930, 139.930, 139.935, 139.925, 139.925, 139.925,
                            139.930, 139.930, 139.930, 139.928, 139.928, 139.928, 139.934, 139.934, 139.934, 139.930, 139.930, 139.930,
                            139.930, 139.930]
        in_target_lon[1] = [139.940, 139.940, 139.945, 139.935, 139.940, 139.940, 139.945, 139.935, 139.935, 139.935,
                            139.938, 139.938, 139.938, 139.939, 139.939, 139.939, 139.942, 139.942, 139.942, 139.941, 139.941, 139.941,
                            139.940, 139.940]
        in_target_lon[2] = [139.950, 139.950, 139.955, 139.945, 139.950, 139.950, 139.955, 139.945, 139.945, 139.945,
                            139.946, 139.946, 139.946, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.952, 139.952, 139.952,
                            139.950, 139.950]

        in_avg_mag = [315, 315, 315, 315, 135, 135, 135, 135, 
                      315, 135, 315, 315, 315, 315, 315, 315,
                      135, 135, 135, 135, 135, 135, 315, 135]

        in_move_mag = [310, 310, 310, 310, 130, 130, 130, 130, 
                      310, 130, 310, 310, 310, 310, 310, 310,
                      130, 130, 130, 130, 130, 130, 310, 130]

        in_state[0] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                         "Hovering", "RTL", "Disarm", "Hovering", "Hovering", "Hovering", "Hovering",  "Disarm", "RTL", "Hovering", "Hovering",
                         "Hovering", "Hovering", "Hovering", "Hovering"]
        in_state[1] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                         "Hovering", "Hovering", "Disarm", "Hovering", "Hovering", "RTL", "Hovering", "Hovering", "Disarm", "Hovering",
                         "Hovering", "RTL", "Hovering", "Hovering"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                         "Hovering", "Hovering", "Hovering", "Hovering", "RTL", "RTL", "Hovering", "Hovering", "Hovering", "Hovering",
                         "Disarm", "Disarm","Hovering", "Hovering"]


        target_yaw_result[0] = [[ 1, 0],   [-1, 180], [ 1, 90],  [-1, 270], [-1, 0],   [1, 180], [-1, 90],  [ 1, 270], [ 1, 310], [1, 130],
                                [-1, 173], [ 0, 315], [ 0, 315], [ 1, 173], [ 1, 303], [1, 0],   [-1, 173], [ 0, 135], [ 0, 135],  [1, 173], 
                                [ 1, 303], [ 1, 180], [ 0, 0], [ 0, 0]]
        target_yaw_result[1] = [[ 1, 0],   [-1, 180], [ 1, 90],  [-1, 270], [-1, 0],   [1, 180], [-1, 90],  [ 1, 270], [ 1, 310], [1, 130],
                                [-1, 173], [-1, 303], [ 0, 315], [ 1, 173], [ 1, 303], [0, 315],   [-1, 173], [-1, 303], [0, 135],  [1, 173], 
                                [ 1, 303], [ 0, 135], [ 0, 0], [ 0, 0]]
        target_yaw_result[2] = [[ 1, 0],   [-1, 180], [ 1, 90],  [-1, 270], [-1, 0],   [1, 180], [-1, 90],  [ 1, 270], [ 1, 310], [1, 130],
                                [-1, 173], [-1, 303], [-1, 270], [ 1, 173], [ 0, 315], [0, 315],   [-1, 173], [-1, 303], [-1, 90],  [1, 173], 
                                [ 0, 135], [ 0, 135], [ 0, 0], [ 0, 0]]

        test_case = 0
        while test_case < len(target_yaw_result[0]):

            xx._turn_dir.return_value = in_turn_dir[test_case]
            self.m._mode = in_mode[test_case]
            self.m._d_outMoveMode.data = in_move_mode[test_case]
            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]
            self.m._leader_target.latitude = in_leader_target_lat[test_case]
            self.m._leader_target.longitude = in_leader_target_lon[test_case]
            self.m._right_pivot = in_right_pivot[test_case]
            self.m._left_pivot = in_left_pivot[test_case]
            self.m._turn_end = in_turn_end[test_case]
            self.m._avg_mag = in_avg_mag[test_case]
            self.m._move_mag = in_move_mag[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inDrCtrl[drone_num].droneGPS.latitude = in_drone_lat[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].droneGPS.longitude = in_drone_lon[drone_num][test_case]
                self.m._d_outTarget[drone_num].pos.latitude = in_target_lat[drone_num][test_case]
                self.m._d_outTarget[drone_num].pos.longitude = in_target_lon[drone_num][test_case]
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]

                self.m._d_outTarget[drone_num].yaw.data = init_target_yaw[drone_num][test_case]
                drone_num += 1

            self.m._set_target_yaw()

            drone_num = 0
            while drone_num < DRONE_NUM:
                if self.m._d_outTarget[drone_num].yaw.data[1] == 360:
                    self.m._d_outTarget[drone_num].yaw.data[1] = 0
                self.assertEqual(self.m._d_outTarget[drone_num].yaw.data, target_yaw_result[drone_num][test_case])
                drone_num += 1

            test_case += 1

    def test_is_drone_on_home(self):

        test_case = 0

        in_home_lat = [0] * INPORT_SET_NUM
        in_home_lon = [0] * INPORT_SET_NUM
        in_home_alt = [0] * INPORT_SET_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM

        in_home_lat[0] = 37.500000
        in_home_lat[1] = 37.510000
        in_home_lat[2] = 37.520000
        in_home_lon[0] = 139.900000
        in_home_lon[1] = 139.910000
        in_home_lon[2] = 139.920000
        in_home_alt[0] = 0.0
        in_home_alt[1] = 0.0
        in_home_alt[2] = 0.0

        in_drone_lat[0] = [37.500010, 37.500002, 37.500004, 37.500005, 37.500007, 37.500016]
        in_drone_lat[1] = [37.510040, 37.510020, 37.510006, 37.510008, 37.510005, 37.510004]
        in_drone_lat[2] = [37.520005, 37.520050, 37.520020, 37.520010, 37.520003, 37.520006]
        in_drone_lon[0] = [139.900020, 139.900008, 139.900006, 139.900010, 139.900005, 139.900009]
        in_drone_lon[1] = [139.910050, 139.910030, 139.910007, 139.910010, 139.910011, 139.910008]
        in_drone_lon[2] = [139.920005, 139.920010, 139.920020, 139.920006, 139.920021, 139.920014]
        in_drone_alt[0] = [0.0, 8.0]
        in_drone_alt[1] = [0.0, 10.0]
        in_drone_alt[2] = [0.0, 12.0]

        in_landing_flag = [False, True]

        result = [False, False, True, True, True, True,
                  False, False, False, True, True, True,
                  False, False, False, False, False, False,
                  False, False, False, True, True, True]


        for drone_num in range(DRONE_NUM):
            self.m._d_inDrCtrl[drone_num].homeGPS.latitude = in_home_lat[drone_num]
            self.m._d_inDrCtrl[drone_num].homeGPS.longitude = in_home_lon[drone_num]
            self.m._d_inDrCtrl[drone_num].homeGPS.altitude = in_home_alt[drone_num]
        for alt_case in range(len(in_drone_alt[0])):
            for landing_case in range(len(in_landing_flag)):
                for lat_lon_case in range(len(in_drone_lat[0])):
                    for drone_num in range(DRONE_NUM):
                        self.m._d_inDrCtrl[drone_num].droneGPS.latitude = in_drone_lat[drone_num][lat_lon_case]
                        self.m._d_inDrCtrl[drone_num].droneGPS.longitude = in_drone_lon[drone_num][lat_lon_case]
                        self.m._d_inDrCtrl[drone_num].droneGPS.altitude = in_drone_alt[drone_num][alt_case]

                    self.m._landing_flag = in_landing_flag[landing_case]

                    self.m._is_drone_on_home()

                    self.assertEqual(self.m._home, result[test_case])
                    test_case += 1

    def test_takeoff_landing(self):
        test_case = 0

        in_home = [False, True]
        in_button = [0, DroneFlightDirector.BUTTON_06_MASK, DroneFlightDirector.BUTTON_08_MASK,
                     DroneFlightDirector.BUTTON_06_MASK | DroneFlightDirector.BUTTON_08_MASK]

        result = [0, 0, -1, -1, 0, 1, 0, 1]

        for home_case in range(len(in_home)):
            self.m._home = in_home[home_case]
            for button_case in range(len(in_button)):
                self.m._d_ButtonIn.data = in_button[button_case]
                self.assertEqual(self.m._takeoff_landing(), result[test_case])
                test_case += 1

    def test_move_mode(self):
        in_target_mode = [1, 1, 2, 2]
        in_button = [0, DroneFlightDirector.BUTTON_10_MASK, 0, DroneFlightDirector.BUTTON_10_MASK]

        result = [1, 2, 2, 1]

        test_case = 0
        while test_case < len(result):
            self.m._target_gps._mode = in_target_mode[test_case]
            self.m._d_ButtonIn.data = in_button[test_case]

            mode = self.m._move_mode()

            self.assertEqual(mode, result[test_case])

            test_case += 1

    def test_moving(self):
        xx = Mock()

        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down

        in_home = [False, True]      

        in_front_back = [-1, 0, 1]
        in_right_left = [-1, 0, 1]
        in_up_down = [-1, 0, 1]

        front_back_result = [[[[-1, -1, -1],[-1, -1, -1],[-1, -1, -1]],
                            [[0, 0, 0],[0, 0, 0],[0, 0, 0]],
                            [[1, 1, 1],[1, 1, 1],[1, 1, 1]]],

                            [[[0, 0, 0],[0, 0, 0],[0, 0, 0]],
                            [[0, 0, 0],[0, 0, 0],[0, 0, 0]],
                            [[0, 0, 0],[0, 0, 0],[0, 0, 0]]]]

        right_left_result = [[[[-1, -1, -1],[0, 0, 0],[1, 1, 1]],
                            [[-1, -1, -1],[0, 0, 0],[1, 1, 1]],
                            [[-1, -1, -1],[0, 0, 0],[1, 1, 1]]],

                            [[[0, 0, 0],[0, 0, 0],[0, 0, 0]],
                            [[0, 0, 0],[0, 0, 0],[0, 0, 0]],
                            [[0, 0, 0],[0, 0, 0],[0, 0, 0]]]]

        up_down_result = [[[[-1, 0, 1],[-1, 0, 1],[-1, 0, 1]],
                         [[-1, 0, 1],[-1, 0, 1],[-1, 0, 1]],
                         [[-1, 0, 1],[-1, 0, 1],[-1, 0, 1]]],

                         [[[0, 0, 0],[0, 0, 0],[0, 0, 0]],
                         [[0, 0, 0],[0, 0, 0],[0, 0, 0]],
                         [[0, 0, 0],[0, 0, 0],[0, 0, 0]]]]

        for i1, d1 in enumerate(in_home):
            self.m._home = in_home[i1]     
            for i2, d2 in enumerate(in_front_back):
                xx._front_back.return_value = in_front_back[i2]
                for i3, d3 in enumerate(in_right_left):
                    xx._right_left.return_value = in_right_left[i3]
                    for i4, d4 in enumerate(in_up_down):
                        xx._up_down.return_value = in_up_down[i4]

                        mv = self.m._moving()
                        self.assertEqual(mv.front_back, front_back_result[i1][i2][i3][i4])
                        self.assertEqual(mv.right_left, right_left_result[i1][i2][i3][i4])
                        self.assertEqual(mv.up_down, up_down_result[i1][i2][i3][i4])

    def test_move_continue(self):
        xx = Mock()
        in_last_move_mode = ["Absolute", "Relative", "Absolute",
                             "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute"]
        in_move_mode = ["Relative", "Absolute", "Absolute",
                        "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute"]

        in_last_front_back = [0, 1, 0, 0, -1, 0, 0, 1, -1]
        in_front_back = [0, 1, 0, 0, 0, 1, 0, 1, -1]

        in_last_right_left = [0, 0, 0, 0, 0, 1, -1, 1, 1]
        in_right_left = [0, 0, 0, 0, 1, 1, -1, 1, 1]

        in_last_up_down = [0, 0, 0, -1, 0, 0, 0, 0, 1]
        in_up_down = [0, 0, 0, 0, 0, 0, 0, 0, 1]



        result = [False, False, False, False, False, False, True, True, True]


        test_case = 0
        while test_case < len(result):
            self.m._last_move_mode = in_last_move_mode[test_case]
            self.m._d_outMoveMode.data = in_move_mode[test_case]
            self.m._last_front_back = in_last_front_back[test_case]
            self.m._last_right_left = in_last_right_left[test_case]
            self.m._last_up_down = in_last_up_down[test_case]

            moved_countinue = self.m._move_continue(in_front_back[test_case], in_right_left[test_case], in_up_down[test_case])

            self.assertEqual(moved_countinue, result[test_case])

            test_case += 1

    def test_move_speed(self):
        in_home = [False, True] 
        in_button = [0,
                     DroneFlightDirector.BUTTON_05_MASK,
                     DroneFlightDirector.BUTTON_07_MASK,
                     DroneFlightDirector.BUTTON_05_MASK | DroneFlightDirector.BUTTON_07_MASK
                     ]

        in_speed = [1.0, 2.0, 5.0,  9.0, 10.0]
        result = [[[1.0, 2.0, 5.0,  9.0, 10.0],
                   [2.0, 3.0, 6.0, 10.0, 10.0],
                   [1.0, 1.0, 4.0,  8.0,  9.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0]],

                   [[1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0]]
                   ]

        for i1, d1 in enumerate(in_home):
            self.m._home = in_home[i1]     
            for i2, d2 in enumerate(in_button):
                self.m._d_ButtonIn.data = in_button[i2]
                for i3, d3 in enumerate(in_speed):
                    self.m._SpeedParam = in_speed[i3]

                    self.m._move_speed()
                    self.assertEqual(self.m._SpeedParam, result[i1][i2][i3])

    def test_is_landing(self):

        in_state = [0] * INPORT_SET_NUM

        in_state[0] = ["RTL", "RTL", "RTL", "RTL", "Hovering", "Hovering", "Disarm", "Armd"]
        in_state[1] = ["RTL", "RTL", "Hovering", "Hovering", "RTL", "RTL", "Disarm", "Armd"]
        in_state[2] = ["RTL", "Hovering", "RTL", "Hovering", "RTL", "Disarm", "RTL", "Disarm"]


        landing_result = [True, True, True, True, True, True, True, False]


        test_case = 0
        while test_case < len(landing_result):

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inDrCtrl[drone_num].state.data = in_state[drone_num][test_case]
                drone_num += 1

            

            self.assertEqual(self.m._is_landing(), landing_result[test_case])

            test_case += 1

    def test_takeoff_range_check(self):
        in_takeoff_alt = [0] * INPORT_SET_NUM


        in_takeoff_alt[0] = [ 1.0,  1.0,  1.0,  2.0,  3.0, 14.0, 15.0, 16.0, 16.0, 16.0,
                             29.0, 29.0, 29.0, 30.0, 31.0, 49.0, 50.0, 51.0, 51.0, 51.0]
        in_takeoff_alt[1] = [ 1.0,  1.0,  2.0,  2.0,  3.0, 14.0, 15.0, 15.0, 16.0, 16.0,
                             29.0, 29.0, 30.0, 30.0, 31.0, 49.0, 50.0, 50.0, 51.0, 51.0]
        in_takeoff_alt[2] = [ 1.0,  2.0,  2.0,  2.0,  3.0, 14.0, 15.0, 15.0, 15.0, 16.0,
                             29.0, 30.0, 30.0, 30.0, 31.0, 49.0, 50.0, 50.0, 50.0, 51.0]

        in_min_alt = [ 2.0,  2.0,  2.0,  2.0,  2.0,  2.0,  2.0,  2.0,  2.0,  2.0,
                      30.0, 30.0, 30.0, 30.0, 30.0, 30.0, 30.0, 30.0, 30.0, 30.0]
        in_max_alt = [15.0, 15.0, 15.0, 15.0, 15.0, 15.0, 15.0, 15.0, 15.0, 15.0,
                      50.0, 50.0, 50.0, 50.0, 50.0, 50.0, 50.0, 50.0, 50.0, 50.0]



        result = [False, False, False, True, True, True, True, False, False, False,
                  False, False, False, True, True, True, True, False, False, False]


        test_case = 0
        while test_case < len(result):
            self.m._MinAlt[0] = in_min_alt[test_case]
            self.m._MaxAlt[0] = in_max_alt[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                #
                #self.m._takeoff_alt[drone_num] = in_takeoff_alt[drone_num][test_case]
                self.m._Drone1TakeoffAlt[0] = in_takeoff_alt[0][test_case]
                self.m._Drone2TakeoffAlt[0] = in_takeoff_alt[1][test_case]
                self.m._Drone3TakeoffAlt[0] = in_takeoff_alt[2][test_case]
                #
                drone_num += 1

            ret = self.m._takeoff_range_check()

            self.assertEqual(ret, result[test_case])

            test_case += 1

    def test_is_valid_button(self):

        in_button = [-1,0,100,16383,16384]

        result = [False, True, True, True, False]

        test_case = 0
        while test_case < len(result):

            self.assertEqual(self.m._is_valid_button(in_button[test_case]), result[test_case])

            test_case += 1

    def test_is_valid_analog(self):
        test_case = 0

        in_no = [0, 1, 2, 3, 4, 5, 6, 7]
        in_analog = [-2, -1, 0, 0.5, 1, 2]

        result = [False, True, True, False, True, False,
                  False, True, True, False, True, False,
                  False, True, True, True, True, False,
                  False, True, True, True, True, False,
                  False, True, True, True, True, False,
                  False, True, True, True, True, False,
                  False, True, True, True, True, False,
                  False, True, True, True, True, False]

        for no in range(len(in_no)):
            for analog in range(len(in_analog)):
                self.assertEqual(self.m._is_valid_analog(in_analog[analog], in_no[no]), result[test_case])            
                test_case += 1

    def test_is_valid_drctrl(self):

        in_id = [-1, -1, -1, 0, -1, -1, -1, 0, -1, -1,
                 0, 0, 0, 0, 0, 0]
        in_home_lat = [100, 100, 0, 100, 100, 100, 0, 100, 0, 0,
                       100, 0, 0, 100, 0, 0]
        in_home_lon = [200, 200, 200, 200, 200, 0, 200, 0, 200, 0,
                       200, 0, 0, 0, 0, 0]
        in_drone_lat = [100, 100, 100, 100, 0, 100, 100, 100, 0, 0,
                        100, 0, 0, 0, 0, 0]
        in_drone_lon = [200, 0, 200, 200, 200, 200, 0, 200, 0, 200,
                        0, 200, 0, 0, 0, 0]
        in_drone_alt = [100, 100, 100, 100, 100, 2, 100, 2, 100, 2, 2,
                        100, 2, 2, 2, 2]
        in_mag = [500, 500, 500, 500, 0, 0, 500, 0, 500, 0,
                  0, 0, 500, 0, 0, 0]
        in_state = ["Other", "Other", "Other", "Hovering", "Other", "Other", "Hovering", "Other", "Hovering", "Other",
                    "Hovering","Hovering", "Other", "Hovering", "Other","Hovering"]

        result = [False, False, False, False, False, False, False, False, False, False,
                  False, False, False, False, False, True]

        for test_case in range(len(result)):
            self.m._d_inDrCtrl[0].id = in_id[test_case]
            self.m._d_inDrCtrl[0].homeGPS.latitude = in_home_lat[test_case]
            self.m._d_inDrCtrl[0].homeGPS.longitude = in_home_lon[test_case]
            self.m._d_inDrCtrl[0].droneGPS.latitude = in_drone_lat[test_case]
            self.m._d_inDrCtrl[0].droneGPS.longitude = in_drone_lon[test_case]
            self.m._d_inDrCtrl[0].droneGPS.altitude = in_drone_alt[test_case]
            self.m._d_inDrCtrl[0].mag.data = in_mag[test_case]
            self.m._d_inDrCtrl[0].state.data = in_state[test_case]
            self.assertEqual(self.m._is_valid_drctrl(self.m._d_inDrCtrl[0], 0), result[test_case])

    def test_is_valid_target(self):

        in_id = [-1, 0, -1, -1, -1, 0, -1, 0,-1, 0,
                 0, 0, 0, 0]
        in_target_lat = [100, 100, 0, 100, 0, 100, 0, 100, 0, 100,
                         0, 0, 0, 0]
        in_target_lon = [200, 200, 200, 0, 200, 0, 200, 0, 0, 0,
                         200, 0, 0, 0]
        in_target_alt = [100, 100, 100, 100, 100, 100, 2, 2, 2, 2,
                         2, 100, 2, 2]

        in_speed = [-1, -1, -1, -1, -1, -1, 0, -1, -1, 0,
                    -1, 0, 0, 0]

        in_yaw_data0 = [10, 10, 10, 10, 0, 10, 10, 0, 10, 0,
                        0, 0, 0, 0]

        in_yaw_data1 = [500, 500, 500, 0, 500, 0, 500, 500, 0, 500,
                        0, 0, 500, 0]

        in_state = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                    "Hovering", "Hovering", "Hovering", "Hovering"]

        result = [False, False, False, False, False, False, False, False, False, False,
                  False, False, False, True]

        for test_case in range(len(result)):
            self.m._d_outTarget[0].id = in_id[test_case]
            self.m._d_outTarget[0].pos.latitude = in_target_lat[test_case]
            self.m._d_outTarget[0].pos.longitude = in_target_lon[test_case]
            self.m._d_outTarget[0].pos.altitude = in_target_alt[test_case]
            self.m._d_outTarget[0].speed.data = in_speed[test_case]
            self.m._d_outTarget[0].yaw.data = [in_yaw_data0[test_case], in_yaw_data1[test_case]]
            self.m._d_inDrCtrl[0].state.data = in_state[test_case]
            self.assertEqual(self.m._is_valid_target(self.m._d_outTarget[0], 0), result[test_case])

    def test_is_valid_virtual_leader(self):

        in_drone_lat = [100, 0, 100, 100, 100, 100, 0, 0, 0, 0,
                        100, 100, 100, 100, 100, 100, 0, 0, 0, 100,
                        100, 100, 0, 0, 100, 100, 0, 0, 0, 0,
                        100, 0]
        in_drone_lon = [200, 200, 0, 200, 200, 200, 0, 200, 200, 200,
                        0, 0, 0, 200, 200, 200, 0, 0, 0, 0,
                        0, 200, 200, 200, 0, 200, 0, 0, 0, 200,
                        0, 0]
        in_target_lat = [100, 100, 100, 0, 100, 100, 100, 0, 100, 100,
                         0, 100, 100, 0, 0, 100, 0, 100, 100, 0,
                         0, 0, 0, 100, 100, 0, 0, 0, 100, 0,
                         0, 0]
        in_target_lon = [200, 200, 200, 200, 0, 200, 200, 200, 0, 200,
                         200, 0, 200, 0, 200, 0, 200, 0, 200, 0,
                         200, 0, 0, 0, 0, 0, 0, 200, 0, 0,
                         0, 0]
        in_mag = [500, 500, 500, 500, 500, 0, 500, 500, 500, 0,
                  500, 500, 0, 500, 0, 0, 500, 500, 0, 500,
                  0, 0, 500, 0, 0, 0, 500, 0, 0, 0,
                  0, 0]

        result = [False, False, False, False, False, False, False, False, False, False,
                  False, False, False, False, False, False, False, False, False, False,
                  False, False, False, False, False, False, False, False, False, False,
                  False, True]

        for test_case in range(len(result)):
            self.m._d_outVirtualLeader.droneGPS.latitude = in_drone_lat[test_case]
            self.m._d_outVirtualLeader.droneGPS.longitude = in_drone_lon[test_case]
            self.m._d_outVirtualLeader.targetGPS.latitude = in_target_lat[test_case]
            self.m._d_outVirtualLeader.targetGPS.longitude = in_target_lon[test_case]
            self.m._d_outVirtualLeader.mag.data = in_mag[test_case]
            self.assertEqual(self.m._is_valid_virtual_leader(self.m._d_outVirtualLeader), result[test_case])

    def test_is_valid_id(self):
        in_id = [-1, 0, 1]

        result = [False, True, True]

        for test_case in range(len(result)):
            self.assertEqual(self.m._is_valid_id(in_id[test_case], 0, "name"), result[test_case])

    def test_is_valid_gps_lat(self):
        in_gps_lat = [-91, -90, -89, -80, 0, 80, 89, 90, 91]

        result = [False, True, True, True, True, True, True, True, False]

        for test_case in range(len(result)):
            self.assertEqual(self.m._is_valid_gps_lat(in_gps_lat[test_case], 0, "name"), result[test_case])

    def test_is_valid_gps_lon(self):
        in_gps_lon = [-181, -180, -179, -90, 0, 90, 179, 180, 181]

        result = [False, True, True, True, True, True, True, True, False]

        for test_case in range(len(result)):
            self.assertEqual(self.m._is_valid_gps_lon(in_gps_lon[test_case], 0, "name"), result[test_case])

    def test_is_valid_gps_alt(self):
        test_case = 0
        in_state = ["Hovering", "In_Move", "Alt_CHG", "Yaw_CHG", "Other"]
        in_gps_alt = [0, 1, 2, 3, 9, 14, 15, 16]

        result = [False, False, False, False, True,
                  False, False, False, False, True,
                  True, True, True, True, True,
                  True, True, True, True, True,
                  True, True, True, True, True,
                  True, True, True, True, True,
                  True, True, True, True, True,
                  False, False, False, False, True]

        for alt_case in range(len(in_gps_alt)):
            for state_case in range(len(in_state)):
                self.assertEqual(self.m._is_valid_gps_alt(
                                 in_gps_alt[alt_case], in_state[state_case], 0, "name"), result[test_case])
                test_case += 1

    def test_is_valid_angle(self):
        in_angle = [-1, 0, 1, 90, 180, 270, 359, 360, 361]

        result = [False, True, True, True, True, True, True, True, False]

        for test_case in range(len(result)):
            self.assertEqual(self.m._is_valid_angle(in_angle[test_case], 0, "name"), result[test_case])

    def test_is_valid_angle_dir(self):
        in_angle_dir = [-2, -1, 0, 0.5, 1, 2]

        result = [False, True, True, False, True, False]

        for test_case in range(len(result)):
            self.assertEqual(self.m._is_valid_angle_dir(in_angle_dir[test_case], 0, "name"), result[test_case])

    def test_is_valid_state(self):
        in_state = ["Armed", "Takeoff", "Hovering", "In_Move", "Alt_CHG", "Yaw_CHG", "RTL", "Disarm", "Other"]

        result = [True, True, True, True, True, True, True, True, False]

        for test_case in range(len(result)):
            self.assertEqual(self.m._is_valid_state(in_state[test_case], 0), result[test_case])

    def test_is_valid_speed(self):
        in_speed = [-1, 0, 1]

        result = [False, True, True]

        for test_case in range(len(result)):
            self.assertEqual(self.m._is_valid_speed(in_speed[test_case], 0), result[test_case])

    def test_ready_to_log_write(self):
        
        self.m._d_inDroneCtrl = RTC.DroneCtrl(RTC.Time(1513041940, 0),
                                             0,
                                             RTC.TimedString(RTC.Time(0, 0), ""),
                                             RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                             RTC.TimedFloat(RTC.Time(0, 0), 0),
                                             RTC.TimedString(RTC.Time(0, 0), "Armed"))
        
        in_file_md = [DroneFlightDirector.FILE_MODE_WRITE, DroneFlightDirector.FILE_MODE_ADD_WRITE]
        self.m._file_name = ""
        self.m._file = None
        self.m._writer = None
        self.m._Drone_Num = 3
        self.m._act_start_date, self.m._act_start_time = self.m._get_datetime_to_str()
        self.m._log_row = [["NULL" for item in range(len(self.m._header))] for robot_num in range(self.m._Drone_Num)]

        for cnt in range(len(in_file_md)):
            self.m._ready_to_log_write(in_file_md[cnt])
            self.m._close_rtc_viewer_log()
            self.assertEqual(self.m._file, None)
            self.assertEqual(self.m._writer, None)

    def test_is_open_rtc_viewer_log(self):

        in_file_md = [DroneFlightDirector.FILE_MODE_WRITE, DroneFlightDirector.FILE_MODE_WRITE, DroneFlightDirector.FILE_MODE_WRITE, DroneFlightDirector.FILE_MODE_ADD_WRITE, DroneFlightDirector.FILE_MODE_ADD_WRITE, DroneFlightDirector.FILE_MODE_ADD_WRITE]
        in_file_name = ["", "20171211_144623_ctrl_log", "20171211_144623_ctrl_log.csv", "", "20171212_104554_ctrl_log", "20171212_104554_ctrl_log.csv"]
        out_rtn = [False, True, True, False, True, True]

        for cnt in range(len(in_file_md)):
            self.m._file_name = in_file_name[cnt]
            rtn = self.m._is_open_rtc_viewer_log(in_file_md[cnt])
            self.assertEqual(rtn, out_rtn[cnt])

    def test_is_csv_writer(self):

        in_file = [None] 
        in_writer = [None] 
        out_rtn = [False]

        for cnt in range(len(in_file)):
            self.m._file = in_file[cnt]
            self.m._writer = in_writer[cnt]
            rtn = self.m._is_csv_writer()

    def test_conv_rtctime_to_datetime(self):
        
        tm = RTC.Time(1512971183, 993842000)
        date, time = self.m._conv_rtctime_to_datetime(tm)
        self.assertEqual(date, "20171211")
        self.assertEqual(time, "14:46:23.993")

    def test_set_log(self):

        self.m._Drone_Num = 3

        in_rtc_state = ["Inactive", "Error", "Exit", "Active", "Active"]

        init_ctrl = RTC.DroneCtrl(RTC.Time(0, 0),
                                0,
                                RTC.TimedString(RTC.Time(0, 0), ""),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                RTC.TimedString(RTC.Time(0, 0), ""))

        ctrl1 = RTC.DroneCtrl(RTC.Time(1512971183, 993842000),
                                1,
                                RTC.TimedString(RTC.Time(0, 0), "drone1"),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.5223693848, 139.9389993, 208.09, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9387618, 9.99, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.19, 0.0]),
                                RTC.TimedFloat(RTC.Time(0, 0), 89.0),
                                RTC.TimedString(RTC.Time(0, 0), "In_Move"))

        ctrl2 = RTC.DroneCtrl(RTC.Time(1512971183, 993842000),
                                2,
                                RTC.TimedString(RTC.Time(0, 0), "drone2"),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.5223693848, 139.9389993, 208.09, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9387618, 9.99, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.19, 0.0]),
                                RTC.TimedFloat(RTC.Time(0, 0), 89.0),
                                RTC.TimedString(RTC.Time(0, 0), "In_Move"))

        ctrl3 = RTC.DroneCtrl(RTC.Time(1512971183, 993842000),
                                3,
                                RTC.TimedString(RTC.Time(0, 0), "drone3"),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.5223693848, 139.9389993, 208.09, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9387618, 9.99, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.19, 0.0]),
                                RTC.TimedFloat(RTC.Time(0, 0), 89.0),
                                RTC.TimedString(RTC.Time(0, 0), "In_Move"))

        in_ctrl1 = [init_ctrl, init_ctrl, init_ctrl, init_ctrl, ctrl1]
        in_ctrl2 = [init_ctrl, init_ctrl, init_ctrl, init_ctrl, ctrl2]
        in_ctrl3 = [init_ctrl, init_ctrl, init_ctrl, init_ctrl, ctrl3]
        self.m._out_target_wrote = [False] * self.m._Drone_Num
        in_wrote = [[False, False, False, False, True], [False, False, False, False, True], [False, False, False, False, True]]

        init_pos = RTC.DroneTarget(RTC.Time(1512971183, 993842000),
                                    0,
                                    RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                0.0, 0, RTC.GPS_FIX_NORMAL),
                                    RTC.TimedFloat(RTC.Time(0, 0), 0),
                                    RTC.TimedShortSeq(RTC.Time(0, 0), [0, 0]))

        pos1 = RTC.DroneTarget(RTC.Time(1512971183, 993842000),
                                1,
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9389993, 10.0, 
                                            0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0, 0), 3.0),
                                RTC.TimedShortSeq(RTC.Time(0, 0), [1, 90]))

        pos2 = RTC.DroneTarget(RTC.Time(1512971183, 993842000),
                                2,
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9389993, 10.0, 
                                            0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0, 0), 3.0),
                                RTC.TimedShortSeq(RTC.Time(0, 0), [1, 90]))

        pos3 = RTC.DroneTarget(RTC.Time(1512971183, 993842000),
                                3,
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9389993, 10.0, 
                                            0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0, 0), 3.0),
                                RTC.TimedShortSeq(RTC.Time(0, 0), [1, 90]))

        out_tgt1 = [init_pos, init_pos, init_pos, init_pos, pos1]
        out_tgt2 = [init_pos, init_pos, init_pos, init_pos, pos2]
        out_tgt3 = [init_pos, init_pos, init_pos, init_pos, pos3]

        out_rcvdate = [["NULL", "NULL", "NULL", "NULL", "20171211"], ["NULL", "NULL", "NULL", "NULL", "20171211"], ["NULL", "NULL", "NULL", "NULL", "20171211"]]
        out_rcvtime = [["NULL", "NULL", "NULL", "NULL", "14:46:23.993"], ["NULL", "NULL", "NULL", "NULL", "14:46:23.993"], ["NULL", "NULL", "NULL", "NULL", "14:46:23.993"]]
        out_rcv_robotid = [["NULL", "NULL", "NULL", "NULL", "drone1"], ["NULL", "NULL", "NULL", "NULL", "drone2"], ["NULL", "NULL", "NULL", "NULL", "drone3"]]
        out_rcv_id = [["NULL", "NULL", "NULL", "NULL", 1], ["NULL", "NULL", "NULL", "NULL", 2], ["NULL", "NULL", "NULL", "NULL", 3]]
        out_state = [["NULL", "NULL", "NULL", "NULL", "In_Move"], ["NULL", "NULL", "NULL", "NULL", "In_Move"], ["NULL", "NULL", "NULL", "NULL", "In_Move"]]
        out_h_lat = [["NULL", "NULL", "NULL", "NULL", "37.5223693848"], ["NULL", "NULL", "NULL", "NULL", "37.5223693848"], ["NULL", "NULL", "NULL", "NULL", "37.5223693848"]]
        out_h_lon = [["NULL", "NULL", "NULL", "NULL", "139.9389993000"], ["NULL", "NULL", "NULL", "NULL", "139.9389993000"], ["NULL", "NULL", "NULL", "NULL", "139.9389993000"]]
        out_h_alt = [["NULL", "NULL", "NULL", "NULL", "208.09"], ["NULL", "NULL", "NULL", "NULL", "208.09"], ["NULL", "NULL", "NULL", "NULL", "208.09"]]
        out_dr_lat = [["NULL", "NULL", "NULL", "NULL", "37.5223680000"], ["NULL", "NULL", "NULL", "NULL", "37.5223680000"], ["NULL", "NULL", "NULL", "NULL", "37.5223680000"]]
        out_dr_lon = [["NULL", "NULL", "NULL", "NULL", "139.9387618000"], ["NULL", "NULL", "NULL", "NULL", "139.9387618000"], ["NULL", "NULL", "NULL", "NULL", "139.9387618000"]]
        out_dr_alt = [["NULL", "NULL", "NULL", "NULL", "9.99"], ["NULL", "NULL", "NULL", "NULL", "9.99"], ["NULL", "NULL", "NULL", "NULL", "9.99"]]
        out_accx = [["NULL", "NULL", "NULL", "NULL", "0.00"], ["NULL", "NULL", "NULL", "NULL", "0.00"], ["NULL", "NULL", "NULL", "NULL", "0.00"]]
        out_accy = [["NULL", "NULL", "NULL", "NULL", "0.19"], ["NULL", "NULL", "NULL", "NULL", "0.19"], ["NULL", "NULL", "NULL", "NULL", "0.19"]]
        out_accz = [["NULL", "NULL", "NULL", "NULL", "0.00"], ["NULL", "NULL", "NULL", "NULL", "0.00"], ["NULL", "NULL", "NULL", "NULL", "0.00"]]
        out_mag = [["NULL", "NULL", "NULL", "NULL", 89], ["NULL", "NULL", "NULL", "NULL", 89], ["NULL", "NULL", "NULL", "NULL", 89]]

        out_snddate = [["NULL", "NULL", "NULL", "20171211", "20171211"], ["NULL", "NULL", "NULL", "20171211", "20171211"], ["NULL", "NULL", "NULL", "20171211", "20171211"]]
        out_sndtime = [["NULL", "NULL", "NULL", "14:46:23.993", "14:46:23.993"], ["NULL", "NULL", "NULL", "14:46:23.993", "14:46:23.993"], ["NULL", "NULL", "NULL", "14:46:23.993", "14:46:23.993"]]
        out_snd_robotid = [["NULL", "NULL", "NULL", "NULL", "drone1"], ["NULL", "NULL", "NULL", "NULL", "drone2"], ["NULL", "NULL", "NULL", "NULL", "drone3"]]
        out_snd_id = [["NULL", "NULL", "NULL", "NULL", 1], ["NULL", "NULL", "NULL", "NULL", 2], ["NULL", "NULL", "NULL", "NULL", 3]]
        out_lat = [["NULL", "NULL", "NULL", "NULL", "37.5223680000"], ["NULL", "NULL", "NULL", "NULL", "37.5223680000"], ["NULL", "NULL", "NULL", "NULL", "37.5223680000"]]
        out_lon = [["NULL", "NULL", "NULL", "NULL", "139.9389993000"], ["NULL", "NULL", "NULL", "NULL", "139.9389993000"], ["NULL", "NULL", "NULL", "NULL", "139.9389993000"]]
        out_alt = [["NULL", "NULL", "NULL", "NULL", "10.00"], ["NULL", "NULL", "NULL", "NULL", "10.00"], ["NULL", "NULL", "NULL", "NULL", "10.00"]]
        out_spd = [["NULL", "NULL", "NULL", "NULL", "3.00"], ["NULL", "NULL", "NULL", "NULL", "3.00"], ["NULL", "NULL", "NULL", "NULL", "3.00"]]
        out_yaw_dir = [["NULL", "NULL", "NULL", "NULL", 1], ["NULL", "NULL", "NULL", "NULL", 1], ["NULL", "NULL", "NULL", "NULL", 1]]
        out_yaw_deg = [["NULL", "NULL", "NULL", "NULL", 90], ["NULL", "NULL", "NULL", "NULL", 90], ["NULL", "NULL", "NULL", "NULL", 90]]
        in_mv_md = [["NULL", "NULL", "NULL", "Absolute", "Absolute"], ["NULL", "NULL", "NULL", "Absolute", "Absolute"], ["NULL", "NULL", "NULL", "Absolute", "Absolute"]]

        self.m._act_start_date, self.m._act_start_time = self.m._get_datetime_to_str()
        mv_mode = RTC.TimedString(RTC.Time(1512971183, 993842000), "Absolute")
        self.m._d_outMoveMode = mv_mode

        for cnt in range(len(in_rtc_state)):
            self.m._d_inDrCtrl1 = in_ctrl1[cnt]
            self.m._d_inDrCtrl2 = in_ctrl2[cnt]
            self.m._d_inDrCtrl3 = in_ctrl3[cnt]
            self.m._d_inDrCtrl[0] = self.m._d_inDrCtrl1
            self.m._d_inDrCtrl[1] = self.m._d_inDrCtrl2
            self.m._d_inDrCtrl[2] = self.m._d_inDrCtrl3

            self.m._d_outTarget1 = out_tgt1[cnt]
            self.m._d_outTarget2 = out_tgt2[cnt]
            self.m._d_outTarget3 = out_tgt3[cnt]
            self.m._d_outTarget[0] = self.m._d_outTarget1
            self.m._d_outTarget[1] = self.m._d_outTarget2
            self.m._d_outTarget[2] = self.m._d_outTarget3
            self.m._out_target_wrote[0] = in_wrote[0][cnt]
            self.m._out_target_wrote[1] = in_wrote[1][cnt]
            self.m._out_target_wrote[2] = in_wrote[2][cnt]
            
            if in_rtc_state[cnt] == "Active":
                self.m._log_row = [["NULL" for item in range(len(self.m._header))] for robot_num in range(self.m._Drone_Num)]
            else:
                self.m._log_row = ["NULL"] * len(self.m._header)

            self.m._set_log(in_rtc_state[cnt])

            for num in range(self.m._Drone_Num):
                if in_rtc_state[cnt] == "Active":
                    self.assertEqual(self.m._log_row[num][self.m._header.index("RcvDate")], out_rcvdate[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("RcvTime")], out_rcvtime[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("RcvRobotID")], out_rcv_robotid[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("RcvID")], out_rcv_id[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("State")], out_state[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("HomeLat")], out_h_lat[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("HomeLon")], out_h_lon[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("HomeAlt")], out_h_alt[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("RobotLat")], out_dr_lat[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("RobotLon")], out_dr_lon[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("RobotAlt")], out_dr_alt[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("AccX")], out_accx[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("AccY")], out_accy[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("AccZ")], out_accz[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("Mag")], out_mag[num][cnt])

                    self.assertEqual(self.m._log_row[num][self.m._header.index("SndDate")], out_snddate[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("SndTime")], out_sndtime[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("SndRobotID")], out_snd_robotid[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("SndID")], out_snd_id[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("TargetLat")], out_lat[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("TargetLon")], out_lon[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("TargetAlt")], out_alt[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("TargetSpd")], out_spd[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("TargetYawDir")], out_yaw_dir[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("TargetYawDeg")], out_yaw_deg[num][cnt])
                    self.assertEqual(self.m._log_row[num][self.m._header.index("MoveMode")], in_mv_md[num][cnt])
        

if __name__ == '__main__':
    unittest.main()
