#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file target_gps_test.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import unittest

import RTC
import target_gps

import sys
import math
from collections import namedtuple

from mock import Mock
from mock import patch, MagicMock


class TestTargetGPS(unittest.TestCase):
    def setUp(self):
        self.m = target_gps.TargetGPS()
        
        xx = Mock()
        target_gps.logger.warning = xx.warning


    def test_calc_lat_lon_degree_dist(self):

        pivot = namedtuple("pivot", ["latitude", "longitude"])
        drone = namedtuple("drone", ["latitude", "longitude"])

        in_pivot_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_pivot_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_lat = [37.520, 37.520, 37.530, 37.530, 37.540, 37.540]
        in_drone_lon = [139.930, 139.930, 139.940, 139.940, 139.980,139.980]
        in_mag = [350, 160, 350, 160, 350, 160]

        in_lat = [43.060, 37.530, 35.680, 26.210]

        lat_degree_dist_result = [111133.333, 111133.333, 111133.333, 111133.333]
        lon_degree_dist_result = [81334.340, 88280.158, 90423.357, 99873.724]

        test_case = 0
        while test_case < len(lat_degree_dist_result):
            self.m._calc_lat_lon_degree_dist(in_lat[test_case])

            self.assertEqual(round(self.m._lat_degree_dist,3), lat_degree_dist_result[test_case])
            self.assertEqual(round(self.m._lon_degree_dist,3), lon_degree_dist_result[test_case])

            test_case += 1

    def test_move_turn_right_lat(self):

        pivot = namedtuple("pivot", ["latitude", "longitude"])
        drone = namedtuple("drone", ["latitude", "longitude"])

        in_pivot_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_pivot_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_lat = [37.520, 37.520, 37.530, 37.530, 37.540, 37.540]
        in_drone_lon = [139.930, 139.930, 139.940, 139.940, 139.980,139.980]
        in_mag = [350, 160, 350, 160, 350, 160]

        right_lat_result = [37.517, 37.545, 37.529, 37.533, 37.544, 37.512]
       

        test_case = 0
        while test_case < len(right_lat_result):
            #pivot_lat=37.530の場合
            self.m._lat_degree_dist = 111133.333
            self.m._lon_degree_dist = 88280.158

            p = pivot(in_pivot_lat[test_case], in_pivot_lon[test_case])
            d = drone(in_drone_lat[test_case], in_drone_lon[test_case])
            lat = round(self.m._move_turn_right_lat(p, d, in_mag[test_case]), 3)

            self.assertEqual(lat, right_lat_result[test_case])

            test_case += 1

    def test_move_turn_right_lon(self):

        pivot = namedtuple("pivot", ["latitude", "longitude"])
        drone = namedtuple("drone", ["latitude", "longitude"])

        in_pivot_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_pivot_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_lat = [37.520, 37.520, 37.530, 37.530, 37.540, 37.540]
        in_drone_lon = [139.930, 139.930, 139.940, 139.940, 139.980,139.980]
        in_mag = [350, 160, 350, 160, 350, 160]

        right_lon_result = [139.932, 139.964, 139.940, 139.959, 139.977, 139.926]
       

        test_case = 0
        while test_case < len(right_lon_result):
            #pivot_lat=37.530の場合
            self.m._lat_degree_dist = 111133.333
            self.m._lon_degree_dist = 88280.158

            p = pivot(in_pivot_lat[test_case], in_pivot_lon[test_case])
            d = drone(in_drone_lat[test_case], in_drone_lon[test_case])
            lon = round(self.m._move_turn_right_lon(p, d, in_mag[test_case]), 3)

            self.assertEqual(lon, right_lon_result[test_case])

            test_case += 1



    def test_move_turn_left_lat(self):

        pivot = namedtuple("pivot", ["latitude", "longitude"])
        drone = namedtuple("drone", ["latitude", "longitude"])

        in_pivot_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_pivot_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_lat = [37.520, 37.520, 37.530, 37.530, 37.540, 37.540]
        in_drone_lon = [139.930, 139.930, 139.940, 139.940, 139.980,139.980]
        in_mag = [350, 160, 350, 160, 350, 160]

        left_lat_result = [37.523, 37.534, 37.531, 37.527, 37.536, 37.529]
       

        test_case = 0
        while test_case < len(left_lat_result):
            #pivot_lat=37.530の場合
            self.m._lat_degree_dist = 111133.333
            self.m._lon_degree_dist = 88280.158

            p = pivot(in_pivot_lat[test_case], in_pivot_lon[test_case])
            d = drone(in_drone_lat[test_case], in_drone_lon[test_case])
            lat = round(self.m._move_turn_left_lat(p, d, in_mag[test_case]), 3)

            self.assertEqual(lat, left_lat_result[test_case])

            test_case += 1

    def test_move_turn_left_lon(self):

        pivot = namedtuple("pivot", ["latitude", "longitude"])
        drone = namedtuple("drone", ["latitude", "longitude"])

        in_pivot_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_pivot_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_lat = [37.520, 37.520, 37.530, 37.530, 37.540, 37.540]
        in_drone_lon = [139.930, 139.930, 139.940, 139.940, 139.980,139.980]
        in_mag = [350, 160, 350, 160, 350, 160]

        left_lon_result = [139.928, 139.973, 139.940, 139.959, 139.982, 139.918]
       

        test_case = 0
        while test_case < len(left_lon_result):
            #pivot_lat=37.530の場合
            self.m._lat_degree_dist = 111133.333
            self.m._lon_degree_dist = 88280.158

            p = pivot(in_pivot_lat[test_case], in_pivot_lon[test_case])
            d = drone(in_drone_lat[test_case], in_drone_lon[test_case])
            lon = round(self.m._move_turn_left_lon(p, d, in_mag[test_case]), 3)

            self.assertEqual(lon, left_lon_result[test_case])

            test_case += 1

if __name__ == '__main__':
    unittest.main()
