#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DroneIO_test.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import unittest
import RTC
import OpenRTM_aist
import drone_distance_cal
import DroneIO

# Import DroneKit
from dronekit import connect, VehicleMode, LocationGlobalRelative, Command, Vehicle
from pymavlink import mavutil

from mock import Mock

class TestDroneIO(unittest.TestCase):

    def setUp(self):

        self.m = DroneIO.DroneIO(0)

        self.xx_logger = Mock()
        DroneIO.logger.warning = self.xx_logger.warning
        DroneIO.logger.error = self.xx_logger.error
        DroneIO.logger.info = self.xx_logger.info
        DroneIO.logger.debug = self.xx_logger.debug

        # if self.m._is_vehicle_connected() is True:
        #     print "vehicle connect & get success home_location"

    # def tearDown(self):
    #     self.m._vehicle.close()
    #     print "vehicle disconnect"

    def test_onExecute(self):

        xx = Mock()
        self.m._vehicle = xx.vehicle


        self.m._batt_thr = (DroneIO.BATT_LIPO_VPC - (DroneIO.BATT_LIPO_VPC_PERCENT -
                                           (self.m._RTLLowBatt[0] / 100.0))) * self.m._BattCell[0]

        in_armed = [False, True, True, True, True, True, True, True, True]
        in_sts = [DroneIO.STS_RTL, DroneIO.STS_RTL, DroneIO.STS_RTL, DroneIO.STS_RTL, 
        DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_HOVERING, DroneIO.STS_IN_MOVE, DroneIO.STS_ALT_CHG]
        in_batt_sts = [False, False, False, False, False, True, True, True, True]
        in_flt_mode = ["GUIDED", "GUIDED", "GUIDED", "GUIDED", "GUIDED", "GUIDED", "GUIDED", "GUIDED", "LOITER"]
        in_read = [False, False, False, False, False, False, False, True, True]
        in_comm_err = [False, False, False, False, False, False, True, False, False]
        in_sys = ['single', 'single', 'formation', 'formation', 'single', 'single', 'single', 'single', 'single']
        in_rtl_cause = [DroneIO.RTL_CAUSE_BY_MANU, DroneIO.RTL_CAUSE_BY_MANU, DroneIO.RTL_CAUSE_BY_MANU, DroneIO.RTL_CAUSE_BY_ERR, DroneIO.RTL_CAUSE_BY_MANU,
        DroneIO.RTL_CAUSE_BY_MANU, DroneIO.RTL_CAUSE_BY_MANU, DroneIO.RTL_CAUSE_BY_MANU, DroneIO.RTL_CAUSE_BY_MANU]

        #out_rtc_rtn = [RTC.RTC_OK, RTC.RTC_OK, RTC.RTC_OK, RTC.RTC_OK, RTC.RTC_ERROR, RTC.RTC_OK, RTC.RTC_ERROR, RTC.RTC_OK, RTC.RTC_OK]
        out_rtc_rtn = [True, True, True, True, False, True, False, True, True]

        for cnt in range(len(in_armed)):
            rtn = True
            rtc_rtn = True
            # Armed Check
            xx.armed.return_value = in_armed[cnt]
            if xx.armed.return_value is True:
                # RTL or not
                self.m._drone_sts = in_sts[cnt]
                if self.m._drone_sts != DroneIO.STS_RTL:
                    # battery check
                    batt_rtn = self.m._is_batt_state()
                    batt_rtn = in_batt_sts[cnt]
                    if batt_rtn is True:
                        # Read the target position
                        xx.mode.name = in_flt_mode[cnt]
                        if xx.mode.name == "GUIDED":
                            #read_inf = self.m._is_read_info()
                            read_inf = in_read[cnt]
                            if  read_inf is True:
                                self.m._comm_err_timer = 0
                                # self.m._flight_ctrl_core()
                            else:
                                self.m._comm_err_timer += 1
                                rtn_comm_err = self.m._is_gcs_comm_err() 
                                rtn_comm_err = in_comm_err[cnt] 
                                if rtn_comm_err is True:
                                    # GCS communication error
                                    rtn = False

                        # judge the target position arrive
                        self.m._arv_judge()
                    else:
                        # Low batt detected. Start the RTL.
                        rtn = False
                else:
                    # RTL for formation-flight
                    self.m._SystemBehavior[0] = in_sys[cnt]
                    self.m._rtl_cause = in_rtl_cause[cnt]
                    if self.m._SystemBehavior[0] == 'formation':
                        if self.m._rtl_cause == DroneIO.RTL_CAUSE_BY_MANU:
                            self.m._continuous_manu_rtl()

                # Read & output the sensor info
                #self.m._read_sensor_info()
                #self.m._write_info()
            else:
                # switching from armed to disarm
                #self.m.deactivate(ec_id)
                self.xx_logger.info(DroneIO.INF_MSG_010)

            if rtn is True:
                rtc_rtn = True # RTC.RTC_OK
            else:
                rtc_rtn = False # RTC.RTC_ERROR

            self.assertEqual(rtc_rtn, out_rtc_rtn[cnt])

    def test_onError(self):

        xx = Mock()
        self.m._vehicle = xx

        in_sts = [DroneIO.STS_DISARM, DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_RTL, DroneIO.STS_IN_MOVE, DroneIO.STS_ALT_CHG, DroneIO.STS_HOVERING, DroneIO.STS_YAW_CHG,
        DroneIO.STS_DISARM, DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_RTL, DroneIO.STS_IN_MOVE, DroneIO.STS_ALT_CHG, DroneIO.STS_HOVERING, DroneIO.STS_YAW_CHG]
        in_armed = [True, True, True, True, True, True, True, True, False, False, False, False, False, False, False, False]

        for cnt in range(len(in_sts)):
            self.m._drone_sts = in_sts[cnt]
            self.m._vehicle.armed = in_armed[cnt]

            # Start RTL, if in flight
            if self.m._drone_sts not in [DroneIO.STS_DISARM, DroneIO.STS_ARMED, DroneIO.STS_RTL]:
                self.m._rtl_by_err()

            # Read & output the sensor info
            #self.m._read_sensor_info()
            #self.m._write_info()

            # State change to disarm, if landed
            if self.m._vehicle.armed is False:
                self.m._drone_sts = DroneIO.STS_DISARM
                self.m._write_io_state()
                self.xx_logger.info(DroneIO.INF_MSG_014)
                #self.m.reset(ec_id)
        
    
    def test_is_read_info(self):

        xx = Mock()

        self.m._inTargetIn.isNew = xx.isNew
        self.m._inTargetIn.read = xx.read

        vv = Mock()
        self.m._vehicle = vv.vehicle

        init_pos = RTC.DroneTarget(RTC.Time(0,0),
                                    0,
                                    RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                0.0, 0, RTC.GPS_FIX_NORMAL),
                                    RTC.TimedFloat(RTC.Time(0,0),0),
                                    RTC.TimedShortSeq(RTC.Time(0,0),[0.0, 0.0]))

        pos1 = RTC.DroneTarget(RTC.Time(0,0),
                                0,
                                RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                            37.522794, 139.938675, -1, 
                                            0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0,0),0),
                                RTC.TimedShortSeq(RTC.Time(0,0),[0.0, 0.0]))

        pos2 = RTC.DroneTarget(RTC.Time(0,0),
                                0,
                                RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                            37.522794, 139.938675, 0, 
                                            0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0,0),0),
                                RTC.TimedShortSeq(RTC.Time(0,0),[0.0, 0.0]))

        pos3 = RTC.DroneTarget(RTC.Time(0,0),
                                0,
                                RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                            37.522794, 139.938675, 1, 
                                            0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0,0),0),
                                RTC.TimedShortSeq(RTC.Time(0,0),[0.0, 0.0]))

        pos4 = RTC.DroneTarget(RTC.Time(0,0),
                                0,
                                RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                            37.522794, 139.938675, 13.4, 
                                            0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0,0),0),
                                RTC.TimedShortSeq(RTC.Time(0,0),[0.0, 0.0]))

        in_is_new = [False, True, True, True, True]
        in_pos = [init_pos, pos1, pos2, pos3, pos4]
        result = [False, False, False, True, True]

        for cnt in range(len(in_is_new)):
            xx.isNew.return_value = in_is_new[cnt]
            xx.read.return_value = in_pos[cnt]
            self.m._vehicle.home_location.lat = 37.52294
            self.m._vehicle.home_location.lon = 139.938675
            self.m._vehicle.home_location.alt = 208.59
            self.m._vehicle.location.global_relative_frame.lat = 37.52294
            self.m._vehicle.location.global_relative_frame.lon = 139.938675
            self.m._vehicle.location.global_relative_frame.alt = 13.4
            rtn = self.m._is_read_info()
            
            self.assertEqual(rtn, result[cnt])

    def test_is_read_target_pos(self):

        pos = [RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0), -90.1, -180.1, -0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0, RTC.GPS_FIX_NORMAL),
                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0), -90.1, -180.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0, RTC.GPS_FIX_NORMAL),
                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0), -90.1, 139.938675, 13.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0, RTC.GPS_FIX_NORMAL),
                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0), 37.522794, 139.938675, 13.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0, RTC.GPS_FIX_NORMAL)
        ]
        in_lat = [-90.1, -90.1, -90.1, 37.522794]
        in_lon = [-180.1, -180.1, 139.938675, 139.938675]
        in_alt = [-0.1, 0.0, 13.4, 13.4]
        result = [False, False, False, True]

        for cnt in range(len(pos)):
            self.m._d_inTarget.pos = pos[cnt]
            rtn = self.m._is_read_target_pos()
            self.assertEqual(self.m._d_inTarget.pos.latitude, in_lat[cnt])
            self.assertEqual(self.m._d_inTarget.pos.longitude, in_lon[cnt])
            self.assertEqual(self.m._d_inTarget.pos.altitude, in_alt[cnt])
            self.assertEqual(rtn, result[cnt])

    def test_is_read_target_yaw(self):

        in_yaw_deg = [-1.0, 0.0, 1.0, 90.0, 359.0, 360.0, 361.0]
        result = [False, True, True, True, True, True, False]
        self.m._d_inTarget.yaw = RTC.TimedShortSeq(RTC.Time(0,0),[0.0, 0.0])

        for cnt in range(len(in_yaw_deg)):
            self.m._d_inTarget.yaw.data[1] = in_yaw_deg[cnt]
            rtn = self.m._is_read_target_yaw()
            
            self.assertEqual(self.m._d_inTarget.yaw.data[1], in_yaw_deg[cnt])
            self.assertEqual(rtn, result[cnt])

    def test_arv_judge(self):

        self.m._takeoff_alt = 10.0

        in_sts = [DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_TAKEOFF, DroneIO.STS_TAKEOFF]
        in_drone_alt = [0.0, 9.8, 9.9, 10.0]
        out_sts = [DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING]
        
        for cnt in range(len(in_sts)):
            self.m._drone_sts = in_sts[cnt]
            self.m._gps_drone.altitude = in_drone_alt[cnt]
            self.m._arv_judge()
            self.assertEqual(self.m._drone_sts, out_sts[cnt])
        
    def test_is_gcs_comm_err(self):

        data = [29, 30, 31]
        result = [False, True, False]

        for i, dat in enumerate(data):
            self.m._comm_err_timer = dat
            rtn = self.m._is_gcs_comm_err()
            self.assertEqual(rtn, result[i])

    def test_takeoff(self):

        data = [5, 10]
        result = [DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF]

        for i, dat in enumerate(data):
            self.m._d_inTarget.pos.altitude = dat
            self.m._drone_sts = DroneIO.STS_ARMED
            #self.m._takeoff()
            #self.assertEqual(self.m._drone_sts, result[i])

    def test_is_rtl_req(self):

        in_lat = [37.522614, 37.522615, 37.522614, 37.522615, 37.522614, 37.522615, 37.522614, 37.522615]
        in_lon = [139.938675, 139.938675, 139.938676, 139.938676, 139.938675, 139.938675, 139.938676, 139.938676]
        in_alt = [208.1, 208.1, 208.1, 208.1, 208.2, 208.2, 208.2, 208.2]
        result = [True, False, False, False, False, False, False, False]

        self.m._gps_home.latitude = 37.522614
        self.m._gps_home.longitude = 139.938675
        self.m._gps_home.altitude = 208.1

        for cnt in range(len(result)):
           self.m._d_inTarget.pos.latitude = in_lat[cnt] 
           self.m._d_inTarget.pos.longitude = in_lon[cnt] 
           self.m._d_inTarget.pos.altitude = in_alt[cnt]
           rtn = self.m._is_rtl_req()
           self.assertEqual(rtn, result[cnt])
 
    def test_is_batt_state(self):

        xx = Mock()
        self.m._vehicle = xx

        self.m._batt_thr = (DroneIO.BATT_LIPO_VPC - (DroneIO.BATT_LIPO_VPC_PERCENT -
                                           (self.m._RTLLowBatt[0] / 100.0))) * self.m._BattCell[0]
        
        in_batt_chk = ['off', 'on', 'on', 'on']
        in_curr_batt = [13.9, 13.9, 14.0, 14.1]
        out_rtn = [True, False, True, True]

        for cnt in range(len(in_batt_chk)):
            self.m._BattChk[0] = in_batt_chk[cnt]
            self.m._vehicle.battery.voltage = in_curr_batt[cnt]
            rtn = self.m._is_batt_state()
            self.assertEqual(rtn, out_rtn[cnt])
    
    def test_continuous_manu_rtl(self):

        in_rtl_stg = [DroneIO.RTL_STAGE_1ST, DroneIO.RTL_STAGE_1ST, DroneIO.RTL_STAGE_2ND, DroneIO.RTL_STAGE_2ND, 
        DroneIO.RTL_STAGE_2ND, DroneIO.RTL_STAGE_2ND, DroneIO.RTL_STAGE_3RD]
        in_is_arv_tgt_alt = [False, True, True, True, True, True, True]
        in_read_pos = [False, False, False, True, False, True, False]
        in_is_rtl_req = [False, False, False, False, True, True, True]

        for cnt in range(len(in_rtl_stg)):
            self.m._rtl_stage = in_rtl_stg[cnt] 

            if self.m._rtl_stage == DroneIO.RTL_STAGE_1ST:
                rtn_is_arv_tgt_alt = self.m._is_arv_tgt_alt()
                rtn_is_arv_tgt_alt = in_is_arv_tgt_alt[cnt]
                if rtn_is_arv_tgt_alt is True:
                    self.m._rtl_stage = DroneIO.RTL_STAGE_2ND
                    point = LocationGlobalRelative(
                        self.m._gps_home.latitude,
                        self.m._gps_home.longitude,
                        self.m._tgt_alt)
                    #self.m._vehicle.simple_goto(point)

            elif self.m._rtl_stage == DroneIO.RTL_STAGE_2ND:
                read_pos = self.m._is_read_info()
                read_pos = in_read_pos[cnt]
                if read_pos:
                    rtn_is_rtl_req = self.m._is_rtl_req()
                    rtn_is_rtl_req = in_is_rtl_req[cnt]
                    if rtn_is_rtl_req:
                        self.m._rtl_stage = DroneIO.RTL_STAGE_3RD
                        #self.m._start_rtl()

            else:
                pass

    def test_is_arv_tgt_alt(self):

        self.m._tgt_alt = 10.0
        in_drone_alt = [9.4, 9.5, 9.6, 10.0, 10.4, 10.5, 10.6]
        out_rtn = [False, True, True, True, True, True, False]

        for cnt in range(len(in_drone_alt)):
            self.m._gps_drone.altitude = in_drone_alt[cnt]
            rtn = self.m._is_arv_tgt_alt()
            self.assertEqual(rtn, out_rtn[cnt])

    def test_move_to_target_pos(self):
        in_curr_lat = [37.5225155, 37.5225155, 37.5225155, 37.5225155, 37.5225155, 37.5225155, 37.5225155, 37.5225155]
        in_curr_lon = [139.9385735, 139.9385735, 139.9385735, 139.9385735, 139.9385735, 139.9385735, 139.9385735, 139.9385735]
        # in_curr_alt = [4.89, 4.9, 4.91, 4.9, 4.9, 4.9, 4.9, 4.9]
        in_curr_alt = [4.79, 4.8, 4.81, 4.8, 4.8, 4.8, 4.8, 4.8]
        in_tar_lat = [37.5225335, 37.5225335, 37.5225335, 37.5225336, 37.5229265, 37.5225335, 37.5225336, 37.5229265]
        in_tar_lon = [139.9385734, 139.9385734, 139.9385734, 139.9385734, 139.9385734, 139.9385734, 139.9385734, 139.9385734]
        in_tar_alt = [5.0, 5.0, 5.0, 5.0, 5.0, 6.0, 6.0, 6.0]
        out_state = [DroneIO.STS_ALT_CHG, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING, DroneIO.STS_IN_MOVE, DroneIO.STS_IN_MOVE, DroneIO.STS_ALT_CHG, DroneIO.STS_IN_MOVE, DroneIO.STS_IN_MOVE]

        vv = Mock()
        self.m._vehicle = vv.vehicle

        for cnt in range(len(in_curr_lat)):
            self.m._gps_drone.latitude = in_curr_lat[cnt]
            self.m._gps_drone.longitude = in_curr_lon[cnt]
            self.m._gps_drone.altitude = in_curr_alt[cnt]
            self.m._d_inTarget.pos.latitude = in_tar_lat[cnt]
            self.m._d_inTarget.pos.longitude = in_tar_lon[cnt]
            self.m._d_inTarget.pos.altitude = in_tar_alt[cnt]
            self.m._move_to_target_pos()
            self.assertEqual(self.m._drone_sts, out_state[cnt])

    def test_move_to_target_yaw(self):
        in_yaw_dir = [1, -1, 0]
        out_rtn = [True, True, False]

        self.m._d_inTarget.yaw.data = [0, 0]
        
        for cnt in range(len(in_yaw_dir)):
            rtn = False
            self.m._d_inTarget.yaw.data[0] = in_yaw_dir[cnt]
            if self.m._d_inTarget.yaw.data[0] in [DroneIO.YAW_TURN_RIGHT, DroneIO.YAW_TURN_LEFT]:
                #self.m._condition_yaw()
                rtn = True
            
            self.assertEqual(rtn, out_rtn[cnt])

    def test_write_drone_ctrl(self):

        self.m._d_outDroneCtrl = RTC.DroneCtrl(RTC.Time(0,0),
                                             0,
                                             RTC.TimedString(RTC.Time(0, 0), ""),
                                             RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.TimedFloatSeq(RTC.Time(0,0),[0.0, 0.0, 0.0]),
                                             RTC.TimedFloat(RTC.Time(0,0),0),
                                             RTC.TimedString(RTC.Time(0,0),0))
        
        in_gps_home = [False, True, True]
        in_gps_drone = [False, False, True]
        in_read_mag = [False, False, True]
        in_read_accel = [False, False, True]

        in_gps_home_lat = [37.522614, 37.522614, 37.522614]
        in_gps_home_lon = [139.938675, 139.938675, 139.938675]
        in_gps_home_alt = [208.1, 208.1, 208.1]
        in_gps_drone_lat = [37.522794, 37.522794, 37.522794]
        in_gps_drone_lon = [139.938675, 139.938675, 139.938675]
        in_gps_drone_alt = [13.4, 13.4, 13.4]
        in_mag = [349.0, 349.0, 349.0]
        in_accel = [[2.38, 1.57, 0.1], [2.38, 1.57, 0.1], [2.38, 1.57, 0.1]]
        in_state = [DroneIO.STS_HOVERING, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING]

        out_id = [1, 2, 3]
        out_gps_home_lat = [0.0, 0.0, 37.522614]
        out_gps_home_lon = [0.0, 0.0, 139.938675]
        out_gps_home_alt = [0.0, 0.0, 208.1]
        out_gps_drone_lat = [0.0, 0.0, 37.522794]
        out_gps_drone_lon = [0.0, 0.0, 139.938675]
        out_gps_drone_alt = [0.0, 0.0, 13.4]
        out_mag = [0.0, 0.0, 349.0]
        out_accel = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [2.38, 1.57, 0.1]]
        out_state = [DroneIO.STS_HOVERING, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING]

        for cnt in range(len(in_gps_home)):
            self.m._is_read_gps_home = in_gps_home[cnt]
            self.m._is_read_gps_drone = in_gps_drone[cnt]
            self.m._is_read_mag = in_read_mag[cnt]
            self.m._is_read_accel = in_read_accel[cnt]
            self.m._gps_home.latitude = in_gps_home_lat[cnt]
            self.m._gps_home.longitude = in_gps_home_lon[cnt]
            self.m._gps_home.altitude = in_gps_home_alt[cnt]
            self.m._gps_drone.latitude = in_gps_drone_lat[cnt]
            self.m._gps_drone.longitude = in_gps_drone_lon[cnt]
            self.m._gps_drone.altitude = in_gps_drone_alt[cnt]
            self.m._mag.data = in_mag[cnt]
            self.m._accel.data = in_accel[cnt]
            self.m._state.data = in_state[cnt]
            self.m._set_drone_ctrl()
            self.m._write_drone_ctrl()

            self.assertEqual(self.m._d_outDroneCtrl.id, out_id[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.homeGPS.latitude, out_gps_home_lat[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.homeGPS.longitude, out_gps_home_lon[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.homeGPS.altitude, out_gps_home_alt[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.droneGPS.latitude, out_gps_drone_lat[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.droneGPS.longitude, out_gps_drone_lon[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.droneGPS.altitude, out_gps_drone_alt[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.droneGPS.altitude, out_gps_drone_alt[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.mag.data, out_mag[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.accel.data, out_accel[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.state.data, out_state[cnt])

    def test_write_map(self):
    
        self._d_outDroneMap = RTC.DroneMap(RTC.Time(0,0),
                                           RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                       0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                       0.0, 0, RTC.GPS_FIX_NORMAL),
                                           RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                       0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                       0.0, 0, RTC.GPS_FIX_NORMAL),
                                           RTC.TimedFloat(RTC.Time(0,0),0),
                                           RTC.TimedString(RTC.Time(0,0),0))
        
        in_gps_drone = [False, True, True]
        in_read_mag = [False, False, True]

        in_gps_drone_lat = [37.522794, 37.522794, 37.522794]
        in_gps_drone_lon = [139.938675, 139.938675, 139.938675]
        in_gps_drone_alt = [13.4, 13.4, 13.4]
        in_gps_target_lat = [37.522800, 37.522800, 37.522800]
        in_gps_target_lon = [139.938675, 139.938675, 139.938675]
        in_gps_target_alt = [13.4, 13.4, 13.4]
        in_mag = [349.0, 349.0, 349.0]
        in_state = [DroneIO.STS_HOVERING, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING]

        out_gps_drone_lat = [0.0, 0.0, 37.522794]
        out_gps_drone_lon = [0.0, 0.0, 139.938675]
        out_gps_drone_alt = [0.0, 0.0, 13.4]
        out_gps_target_lat = [0.0, 0.0, 37.522800]
        out_gps_target_lon = [0.0, 0.0, 139.938675]
        out_gps_target_alt = [0.0, 0.0, 13.4]
        out_mag = [0.0, 0.0, 349.0]
        out_state = [0, 0, DroneIO.STS_HOVERING]

        for cnt in range(len(in_gps_drone)):
            self.m._is_read_gps_drone = in_gps_drone[cnt]
            self.m._is_read_mag = in_read_mag[cnt]
            self.m._gps_drone.latitude = in_gps_drone_lat[cnt]
            self.m._gps_drone.longitude = in_gps_drone_lon[cnt]
            self.m._gps_drone.altitude = in_gps_drone_alt[cnt]
            self.m._d_inTarget.pos.latitude = in_gps_target_lat[cnt]
            self.m._d_inTarget.pos.longitude = in_gps_target_lon[cnt]
            self.m._d_inTarget.pos.altitude = in_gps_target_alt[cnt]
            self.m._mag.data = in_mag[cnt]
            self.m._state.data = in_state[cnt]
            self.m._set_map()
            self.m._write_map()

            self.assertEqual(self.m._d_outDroneMap.droneGPS.latitude, out_gps_drone_lat[cnt])
            self.assertEqual(self.m._d_outDroneMap.droneGPS.longitude, out_gps_drone_lon[cnt])
            self.assertEqual(self.m._d_outDroneMap.droneGPS.altitude, out_gps_drone_alt[cnt])
            self.assertEqual(self.m._d_outDroneMap.targetGPS.latitude, out_gps_target_lat[cnt])
            self.assertEqual(self.m._d_outDroneMap.targetGPS.longitude, out_gps_target_lon[cnt])
            self.assertEqual(self.m._d_outDroneMap.targetGPS.altitude, out_gps_target_alt[cnt])
            self.assertEqual(self.m._d_outDroneMap.mag.data, out_mag[cnt])
            self.assertEqual(self.m._d_outDroneMap.state.data, out_state[cnt])

    def test_write_db(self):
        
        vv = Mock()
        self.m._vehicle = vv.vehicle
        
        in_read_gps_home = [False, True]
        in_read_gps_drone = [False, True]
        in_read_battV = [False, True]
        in_read_battA = [False, True]
        in_read_gyro = [False, True]
        in_read_accel = [False, True]
        in_read_mag = [False, True]
        in_read_temp = [False, True]
        in_read_pres = [False, True]

        in_robotID= ["drone1", "drone1"]
        in_gps_home_lat = [100.0, 37.522530]
        in_gps_home_lon = [200.0, 139.938615]
        in_gps_home_alt = [0.0, 216.650000]
        in_gps_drone_lat = [100.0, 37.522532]
        in_gps_drone_lon = [200.0, 139.938613]
        in_gps_drone_alt = [0.0, 9.990000]
        in_battV = [0.0, 15.559000]
        in_battA = [0.0, 0.0]
        in_flttime = [0, 16]
        in_gyro_r = [0, -0.00975905917585]
        in_gyro_p = [0, 0.0214286409318]
        in_gyro_y = [0, 1.79124319553]
        in_accel = [[0.0, 0.0, 0.0], [-0.06, -0.08, 0.01]]
        in_mag = [0.0, 102.0]
        in_temp = [0.0, 38.48]
        in_pres = [0.0, 980.204772949]
        in_fltMode = ["GUIDED", "GUIDED"]
        in_state = ["Hovering", "Hovering"]
        in_sysErr = ["", ""]
        in_rawIMU = [[0, 0, 0, 0, 0, 0, 0, 0, 0], [35, 8, -1011, -17, -9, 41, -193, -451, 603]]
        in_rawIMU_xacc = [0, 35]
        in_rawIMU_yacc = [0, 8]
        in_rawIMU_zacc = [0, -1011]
        in_rawIMU_xgyro = [0, -17]
        in_rawIMU_ygyro = [0, -9]
        in_rawIMU_zgyro = [0, 41]
        in_rawIMU_xmag = [0, -193]
        in_rawIMU_ymag = [0, -451]
        in_rawIMU_zmag = [0, 603]
        in_correctIMU = [[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [-0.06, -0.08, 0.01, -0.00975905917585, 0.0214286409318, 1.79124319553, 102.0]]

        out_battV = [0.0, 15.559]
        out_gyro_r = [0, -0.01]
        out_gyro_p = [0, 0.021]
        out_gyro_y = [0, 1.791]
        out_temp = [0.0, 38.48]
        out_pres = [0.0, 980.204772949]
        out_correctIMU = [[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [-0.06, -0.08, 0.01, -0.01, 0.021, 1.791, 102.0]]

        self.m._d_outDroneDB = RTC.DroneDB(RTC.Time(0, 0),
                                        RTC.TimedString(RTC.Time(0, 0), ""),
                                        RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                    100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                    100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedULong(RTC.Time(0, 0), 0),
                                        RTC.TimedOrientation3D(RTC.Time(0, 0),RTC.Orientation3D(0.0, 0.0, 0.0)),
                                        RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedString(RTC.Time(0, 0), ""),
                                        RTC.TimedString(RTC.Time(0, 0), ""),
                                        RTC.TimedString(RTC.Time(0, 0), ""),
                                        RTC.TimedLongSeq(RTC.Time(0, 0), [0, 0, 0, 0, 0, 0, 0, 0, 0]),
                                        RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]))

        for cnt in range(len(in_read_gps_home)):
            self.m._is_read_gps_home = in_read_gps_home[cnt]
            self.m._is_read_gps_drone = in_read_gps_drone[cnt]
            self.m._is_read_battV = in_read_battV[cnt]
            self.m._is_read_battA = in_read_battA[cnt]
            self.m._is_read_gyro = in_read_gyro[cnt]
            self.m._is_read_accel = in_read_accel[cnt]
            self.m._is_read_mag = in_read_mag[cnt]
            self.m._is_read_temp = in_read_temp[cnt]
            self.m._is_read_pres = in_read_pres[cnt]
            
            self.m._RobotID[0] = in_robotID[cnt]
            self.m._gps_home.latitude = in_gps_home_lat[cnt]
            self.m._gps_home.longitude = in_gps_home_lon[cnt]
            self.m._gps_home.altitude = in_gps_home_alt[cnt]
            self.m._gps_drone.latitude = in_gps_drone_lat[cnt]
            self.m._gps_drone.longitude = in_gps_drone_lon[cnt]
            self.m._gps_drone.altitude = in_gps_drone_alt[cnt]
            self.m._batt.data[0] = in_battV[cnt]
            self.m._batt.data[1] = in_battA[cnt]
            self.m._fltTime.data = in_flttime[cnt]
            self.m._gyro.data.r = in_gyro_r[cnt]
            self.m._gyro.data.p = in_gyro_p[cnt]
            self.m._gyro.data.y = in_gyro_y[cnt]
            self.m._accel.data = in_accel[cnt]
            self.m._mag.data = in_mag[cnt]
            self.m._temp.data = in_temp[cnt]
            self.m._pres.data = in_pres[cnt]
            self.m._fltMode.data = in_fltMode[cnt]
            self.m._state.data = in_state[cnt]
            self.m._sysErr.data = in_sysErr[cnt]
            # self.m._vehicle.raw_imu = in_rawIMU[cnt]
            self.m._vehicle.raw_imu.xacc = in_rawIMU_xacc[cnt]
            self.m._vehicle.raw_imu.yacc = in_rawIMU_yacc[cnt]
            self.m._vehicle.raw_imu.zacc = in_rawIMU_zacc[cnt]
            self.m._vehicle.raw_imu.xgyro = in_rawIMU_xgyro[cnt]
            self.m._vehicle.raw_imu.ygyro = in_rawIMU_ygyro[cnt]
            self.m._vehicle.raw_imu.zgyro = in_rawIMU_zgyro[cnt]
            self.m._vehicle.raw_imu.xmag = in_rawIMU_xmag[cnt]
            self.m._vehicle.raw_imu.ymag = in_rawIMU_ymag[cnt]
            self.m._vehicle.raw_imu.zmag = in_rawIMU_zmag[cnt]
            
            self.m._set_db()
            self.m._write_db()

            self.assertEqual(self.m._d_outDroneDB.robotID.data, in_robotID[cnt])
            self.assertEqual(self.m._d_outDroneDB.homeGPS.latitude, in_gps_home_lat[cnt])
            self.assertEqual(self.m._d_outDroneDB.homeGPS.longitude, in_gps_home_lon[cnt])
            self.assertEqual(self.m._d_outDroneDB.homeGPS.altitude, in_gps_home_alt[cnt])
            self.assertEqual(self.m._d_outDroneDB.droneGPS.latitude, in_gps_drone_lat[cnt])
            self.assertEqual(self.m._d_outDroneDB.droneGPS.longitude, in_gps_drone_lon[cnt])
            self.assertEqual(self.m._d_outDroneDB.droneGPS.altitude, in_gps_drone_alt[cnt])
            self.assertEqual(self.m._d_outDroneDB.battV.data, out_battV[cnt])
            self.assertEqual(self.m._d_outDroneDB.battA.data, in_battA[cnt])
            self.assertEqual(self.m._d_outDroneDB.fltTime.data, in_flttime[cnt])
            self.assertEqual(self.m._d_outDroneDB.gyro.data.r, out_gyro_r[cnt])
            self.assertEqual(self.m._d_outDroneDB.gyro.data.p, out_gyro_p[cnt])
            self.assertEqual(self.m._d_outDroneDB.gyro.data.y, out_gyro_y[cnt])
            self.assertEqual(self.m._d_outDroneDB.accel.data, in_accel[cnt])
            self.assertEqual(self.m._d_outDroneDB.mag.data, in_mag[cnt])
            self.assertEqual(self.m._d_outDroneDB.temp.data, out_temp[cnt])
            self.assertEqual(self.m._d_outDroneDB.pres.data, out_pres[cnt])
            self.assertEqual(self.m._d_outDroneDB.fltMode.data, in_fltMode[cnt])
            self.assertEqual(self.m._d_outDroneDB.state.data, in_state[cnt])
            self.assertEqual(self.m._d_outDroneDB.sysErr.data, in_sysErr[cnt])
            self.assertEqual(self.m._d_outDroneDB.rawIMU.data, in_rawIMU[cnt])
            self.assertEqual(self.m._d_outDroneDB.correctIMU.data, out_correctIMU[cnt])


    def test_write_view(self):
        
        vv = Mock()
        self.m._vehicle = vv.vehicle
        
        in_read_gps_home = [False, True]
        in_read_gps_drone = [False, True]
        in_read_battV = [False, True]
        in_read_battA = [False, True]
        in_read_gyro = [False, True]
        in_read_accel = [False, True]
        in_read_mag = [False, True]
        in_read_temp = [False, True]
        in_read_pres = [False, True]

        in_robotID= ["drone1", "drone1"]
        in_gps_home_lat = [100.0, 37.522530]
        in_gps_home_lon = [200.0, 139.938615]
        in_gps_home_alt = [0.0, 216.650000]
        in_gps_drone_lat = [100.0, 37.522532]
        in_gps_drone_lon = [200.0, 139.938613]
        in_gps_drone_alt = [0.0, 9.990000]
        in_battV = [0.0, 15.559000]
        in_battA = [0.0, 0.0]
        in_flttime = [0, 16]
        in_gyro_r = [0, -0.00975905917585]
        in_gyro_p = [0, 0.0214286409318]
        in_gyro_y = [0, 1.79124319553]
        in_accel = [[0.0, 0.0, 0.0], [-0.06, -0.08, 0.01]]
        in_mag = [0.0, 102.0]
        in_temp = [0.0, 38.48]
        in_pres = [0.0, 980.204772949]
        in_fltMode = ["GUIDED", "GUIDED"]
        in_state = ["Hovering", "Hovering"]
        in_sysErr = ["", ""]

        self._d_outDroneView = RTC.DroneView(RTC.Time(0, 0),
                                            RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                        100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                        0.0, 0, RTC.GPS_FIX_NORMAL),
                                            RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                        100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                        0.0, 0, RTC.GPS_FIX_NORMAL),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0]),
                                            RTC.TimedULong(RTC.Time(0, 0), 0),
                                            RTC.TimedOrientation3D(
                                                RTC.Time(0, 0), RTC.Orientation3D(0.0, 0.0, 0.0)),
                                            RTC.TimedFloatSeq(
                                                RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedString(RTC.Time(0, 0), ""),
                                            RTC.TimedString(RTC.Time(0, 0), ""),
                                            RTC.TimedString(RTC.Time(0, 0), ""))

        for cnt in range(len(in_read_gps_home)):
            self.m._is_read_gps_home = in_read_gps_home[cnt]
            self.m._is_read_gps_drone = in_read_gps_drone[cnt]
            self.m._is_read_battV = in_read_battV[cnt]
            self.m._is_read_battA = in_read_battA[cnt]
            self.m._is_read_gyro = in_read_gyro[cnt]
            self.m._is_read_accel = in_read_accel[cnt]
            self.m._is_read_mag = in_read_mag[cnt]
            self.m._is_read_temp = in_read_temp[cnt]
            self.m._is_read_pres = in_read_pres[cnt]
            
            self.m._RobotID[0] = in_robotID[cnt]
            self.m._gps_home.latitude = in_gps_home_lat[cnt]
            self.m._gps_home.longitude = in_gps_home_lon[cnt]
            self.m._gps_home.altitude = in_gps_home_alt[cnt]
            self.m._gps_drone.latitude = in_gps_drone_lat[cnt]
            self.m._gps_drone.longitude = in_gps_drone_lon[cnt]
            self.m._gps_drone.altitude = in_gps_drone_alt[cnt]
            self.m._batt.data[0] = in_battV[cnt]
            self.m._batt.data[1] = in_battA[cnt]
            self.m._fltTime.data = in_flttime[cnt]
            self.m._gyro.data.r = in_gyro_r[cnt]
            self.m._gyro.data.p = in_gyro_p[cnt]
            self.m._gyro.data.y = in_gyro_y[cnt]
            self.m._accel.data = in_accel[cnt]
            self.m._mag.data = in_mag[cnt]
            self.m._temp.data = in_temp[cnt]
            self.m._pres.data = in_pres[cnt]
            self.m._fltMode.data = in_fltMode[cnt]
            self.m._state.data = in_state[cnt]
            self.m._sysErr.data = in_sysErr[cnt]
            
            self.m._set_view()
            self.m._write_view()

            self.assertEqual(self.m._d_outDroneView.homeGPS.latitude, in_gps_home_lat[cnt])
            self.assertEqual(self.m._d_outDroneView.homeGPS.longitude, in_gps_home_lon[cnt])
            self.assertEqual(self.m._d_outDroneView.homeGPS.altitude, in_gps_home_alt[cnt])
            self.assertEqual(self.m._d_outDroneView.droneGPS.latitude, in_gps_drone_lat[cnt])
            self.assertEqual(self.m._d_outDroneView.droneGPS.longitude, in_gps_drone_lon[cnt])
            self.assertEqual(self.m._d_outDroneView.droneGPS.altitude, in_gps_drone_alt[cnt])
            self.assertEqual(self.m._d_outDroneView.batt.data[0], in_battV[cnt])
            self.assertEqual(self.m._d_outDroneView.batt.data[1], in_battA[cnt])
            self.assertEqual(self.m._d_outDroneView.fltTime.data, in_flttime[cnt])
            self.assertEqual(self.m._d_outDroneView.gyro.data.r, in_gyro_r[cnt])
            self.assertEqual(self.m._d_outDroneView.gyro.data.p, in_gyro_p[cnt])
            self.assertEqual(self.m._d_outDroneView.gyro.data.y, in_gyro_y[cnt])
            self.assertEqual(self.m._d_outDroneView.accel.data, in_accel[cnt])
            self.assertEqual(self.m._d_outDroneView.mag.data, in_mag[cnt])
            self.assertEqual(self.m._d_outDroneView.temp.data, in_temp[cnt])
            self.assertEqual(self.m._d_outDroneView.pres.data, in_pres[cnt])
            self.assertEqual(self.m._d_outDroneView.fltMode.data, in_fltMode[cnt])
            self.assertEqual(self.m._d_outDroneView.state.data, in_state[cnt])
            self.assertEqual(self.m._d_outDroneView.sysErr.data, in_sysErr[cnt])


    def test_is_valid_lat(self):

        data = [-90.1, -90.0, -89.9, 0.0, 89.9, 90.0, 90.1]
        result = [False, True, True, True, True, True, False]

        for i, dat in enumerate(data):
            rtn = self.m._is_valid_lat(dat)
            self.assertEqual(rtn, result[i])

    def test_is_valid_lon(self):

        data = [-180.1, -180.0, -179.9, 0.0, 179.9, 180.0, 180.1]
        result = [False, True, True, True, True, True, False]

        for i, dat in enumerate(data):
            rtn = self.m._is_valid_lon(dat)
            self.assertEqual(rtn, result[i])

    def test_is_valid_alt(self):

        in_sts = [DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_RTL, DroneIO.STS_IN_MOVE, DroneIO.STS_ALT_CHG, DroneIO.STS_HOVERING, DroneIO.STS_YAW_CHG,
                DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_RTL, DroneIO.STS_IN_MOVE, DroneIO.STS_ALT_CHG, DroneIO.STS_HOVERING, DroneIO.STS_YAW_CHG]
        in_alt = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0]
        result = [True, False, False, False, False, False, False,
                True, True, True, True, True, True, True]

        for cnt in range(len(in_sts)):
            self.m._drone_sts = in_sts[cnt]
            rtn = self.m._is_valid_alt(in_alt[cnt])
            self.assertEqual(rtn, result[cnt])

    def test_is_valid_yaw_deg(self):

        data = [-0.1, 0.0, 0.1, 90.0, 359.9, 360.0, 360.1]
        result = [False, True, True, True, True, True, False]

        for i, dat in enumerate(data):
            rtn = self.m._is_valid_yaw_deg(dat)
            self.assertEqual(rtn, result[i])

    def test_ready_to_log_write(self):
        
        self.m._d_outDroneCtrl = RTC.DroneCtrl(RTC.Time(1513041940, 0),
                                             0,
                                             RTC.TimedString(RTC.Time(0, 0), ""),
                                             RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                             RTC.TimedFloat(RTC.Time(0, 0), 0),
                                             RTC.TimedString(RTC.Time(0, 0), "Armed"))
        
        in_file_md = [DroneIO.FILE_MODE_WRITE, DroneIO.FILE_MODE_ADD_WRITE]
        self.m._file_name = ""
        self.m._file = None
        self.m._writer = None
        self.m._act_start_date, self.m._act_start_time = self.m._get_datetime_to_str()
        self.m._RobotID[0] = "drone1"
        self.m._fltMode.data = "GUIDED"
        self.m._log_row = ["NULL"] * len(self.m._header)
        self.m._init_read_info()

        for cnt in range(len(in_file_md)):
            self.m._ready_to_log_write(in_file_md[cnt])
            self.m._close_rtc_viewer_log()
            self.assertEqual(self.m._file, None)
            self.assertEqual(self.m._writer, None)

    def test_is_open_rtc_viewer_log(self):

        in_file_md = [DroneIO.FILE_MODE_WRITE, DroneIO.FILE_MODE_WRITE, DroneIO.FILE_MODE_WRITE, DroneIO.FILE_MODE_ADD_WRITE, DroneIO.FILE_MODE_ADD_WRITE, DroneIO.FILE_MODE_ADD_WRITE]
        in_file_name = ["", "20171211_144623_drone1_log", "20171211_144623_drone1_log.csv", "", "20171212_104554_drone1_log", "20171212_104554_drone1_log.csv"]
        out_rtn = [False, True, True, False, True, True]

        for cnt in range(len(in_file_md)):
            self.m._file_name = in_file_name[cnt]
            rtn = self.m._is_open_rtc_viewer_log(in_file_md[cnt])
            self.assertEqual(rtn, out_rtn[cnt])

    def test_is_csv_writer(self):

        in_file = [None] 
        in_writer = [None] 
        out_rtn = [False]

        for cnt in range(len(in_file)):
            self.m._file = in_file[cnt]
            self.m._writer = in_writer[cnt]
            rtn = self.m._is_csv_writer()

    def test_conv_rtctime_to_datetime(self):
        
        tm = RTC.Time(1512971183, 993842000)
        date, time = self.m._conv_rtctime_to_datetime(tm)
        self.assertEqual(date, "20171211")
        self.assertEqual(time, "14:46:23.993")

    def test_set_rcv_log(self):

        init_pos = RTC.DroneTarget(RTC.Time(0, 0),
                                    0,
                                    RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                0.0, 0, RTC.GPS_FIX_NORMAL),
                                    RTC.TimedFloat(RTC.Time(0, 0), 0),
                                    RTC.TimedShortSeq(RTC.Time(0, 0), [0, 0]))

        pos1 = RTC.DroneTarget(RTC.Time(1512971183, 993842000),
                                1,
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9389993, 10.0, 
                                            0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0, 0), 3.0),
                                RTC.TimedShortSeq(RTC.Time(0, 0), [1, 90]))

        in_rtc_state = ["Inactive", "Error", "Exit", "Active", "Active"]
        in_pos = [init_pos, init_pos, init_pos, init_pos, pos1]

        out_rcvdate = ["NULL", "NULL", "NULL", "NULL", "20171211"]
        out_rcvtime = ["NULL", "NULL", "NULL", "NULL", "14:46:23.993"]
        out_robotid = ["NULL", "NULL", "NULL", "NULL", "GCS"]
        out_id = ["NULL", "NULL", "NULL", "NULL", 1]
        out_lat = ["NULL", "NULL", "NULL", "NULL", "37.5223680000"]
        out_lon = ["NULL", "NULL", "NULL", "NULL", "139.9389993000"]
        out_alt = ["NULL", "NULL", "NULL", "NULL", "10.00"]
        out_spd = ["NULL", "NULL", "NULL", "NULL", "3.00"]
        out_yaw_dir = ["NULL", "NULL", "NULL", "NULL", 1]
        out_yaw_deg = ["NULL", "NULL", "NULL", "NULL", 90]

        for cnt in range(len(in_rtc_state)):
            self.m._d_inTarget = in_pos[cnt]
            self.m._log_row = ["NULL"] * len(self.m._header)

            self.m._set_rcv_log(in_rtc_state[cnt])

            self.assertEqual(self.m._log_row[self.m._header.index("RcvDate")], out_rcvdate[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RcvTime")], out_rcvtime[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RcvRobotID")], out_robotid[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RcvID")], out_id[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetLat")], out_lat[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetLon")], out_lon[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetAlt")], out_alt[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetSpd")], out_spd[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetYawDir")], out_yaw_dir[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetYawDeg")], out_yaw_deg[cnt])


    def test_set_snd_log(self):

        init_ctrl = RTC.DroneCtrl(RTC.Time(0, 0),
                                0,
                                RTC.TimedString(RTC.Time(0, 0), ""),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                RTC.TimedString(RTC.Time(0, 0), ""))

        ctrl1 = RTC.DroneCtrl(RTC.Time(1512971183, 993842000),
                                1,
                                RTC.TimedString(RTC.Time(0, 0), "drone1"),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.5223693848, 139.9389993, 208.09, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9387618, 9.99, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.19, 0.0]),
                                RTC.TimedFloat(RTC.Time(0, 0), 89.0),
                                RTC.TimedString(RTC.Time(0, 0), "In_Move"))

        in_rtc_state = ["Inactive", "Error", "Exit", "Active", "Active"]
        in_ctrl = [ctrl1, ctrl1, init_ctrl, ctrl1, ctrl1]
        in_read_gps_home = [False, False, False, False, True]
        in_read_gps_drone = [False, False, False, False, True]
        in_read_accel = [False, False, False, False, True]
        in_read_mag = [False, False, False, False, True]

        out_snddate = ["20171211", "20171211", "NULL", "20171211", "20171211"]
        out_sndtime = ["14:46:23.993", "14:46:23.993", "NULL", "14:46:23.993", "14:46:23.993"]
        out_robotid = ["GCS", "GCS", "NULL", "GCS", "GCS"]
        out_id = [1, 1, "NULL", 1, 1]
        out_state = ["In_Move", "In_Move", "NULL", "In_Move", "In_Move"]
        out_h_lat = ["NULL", "NULL", "NULL", "NULL", "37.5223693848"]
        out_h_lon = ["NULL", "NULL", "NULL", "NULL", "139.9389993000"]
        out_h_alt = ["NULL", "NULL", "NULL", "NULL", "208.09"]
        out_dr_lat = ["NULL", "NULL", "NULL", "NULL", "37.5223680000"]
        out_dr_lon = ["NULL", "NULL", "NULL", "NULL", "139.9387618000"]
        out_dr_alt = ["NULL", "NULL", "NULL", "NULL", "9.99"]
        out_accx = ["NULL", "NULL", "NULL", "NULL", "0.00"]
        out_accy = ["NULL", "NULL", "NULL", "NULL", "0.19"]
        out_accz = ["NULL", "NULL", "NULL", "NULL", "0.00"]
        out_mag = ["NULL", "NULL", "NULL", "NULL", 89]

        for cnt in range(len(in_rtc_state)):
            self.m._d_outDroneCtrl = in_ctrl[cnt]
            self.m._is_read_gps_home = in_read_gps_home[cnt]
            self.m._is_read_gps_drone = in_read_gps_drone[cnt]
            self.m._is_read_accel = in_read_accel[cnt]
            self.m._is_read_mag = in_read_mag[cnt]
            self.m._log_row = ["NULL"] * len(self.m._header)

            self.m._set_snd_log(in_rtc_state[cnt])

            self.assertEqual(self.m._log_row[self.m._header.index("SndDate")], out_snddate[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("SndTime")], out_sndtime[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("SndRobotID")], out_robotid[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("SndID")], out_id[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("State")], out_state[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("HomeLat")], out_h_lat[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("HomeLon")], out_h_lon[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("HomeAlt")], out_h_alt[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RobotLat")], out_dr_lat[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RobotLon")], out_dr_lon[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RobotAlt")], out_dr_alt[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("AccX")], out_accx[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("AccY")], out_accy[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("AccZ")], out_accz[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("Mag")], out_mag[cnt])

    def test_set_sensor_log(self):

        in_rtc_state = ["Inactive", "Error", "Error", "Exit", "Active", "Active"]
        in_read_batt = [False, False, True, False, False, True]
        in_read_gyro = [False, False, True, False, False, True]
        in_read_temp = [False, False, True, False, False, True]
        in_read_pres = [False, False, True, False, False, True]
        in_batt = [0.0, 0.0, 15.753, 0.0, 0.0, 15.753]
        in_gyro = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [-0.00214, -0.08935, 1.57126], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [-0.00214, -0.08935, 1.57126]]
        in_temp = [0.0, 0.0, 28.513, 0.0, 0.0, 28.513]
        in_pres = [0.0, 0.0, 987.38402, 0.0, 0.0, 987.38402]
        out_batt = ["NULL", "NULL", "15.75", "NULL", "NULL", "15.75"]
        out_gyro = [["NULL", "NULL", "NULL"], ["NULL", "NULL", "NULL"], ["-0.002", "-0.089", "1.571"], ["NULL", "NULL", "NULL"], ["NULL", "NULL", "NULL"], ["-0.002", "-0.089", "1.571"]]
        out_temp = ["NULL", "NULL", "28.51", "NULL", "NULL", "28.51"]
        out_pres = ["NULL", "NULL", "987.38", "NULL", "NULL", "987.38"]

        for cnt in range(len(in_rtc_state)):
            self.m._is_read_battV = in_read_batt[cnt]
            self.m._is_read_gyro = in_read_gyro[cnt]
            self.m._is_read_temp = in_read_temp[cnt]
            self.m._is_read_pres = in_read_pres[cnt]
            self.m._batt.data[0] = in_batt[cnt]
            self.m._gyro.data.r, self.m._gyro.data.p, self.m._gyro.data.y = in_gyro[cnt][0], in_gyro[cnt][1], in_gyro[cnt][2]
            self.m._temp.data = in_temp[cnt]
            self.m._pres.data = in_pres[cnt]
            self.m._log_row = ["NULL"] * len(self.m._header)

            self.m._set_sensor_log(in_rtc_state[cnt])

            self.assertEqual(self.m._log_row[self.m._header.index("BattV")], out_batt[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("GyroR")], out_gyro[cnt][0])
            self.assertEqual(self.m._log_row[self.m._header.index("GyroP")], out_gyro[cnt][1])
            self.assertEqual(self.m._log_row[self.m._header.index("GyroY")], out_gyro[cnt][2])
            self.assertEqual(self.m._log_row[self.m._header.index("Temp")], out_temp[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("Pres")], out_pres[cnt])

    def test_set_other_log(self):

        in_rtc_state = ["Inactive", "Active", "Error", "Exit"]
        in_err_inf = ["", "", "Communication with the GCS has been lost.", ""]
        in_fltmode = ["Disarm", "Hovering", "RTL", "Disarm"]
        out_err_inf = ["NULL", "NULL", "Communication with the GCS has been lost.", "NULL"]

        for cnt in range(len(in_rtc_state)):
            self.m._fltMode.data = in_fltmode[cnt]
            self.m._sysErr.data = in_err_inf[cnt]
            self.m._RobotID[0] = "drone1"
            self.m._act_start_date, self.m._act_start_time = "20171211", "14:46:23.993"
            self.m._log_row = ["NULL"] * len(self.m._header)

            self.m._set_other_log(in_rtc_state[cnt])

            self.assertEqual(self.m._log_row[self.m._header.index("Category")], "drone1")
            self.assertEqual(self.m._log_row[self.m._header.index("ActStartDate")], "20171211")
            self.assertEqual(self.m._log_row[self.m._header.index("ActStartTime")], "14:46:23.993")
            self.assertEqual(self.m._log_row[self.m._header.index("FltMode")], in_fltmode[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RTCState")], in_rtc_state[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("Err")], out_err_inf[cnt])


if __name__ == '__main__':
    unittest.main()
