#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DroneMapViewer.py
 @brief Display the position of the drone.
 @date $Date$

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import sys

# Import RTM module
import RTC
import OpenRTM_aist

# Import map_create
import map_create

# Setup logger
from logging import getLogger, StreamHandler
from logging import DEBUG as LOG_LV

sys.path.append(".")

logger = getLogger(__name__)
handler = StreamHandler()
logger.setLevel(LOG_LV)
handler.setLevel(LOG_LV)
logger.addHandler(handler)

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
dronemapviewer_spec = ["implementation_id", "DroneMapViewer", 
         "type_name",         "DroneMapViewer", 
         "description",       "Display the position of the drone.", 
         "version",           "1.1.0", 
         "vendor",            "EAC", 
         "category",          "Drone", 
         "activity_type",     "STATIC", 
         "max_instance",      "1", 
         "language",          "Python", 
         "lang_type",         "SCRIPT",
         "conf.default.MapZoomLevel", "18",
         "conf.default.DroneNum", "3",
         "conf.default.VirtualLeader", "on",
         "conf.default.VirtualLeaderLocus", "on",
         "conf.__widget__.MapZoomLevel", "spin",
         "conf.__widget__.DroneNum", "spin",
         "conf.__widget__.VirtualLeader", "radio",
         "conf.__widget__.VirtualLeaderLocus", "radio",
         "conf.__constraints__.MapZoomLevel", "0<=x<=19",
         "conf.__constraints__.DroneNum", "1<=x<=3",
         "conf.__constraints__.VirtualLeader", "(off,on)",
         "conf.__constraints__.VirtualLeaderLocus", "(off,on)",
         ""]
# </rtc-template>

##
# @class DroneMapViewer
# @brief Display the position of the drone.
# 
# Refer to the "機能仕様書_DroneMapViewer" for details.
# 
# Refer to the "機能仕様書_DroneMapViewer" for details.
# 
# Refer to the "機能仕様書_DroneMapViewer" for details.
# 
# Refer to the "機能仕様書_DroneMapViewer" for details.
# 
# 
class DroneMapViewer(OpenRTM_aist.DataFlowComponentBase):
    
    ##
    # @brief constructor
    # @param manager Maneger Object
    # 
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_inVirtualLeader = RTC.DroneMap(RTC.Time(0,0),
                                               RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                           0.0, 0, RTC.GPS_FIX_NORMAL),
                                               RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                           0.0, 0, RTC.GPS_FIX_NORMAL),
                                               RTC.TimedFloat(RTC.Time(0,0),0),
                                               RTC.TimedString(RTC.Time(0,0),0))
        """
        マップ上に仮想リーダー機の現在地を表示するための情報。以下の情報をパッケージ
        。
        ・ドローン現在地座標
        ・ターゲット座標
        ・磁気コンパス
        ・ドローンI/Oの内部ステータス
         - Type: RTC::DroneMap
        """
        self._inVirtualLeaderIn = OpenRTM_aist.InPort("inVirtualLeader", self._d_inVirtualLeader)
        self._d_inDrone1 = RTC.DroneMap(RTC.Time(0,0),
                                        RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.TimedFloat(RTC.Time(0,0),0),
                                        RTC.TimedString(RTC.Time(0,0),0))
        """
        マップ上にドローン1号機の現在地を表示するための情報。以下の情報をパッケージ。
        ・ドローン現在地座標
        ・ターゲット座標
        ・磁気コンパス
        ・ドローンI/Oの内部ステータス
         - Type: RTC::DroneMap
        """
        self._inDrone1In = OpenRTM_aist.InPort("inDrone1", self._d_inDrone1)
        self._d_inDrone2 = RTC.DroneMap(RTC.Time(0,0),
                                        RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.TimedFloat(RTC.Time(0,0),0),
                                        RTC.TimedString(RTC.Time(0,0),0))
        """
        マップ上にドローン2号機の現在地を表示するための情報。以下の情報をパッケージ。
        ・ドローン現在地座標
        ・ターゲット座標
        ・磁気コンパス
        ・ドローンI/Oの内部ステータス
         - Type: RTC::DroneMap
        """
        self._inDrone2In = OpenRTM_aist.InPort("inDrone2", self._d_inDrone2)
        self._d_inDrone3 = RTC.DroneMap(RTC.Time(0,0),
                                        RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.TimedFloat(RTC.Time(0,0),0),
                                        RTC.TimedString(RTC.Time(0,0),0))
        """
        マップ上にドローン3号機の現在地を表示するための情報。以下の情報をパッケージ。
        ・ドローン現在地座標
        ・ターゲット座標
        ・磁気コンパス
        ・ドローンI/Oの内部ステータス
         - Type: RTC::DroneMap
        """
        self._inDrone3In = OpenRTM_aist.InPort("inDrone3", self._d_inDrone3)


        


        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
        """
        地図の縮尺レベル
         - Name: MapZoomLevel MapZoomLevel
         - DefaultValue: 18
         - Range: 0<=x<=19
        """
        self._MapZoomLevel = [18]
        """
        表示するドローンの数
         - Name: DroneNum DroneNum
         - DefaultValue: 3
         - Range: 1<=x<=3
        """
        self._DroneNum = [3]
        """
        仮想リーダー機の表示の有無
         - Name: VirtualLeader VirtualLeader
         - DefaultValue: on
         - Range: (off, on)
        """
        self._VirtualLeader = ['on']
        """
        仮想リーダー機の軌跡表示の有無
         - Name: VirtualLeaderLocus VirtualLeaderLocus
         - DefaultValue: on
         - Range: (off, on)
        """
        self._VirtualLeaderLocus = ['on']
        
        # </rtc-template>

        inport_num = 3
        self._d_inDrone = [0] * inport_num
        self._inDroneIn = [0] * inport_num

        self._d_inDrone[0] = self._d_inDrone1
        self._inDroneIn[0] = self._inDrone1In
        self._d_inDrone[1] = self._d_inDrone2
        self._inDroneIn[1] = self._inDrone2In
        self._d_inDrone[2] = self._d_inDrone3
        self._inDroneIn[2] = self._inDrone3In

        self._virtual_leader = False
        self._drone = [False] * self._DroneNum[0]

        self.drone_num_set = map_create.DroneNumSet()
        self.virtual_leader_disp = map_create.VirtualLeaderDisp()
        self.virtual_leader_locus = map_create.VirtualLeaderLocus()
        self.virtual_leader_update = map_create.VirtualLeaderUpdate()
        self.drone_pos_update = map_create.DronePosUpdate()
        self.drone_angle_update = map_create.DroneAngleUpdate()
        self.target_pos_update = map_create.TargetPosUpdate()
        self.drone_state_update = map_create.DroneStateUpdate()
        self.map_zoom_update = map_create.MapZoomUpdate()

    ##
    # RTC初期化
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry() 
    # 
    # @return RTC::ReturnCode_t
    # 
    #
    def onInitialize(self):
        # Bind variables and configuration variable
        self.bindParameter("MapZoomLevel", self._MapZoomLevel, "18")
        self.bindParameter("DroneNum", self._DroneNum, "3")
        self.bindParameter("VirtualLeader", self._VirtualLeader, "on")
        self.bindParameter("VirtualLeaderLocus", self._VirtualLeaderLocus, "on")
        
        # Set InPort buffers
        self.addInPort("inVirtualLeader",self._inVirtualLeaderIn)
        self.addInPort("inDrone1",self._inDrone1In)
        self.addInPort("inDrone2",self._inDrone2In)
        self.addInPort("inDrone3",self._inDrone3In)
        
        # Set OutPort buffers
        
        # Set service provider to Ports
        
        # Set service consumers to Ports
        
        # Set CORBA Service Ports
        
        return RTC.RTC_OK
    
    #    ##
    #    # 
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    # 
    #    # @return RTC::ReturnCode_t
    #
    #    # 
    #def onFinalize(self):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    # 
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK
    
        ##
        # ドローン台数の決定
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        # 
        # @return RTC::ReturnCode_t
        #
        #
    def onActivated(self, ec_id):
        self._Drone_Num = self._DroneNum[0]
        self.drone_num_set.drone_num(self._Drone_Num)

        return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The deactivated action (Active state exit action)
    #    # former rtc_active_exit()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onDeactivated(self, ec_id):
    #
    #    return RTC.RTC_OK
    
        ##
        # ドローンGPS、MAGデータ取得およびマップビューアの表示
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onExecute(self, ec_id):
        self._read_inport()

        self._set_map_data()

        return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The aborting action when main logic error occurred.
    #    # former rtc_aborting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onAborting(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onError(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The reset action that is invoked resetting
    #    # This is same but different the former rtc_init_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onReset(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    #def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    def _read_inport(self):
        self._virtual_leader = False
        self._drone = [False] * self._Drone_Num

        # virtual leader
        if self._inVirtualLeaderIn.isNew():
            self._d_inVirtualLeader = self._inVirtualLeaderIn.read()
            self._virtual_leader = True

        # drone
        num = 0
        while num < self._Drone_Num:
            # read_map_data
            if self._inDroneIn[num].isNew():
                self._d_inDrone[num] = self._inDroneIn[num].read()
                self._drone[num] = True
                #logger.debug("Drone%dGPS:  % f % f % f",
                #             (num + 1),
                #             self._d_inDrone[num].droneGPS.latitude,
                #             self._d_inDrone[num].droneGPS.longitude,
                #             self._d_inDrone[num].droneGPS.altitude)


                #logger.debug("Target%dGPS:   % f % f % f",
                #             (num + 1),
                #             self._d_inDrone[num].targetGPS.latitude,
                #             self._d_inDrone[num].targetGPS.longitude,
                #             self._d_inDrone[num].targetGPS.altitude)

                #logger.debug("Mag%d: %d",
                #             (num + 1),
                #             self._d_inDrone[num].mag.data)

                #logger.debug("State%d: %s",
                #             (num + 1),
                #             self._d_inDrone[num].state.data)

            num += 1
 
    def _set_map_data(self):
        # config
        self.map_zoom_update.map_zoom(self._MapZoomLevel[0])
        self.virtual_leader_disp.virtual_leader_disp(self._VirtualLeader[0])
        self.virtual_leader_locus.virtual_leader_locus(self._VirtualLeaderLocus[0])

        # virtual leader
        if self._virtual_leader:
            self.virtual_leader_update.virtual_leader(self._d_inVirtualLeader)

        # drone
        num = 0
        while num < self._Drone_Num:
            if self._drone[num]:
                self.drone_pos_update.drone_move(self._d_inDrone[num].droneGPS.latitude, 
                                                 self._d_inDrone[num].droneGPS.longitude, 
                                                 self._d_inDrone[num].droneGPS.altitude, num)

                self.target_pos_update.drone_target(
                    self._d_inDrone[num].targetGPS.latitude, self._d_inDrone[num].targetGPS.longitude, num)

                self.drone_angle_update.drone_rotate(
                    self._d_inDrone[num].mag.data, num)
             
                self.drone_state_update.drone_state(self._d_inDrone[num].state.data, num)
            num += 1

def DroneMapViewerInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=dronemapviewer_spec)
    manager.registerFactory(profile,
                            DroneMapViewer,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    DroneMapViewerInit(manager)

    # Create a component
    comp = manager.createComponent("DroneMapViewer")

def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager(True)
    map_create.MapCreateApp().run()

if __name__ == "__main__":
    main()

