#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DroneMapViewer_test.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import unittest

import RTC
import OpenRTM_aist
import DroneMapViewer

import sys

from mock import Mock
from mock import patch, MagicMock

INPORT_SET_NUM = 3
DRONE_NUM = 3 #制御するドローン数

class TestDroneMapViewer(unittest.TestCase):
    def setUp(self):
        self.m = DroneMapViewer.DroneMapViewer(0)

        self.xx_logger = Mock()
        DroneMapViewer.logger.warning = self.xx_logger.warning
        DroneMapViewer.logger.error = self.xx_logger.error
        DroneMapViewer.logger.info = self.xx_logger.info
        DroneMapViewer.logger.debug = self.xx_logger.debug

    def test_read_inport(self):
        xxL = Mock()
        xx1 = Mock()
        xx2 = Mock()
        xx3 = Mock()
        self.m._inVirtualLeaderIn.isNew = xxL.isNew
        self.m._inVirtualLeaderIn.read = xxL.read
        self.m._inDroneIn[0].isNew = xx1.isNew
        self.m._inDroneIn[0].read = xx1.read
        self.m._inDroneIn[1].isNew = xx2.isNew
        self.m._inDroneIn[1].read = xx2.read
        self.m._inDroneIn[2].isNew = xx3.isNew
        self.m._inDroneIn[2].read = xx3.read

        init_drone = RTC.DroneMap(RTC.Time(0,0),
                                        RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.TimedFloat(RTC.Time(0,0),0),
                                        RTC.TimedString(RTC.Time(0,0),0))

        virtual_leader = RTC.DroneMap(RTC.Time(0,0),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               36.5, 137.5, 10.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               36.6, 137.6, 11.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.TimedFloat(RTC.Time(0,0),40),
                                   RTC.TimedString(RTC.Time(0,0),"Armed"))

        drone1 = RTC.DroneMap(RTC.Time(0,0),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               37.0, 138.0, 215.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               37.1, 138.1, 10.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.TimedFloat(RTC.Time(0,0),45),
                                   RTC.TimedString(RTC.Time(0,0),"Hovering"))

        drone2 = RTC.DroneMap(RTC.Time(0,0),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               37.5, 138.5, 215.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               37.6, 138.6, 10.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.TimedFloat(RTC.Time(0,0),50),
                                   RTC.TimedString(RTC.Time(0,0),"Alt_CHG"))

        drone3 = RTC.DroneMap(RTC.Time(0,0),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               38.0, 139.0, 215.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                               38.1, 139.1, 10.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.TimedFloat(RTC.Time(0,0),55),
                                   RTC.TimedString(RTC.Time(0,0),"In_Move"))

        drone_lat_result = [0] * INPORT_SET_NUM
        drone_lon_result = [0] * INPORT_SET_NUM
        drone_alt_result = [0] * INPORT_SET_NUM
        target_lat_result = [0] * INPORT_SET_NUM
        target_lon_result = [0] * INPORT_SET_NUM
        target_alt_result = [0] * INPORT_SET_NUM
        mag_result = [0] * INPORT_SET_NUM
        state_result = [0] * INPORT_SET_NUM

        self.m._d_inVirtualLeader = init_drone
        self.m._d_inDrone[0] = init_drone
        self.m._d_inDrone[1] = init_drone
        self.m._d_inDrone[2] = init_drone
        xxL.read.return_value = virtual_leader
        xx1.read.return_value = drone1
        xx2.read.return_value = drone2
        xx3.read.return_value = drone3

        in_is_new = [False, True]
        virtual_leader_pos_lat_result = [0, 36.5]
        virtual_leader_pos_lon_result = [0, 137.5]
        virtual_leader_pos_alt_result = [0, 10.0]
        virtual_leader_target_lat_result = [0, 36.6]
        virtual_leader_target_lon_result = [0, 137.6]
        virtual_leader_target_alt_result = [0, 11.0]
        virtual_leader_mag_result = [0, 40]
        virtual_leader_state_result = [0, "Armed"]
        drone_lat_result[0] = [0, 37.0]
        drone_lon_result[0] = [0, 138.0]
        drone_alt_result[0] = [0, 215.0]
        target_lat_result[0] = [0, 37.1]
        target_lon_result[0] = [0, 138.1]
        target_alt_result[0] = [0, 10.0]
        mag_result[0] = [0, 45]
        state_result[0] = [0, "Hovering"]
        drone_lat_result[1] = [0, 37.5]
        drone_lon_result[1] = [0, 138.5]
        drone_alt_result[1] = [0, 215.0]
        target_lat_result[1] = [0, 37.6]
        target_lon_result[1] = [0, 138.6]
        target_alt_result[1] = [0, 10.0]
        mag_result[1] = [0, 50]
        state_result[1] = [0, "Alt_CHG"]
        drone_lat_result[2] = [0, 38.0]
        drone_lon_result[2] = [0, 139.0]
        drone_alt_result[2] = [0, 215.0]
        target_lat_result[2] = [0, 38.1]
        target_lon_result[2] = [0, 139.1]
        target_alt_result[2] = [0, 10.0]
        mag_result[2] = [0, 55]
        state_result[2] = [0, "In_Move"]

        self.m._Drone_Num = DRONE_NUM
        for i,d in enumerate(in_is_new):
            xxL.isNew.return_value = in_is_new[i]
            xx1.isNew.return_value = in_is_new[i]
            xx2.isNew.return_value = in_is_new[i]
            xx3.isNew.return_value = in_is_new[i]
            self.m._read_inport()

            # virtual leader
            self.assertEqual(self.m._d_inVirtualLeader.droneGPS.latitude, virtual_leader_pos_lat_result[i])
            self.assertEqual(self.m._d_inVirtualLeader.droneGPS.longitude, virtual_leader_pos_lon_result[i])
            self.assertEqual(self.m._d_inVirtualLeader.droneGPS.altitude, virtual_leader_pos_alt_result[i])
            self.assertEqual(self.m._d_inVirtualLeader.targetGPS.latitude, virtual_leader_target_lat_result[i])
            self.assertEqual(self.m._d_inVirtualLeader.targetGPS.longitude, virtual_leader_target_lon_result[i])
            self.assertEqual(self.m._d_inVirtualLeader.targetGPS.altitude, virtual_leader_target_alt_result[i])
            self.assertEqual(self.m._d_inVirtualLeader.mag.data, virtual_leader_mag_result[i])
            self.assertEqual(self.m._d_inVirtualLeader.state.data, virtual_leader_state_result[i])

            # drone
            for drone_num in range(DRONE_NUM):
                self.assertEqual(self.m._d_inDrone[drone_num].droneGPS.latitude, drone_lat_result[drone_num][i])
                self.assertEqual(self.m._d_inDrone[drone_num].droneGPS.longitude, drone_lon_result[drone_num][i])
                self.assertEqual(self.m._d_inDrone[drone_num].droneGPS.altitude, drone_alt_result[drone_num][i])
                self.assertEqual(self.m._d_inDrone[drone_num].targetGPS.latitude, target_lat_result[drone_num][i])
                self.assertEqual(self.m._d_inDrone[drone_num].targetGPS.longitude, target_lon_result[drone_num][i])
                self.assertEqual(self.m._d_inDrone[drone_num].targetGPS.altitude, target_alt_result[drone_num][i])
                self.assertEqual(self.m._d_inDrone[drone_num].mag.data, mag_result[drone_num][i])
                self.assertEqual(self.m._d_inDrone[drone_num].state.data, state_result[drone_num][i])

    """
    def test_read_inport(self):

        m_map = [0] * INPORT_SET_NUM

        num = 0
        while num < INPORT_SET_NUM:
            m_map[num] = Mock()
            self.m._inDroneIn[num].isNew = m_map[num].isNew
            num += 1

        # inport:
        map_is_new = [0] * INPORT_SET_NUM
        map_is_new[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        map_is_new[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        map_is_new[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]

        

        # result:
        map_result = [0] * INPORT_SET_NUM
        map_result[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        map_result[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        map_result[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]

        #
        count = 1
        elem = 0
        drone_num = 1
        while drone_num < 4:
            # conf:
            self.m._Drone_Num = drone_num
            count = (2 * count)
            i = 0
            while i < count:

                num = 0
                while num < self.m._Drone_Num:
                    m_map[num].isNew.return_value = map_is_new[num][elem]
                    num += 1

                self.m._read_inport()

                num = 0
                while num < self.m._Drone_Num:
                    self.assertEqual(self.m._map[num], map_result[num][elem])
                    num += 1

                i += 1
                elem += 1
            drone_num += 1
    """
if __name__ == '__main__':
    unittest.main()
