"""
 @file map_create.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import kivy
kivy.require('1.0.8')

from kivy.app import App

from kivy.uix.label import Label
from kivy.uix.widget import Widget

from kivy.uix.gridlayout import GridLayout

from kivy.garden.mapview import MapView, MapMarker
from kivy.graphics import Color, Line, Rectangle, Rotate, PushMatrix, PopMatrix

from kivy.core.window import Window
from kivy.clock import Clock


import time
import math


MAP_SIZE_WIDTH = 880
MAP_SIZE_HEIGHT = 660

READ_CHECK_TIME = 20.0

LOCUS_NUM = 10

class DroneNumSet():

    def drone_num(self, num):
        global drone_num
        drone_num = num
        print "drone_num:",drone_num

class VirtualLeaderDisp():

    def virtual_leader_disp(self, disp):
        global virtual_leader_disp
        virtual_leader_disp = disp

class VirtualLeaderLocus():

    def virtual_leader_locus(self, locus):
        global virtual_leader_locus
        virtual_leader_locus = locus

class VirtualLeaderUpdate():

    def __init__(self):
        global virtual_leader_GPS
        virtual_leader_GPS = False

    def virtual_leader(self, virtualleader):
        global virtual_leader_pos_lat,virtual_leader_pos_lon,virtual_leader_pos_alt
        global virtual_leader_target_lat,virtual_leader_target_lon,virtual_leader_target_alt
        global virtual_leader_mag

        virtual_leader_pos_lat = virtualleader.droneGPS.latitude
        virtual_leader_pos_lon = virtualleader.droneGPS.longitude
        virtual_leader_pos_alt = virtualleader.droneGPS.altitude
        virtual_leader_target_lat = virtualleader.targetGPS.latitude
        virtual_leader_target_lon = virtualleader.targetGPS.longitude
        virtual_leader_target_alt = virtualleader.targetGPS.altitude
        virtual_leader_mag = virtualleader.mag.data

        global virtual_leader_GPS
        virtual_leader_GPS = True

class DronePosUpdate():

    def drone_move(self, lat, lon, alt, no):
        drone_lat[no] = lat
        drone_lon[no] = lon
        drone_alt[no] = alt


class DroneAngleUpdate():

    def drone_rotate(self, angle, no):
        drone_angle[no] = angle


class TargetPosUpdate():

    def drone_target(self, target_lat, target_lon, no):
        drone_target_lat[no] = target_lat
        drone_target_lon[no] = target_lon


class DroneStateUpdate():

    def drone_state(self, state, no):
        drone_state[no] = state


class MapZoomUpdate():

    def map_zoom(self, zoom):
        global map_zoom_level
        map_zoom_level = zoom


class MapDisplay(Widget):

    def __init__(self, **kwargs):
        super(MapDisplay, self).__init__(**kwargs)

        self._calc_c_pos = [False] * drone_num

        self.map_leader_pos_width = 0
        self.map_leader_pos_height = 0
        self.map_leader_target_width = 0
        self.map_leader_target_height = 0

        self.map_drone_width = [0] * drone_num
        self.map_drone_height = [0] * drone_num
        self.map_target_width = [0] * drone_num
        self.map_target_height = [0] * drone_num

        self.angle = [0] * drone_num
        self.mapmarker = [0] * drone_num

        self.locus_lat = [None] * LOCUS_NUM
        self.locus_lon = [None] * LOCUS_NUM
        Clock.schedule_interval(self.locus_update, 3.0)
        self.mapmarker_locus = [None] * LOCUS_NUM
        self.mapmarker_leader = None

        self.all_rtl = False

        self.select_drone_calc_c_pos()
        self.map_center_calc()

        self.mapview = MapView(zoom=map_zoom_level, lat=drone_lat[0], lon=drone_lon[0])
        self.mapview.size = (MAP_SIZE_WIDTH, MAP_SIZE_HEIGHT)
        self.mapview.center_on(self.map_center_lat, self.map_center_lon)

        self.target_update()

        num = 0
        while num < drone_num:
            self.angle[num] = drone_angle[num]

            self.mapmarker[num] = MapMarker(anchor_x=0.5, anchor_y=0.5, lat=drone_lat[num], lon=drone_lon[num],
                                            source="icon/drone%d.png" % (num + 1))

            with self.mapmarker[num].canvas.before:
                PushMatrix()
                Rotate(angle=self.angle[num], axis=(
                    0, 0, -1), origin=(self.map_drone_width[num], self.map_drone_height[num]))

            with self.mapmarker[num].canvas.after:
                PopMatrix()
                Color(rgba=(0.4, 0.8, 0.68, 0.9))
                Line(points=[self.map_drone_width[num], self.map_drone_height[num],
                             self.map_target_width[num], self.map_target_height[num]], width=1.5)

            self.mapview.add_marker(self.mapmarker[num])
            num += 1

        self.add_widget(self.mapview)

    def _is_all_rtl(self):
        all_rtl = False

        for num in range(drone_num):
            if drone_state[num] == "RTL" or drone_state[num] == "Disarm":
                all_rtl = True
            else:
                all_rtl = False
                break

        self.all_rtl = all_rtl


    def select_drone_calc_c_pos(self):
        self._is_all_rtl()
        if self.all_rtl == False:
            num = 0
            while num < drone_num:
                if drone_state[num] != "RTL" and drone_state[num] != "Disarm":
                    self._calc_c_pos[num] = True
                else:
                    self._calc_c_pos[num] = False            
                num += 1

    def map_update(self):
        self.target_update()
        self.icon_update()

        if self.map_center_change():
            print "===========MAP_UPDATE=============="
            self.map_center_calc()

        self.mapview.zoom = map_zoom_level
        self.mapview.center_on(self.map_center_lat, self.map_center_lon)

    def map_center_change(self):
        self.select_drone_calc_c_pos()
        ret = False
        num = 0
        while num < drone_num:
            if self._calc_c_pos[num]:
                if (self.map_drone_height[num] < 100 or self.map_drone_height[num] > (MAP_SIZE_HEIGHT - 100) or
                        self.map_drone_width[num] < 100 or self.map_drone_width[num] > (MAP_SIZE_WIDTH - 100)):
                    ret = True
                    break
            num += 1

        return ret

    def map_center_calc(self):
        lat = 0
        lon = 0
        num = 0
        control_num = 0
        while num < drone_num:
            if self._calc_c_pos[num]:
                lat += drone_lat[num]
                lon += drone_lon[num]
                control_num += 1
            num += 1

        self.map_center_lat = lat/control_num
        self.map_center_lon = lon/control_num

    def icon_update(self):

        # virtualleader
        if virtual_leader_GPS:

            # icon remove
            if self.mapmarker_leader is not None:
                self.mapview.remove_marker(self.mapmarker_leader)
            # locus remove
            for num in range(len(self.locus_lat)):
                if self.mapmarker_locus[num] is not None:
                    self.mapview.remove_marker(self.mapmarker_locus[num])

            if virtual_leader_disp == "on":
                if virtual_leader_locus == "on":
                    if self.all_rtl == False:
                        # locus add
                        for num in range(len(self.locus_lat)):
                            if self.locus_lat[num] is not None:
                                self.mapmarker_locus[num] = MapMarker(anchor_x=0.5, anchor_y=0.5, 
                                                              lat=self.locus_lat[num], lon=self.locus_lon[num],
                                                              source="icon/locus.png")
                                self.mapview.add_marker(self.mapmarker_locus[num])

                # icon image
                if self.all_rtl:
                    virtual_leader_icon = "icon/home.png"
                else:
                    virtual_leader_icon = "icon/leader.png"

                # icon add
                self.mapmarker_leader = MapMarker(anchor_x=0.5, anchor_y=0.5, 
                                                  lat=virtual_leader_pos_lat, lon=virtual_leader_pos_lon,
                                                  source=virtual_leader_icon)
                self.mapview.add_marker(self.mapmarker_leader)

                # target_line and angle update
                self.mapmarker_leader.canvas.before.clear()
                self.mapmarker_leader.canvas.after.clear()
                if self.all_rtl == False:
                    with self.mapmarker_leader.canvas.before:
                        PushMatrix()
                        Rotate(angle=virtual_leader_mag, axis=(
                            0, 0, -1), origin=(self.map_leader_pos_width, self.map_leader_pos_height))

                    with self.mapmarker_leader.canvas.after:
                        PopMatrix()
                        Color(rgba=(0.95, 0.6, 0.0, 1.0))
                        Line(points=[self.map_leader_pos_width, self.map_leader_pos_height,
                                     self.map_leader_target_width, self.map_leader_target_height], width=1.5)

        # drone
        num = 0
        while num < drone_num:

            self.mapmarker[num].lat = drone_lat[num]
            self.mapmarker[num].lon = drone_lon[num]
            self.angle[num] = drone_angle[num]

            self.mapmarker[num].canvas.before.clear()
            self.mapmarker[num].canvas.after.clear()

            with self.mapmarker[num].canvas.before:
                PushMatrix()
                Rotate(angle=self.angle[num], axis=(
                    0, 0, -1), origin=(self.map_drone_width[num], self.map_drone_height[num]))

            with self.mapmarker[num].canvas.after:
                PopMatrix()
                Color(rgba=(0.95, 0.6, 0.0, 1.0))
                Line(points=[self.map_drone_width[num], self.map_drone_height[num],
                             self.map_target_width[num], self.map_target_height[num]], width=1.5)

            num += 1

    def target_update(self):
        min_lon = (180 / math.pow(2, map_zoom_level + 7)) * \
            ((MAP_SIZE_WIDTH / 2) * -1) + self.map_center_lon
        min_lat = (180 / math.pi) * math.asin(math.tanh((math.pi / math.pow(2, map_zoom_level + 7)) * ((MAP_SIZE_HEIGHT / 2) * -1) +
                                                        math.atanh(math.sin((math.pi / 180) * self.map_center_lat))))
        max_lon = (180 / math.pow(2, map_zoom_level + 7)) * \
            ((MAP_SIZE_WIDTH / 2)) + self.map_center_lon
        max_lat = (180 / math.pi) * math.asin(math.tanh((math.pi / math.pow(2, map_zoom_level + 7)) * ((MAP_SIZE_HEIGHT / 2)) +
                                                        math.atanh(math.sin((math.pi / 180) * self.map_center_lat))))

        # virtualleader
        if virtual_leader_GPS:
            self.map_leader_pos_width = (MAP_SIZE_WIDTH / 2 + 
                                         (MAP_SIZE_WIDTH * (virtual_leader_pos_lon - self.map_center_lon)) /
                                         math.fabs(max_lon - min_lon))
            self.map_leader_pos_height = (MAP_SIZE_HEIGHT / 2 +
                                          (MAP_SIZE_HEIGHT * (virtual_leader_pos_lat - self.map_center_lat)) /
                                          math.fabs(max_lat - min_lat))
            self.map_leader_target_width = (MAP_SIZE_WIDTH / 2 + 
                                            (MAP_SIZE_WIDTH * (virtual_leader_target_lon - self.map_center_lon)) /
                                            math.fabs(max_lon - min_lon))
            self.map_leader_target_height = (MAP_SIZE_HEIGHT / 2 +
                                             (MAP_SIZE_HEIGHT * (virtual_leader_target_lat - self.map_center_lat)) /
                                             math.fabs(max_lat - min_lat))
        # drone
        num = 0
        while num < drone_num:
            self.map_drone_width[num] = (MAP_SIZE_WIDTH / 2 + (MAP_SIZE_WIDTH * (drone_lon[num] - self.map_center_lon)) /
                                         math.fabs(max_lon - min_lon))
            self.map_drone_height[num] = (MAP_SIZE_HEIGHT / 2 + (MAP_SIZE_HEIGHT * (drone_lat[num] - self.map_center_lat)) /
                                          math.fabs(max_lat - min_lat))

            self.map_target_width[num] = (MAP_SIZE_WIDTH / 2 + (MAP_SIZE_WIDTH * (drone_target_lon[num] - self.map_center_lon)) /
                                          math.fabs(max_lon - min_lon))
            self.map_target_height[num] = (MAP_SIZE_HEIGHT / 2 + (MAP_SIZE_HEIGHT * (drone_target_lat[num] - self.map_center_lat)) /
                                           math.fabs(max_lat - min_lat))

            num += 1

    def locus_update(self, dt):
        if virtual_leader_GPS:
            # lat update
            self.locus_lat.pop()
            self.locus_lat.insert(0, virtual_leader_pos_lat)

            # lon update
            self.locus_lon.pop()
            self.locus_lon.insert(0, virtual_leader_pos_lon)


class DroneInfo(Widget):

    def __init__(self, **kwargs):
        super(DroneInfo, self).__init__(**kwargs)

        # virtualleader
        self.virtualleader_info = None
        self.virtualleader_info_pos_x = None
        self.virtualleader_info_pos_y = None

        self.virtualleader_info_name = None
        self.virtualleader_info_lat = None
        self.virtualleader_info_lon = None
        self.virtualleader_info_alt = None

        # drone
        self.info_pos_x = [0] * drone_num
        self.info_pos_y = [0] * drone_num

        self.drone_info = [0] * drone_num
        self.info_name = [0] * drone_num
        self.info_lat = [0] * drone_num
        self.info_lon = [0] * drone_num
        self.info_alt = [0] * drone_num

        num = 0
        while num < drone_num:
            self.info_pos_x[num] = MAP_SIZE_WIDTH - 145
            self.info_pos_y[num] = MAP_SIZE_HEIGHT - (115 * (num + 1))

            self.drone_info[num] = Label()
            with self.drone_info[num].canvas.before:
                Color(rgba=(0, 0, 0, 0.5))
                Rectangle(size=(140, 110), pos=(
                    self.info_pos_x[num], self.info_pos_y[num]))

            self.info_name[num] = Label(text="Drone%d" % (num + 1), boil=True, font_size=20, text_size=(180, None),
                                   pos=(self.info_pos_x[num] + 50, self.info_pos_y[num] + 47))
            self.info_lat[num] = Label(text="lat:  %f" % drone_lat[num], font_size=15, text_size=(180, None),
                                       pos=(self.info_pos_x[num] + 60, self.info_pos_y[num] + 17))
            self.info_lon[num] = Label(text="lon: %f" % drone_lon[num], font_size=15, text_size=(180, None),
                                       pos=(self.info_pos_x[num] + 60, self.info_pos_y[num] - 8))
            self.info_alt[num] = Label(text="alt:  %f" % drone_alt[num], font_size=15, text_size=(180, None),
                                       pos=(self.info_pos_x[num] + 60, self.info_pos_y[num] - 33))

            self.drone_info[num].add_widget(self.info_name[num])
            self.drone_info[num].add_widget(self.info_lat[num])
            self.drone_info[num].add_widget(self.info_lon[num])
            self.drone_info[num].add_widget(self.info_alt[num])

            self.add_widget(self.drone_info[num])
            num += 1

    def info_update(self):

        # virtualleader
        if virtual_leader_GPS:

            # info widget remove
            if self.virtualleader_info is not None:
                self.remove_widget(self.virtualleader_info)

            # info widget add
            if virtual_leader_disp == "on":
                self.virtualleader_info_pos_x = MAP_SIZE_WIDTH - 290
                self.virtualleader_info_pos_y = MAP_SIZE_HEIGHT - 115

                self.virtualleader_info = Label()
                with self.virtualleader_info.canvas.before:
                    Color(rgba=(0, 0, 0, 0.5))
                    Rectangle(size=(140, 110), pos=(self.virtualleader_info_pos_x, self.virtualleader_info_pos_y))

                self.virtualleader_info_name = Label(text="Virtual Leader", boil=True,
                                                     font_size=20, text_size=(180, None),
                                                     pos=(self.virtualleader_info_pos_x + 50,
                                                          self.virtualleader_info_pos_y + 47))
                self.virtualleader_info_lat = Label(text="lat:  %f" % virtual_leader_pos_lat,
                                                    font_size=15, text_size=(180, None),
                                                    pos=(self.virtualleader_info_pos_x + 60, 
                                                         self.virtualleader_info_pos_y + 17))
                self.virtualleader_info_lon = Label(text="lon: %f" % virtual_leader_pos_lon,
                                                    font_size=15, text_size=(180, None),
                                                    pos=(self.virtualleader_info_pos_x + 60,
                                                         self.virtualleader_info_pos_y - 8))
                self.virtualleader_info_alt = Label(text="alt:  %f" % virtual_leader_pos_alt,
                                                    font_size=15, text_size=(180, None),
                                                    pos=(self.virtualleader_info_pos_x + 60,
                                                         self.virtualleader_info_pos_y - 33))

                self.virtualleader_info.add_widget(self.virtualleader_info_name)
                self.virtualleader_info.add_widget(self.virtualleader_info_lat)
                self.virtualleader_info.add_widget(self.virtualleader_info_lon)
                self.virtualleader_info.add_widget(self.virtualleader_info_alt)

                self.add_widget(self.virtualleader_info)

        # drone
        num = 0
        while num < drone_num:
            self.info_lat[num].text = "lat:  %f" % drone_lat[num]
            self.info_lon[num].text = "lon: %f" % drone_lon[num]
            self.info_alt[num].text = "alt:  %f" % drone_alt[num]

            num += 1


class MapLoadTimeout(Widget):

    def __init__(self, **kwargs):
        super(MapLoadTimeout, self).__init__(**kwargs)

        self.timeout_msg = Label(text="Map Loaded Timeout", font_size=self.height / 2,
                                 halign="center", pos=(MAP_SIZE_WIDTH / 2.2, MAP_SIZE_HEIGHT / 2.2))

        self.add_widget(self.timeout_msg)


class MapCreateApp(App):

    def __init__(self, **kwargs):
        super(MapCreateApp, self).__init__(**kwargs)
        
        global drone_num
        drone_num = 0
        while True:
            if drone_num > 0:
                break

        global drone_lat, drone_lon, drone_alt, drone_target_lat, drone_target_lon, drone_angle, drone_state
        drone_lat = [0] * drone_num
        drone_lon = [0] * drone_num
        drone_alt = [0] * drone_num
        drone_target_lat = [0] * drone_num
        drone_target_lon = [0] * drone_num
        drone_angle = [0] * drone_num
        drone_state = ["None"] * drone_num

        check_start = time.time()
        self.timeout = False
        num = 0
        while num < drone_num:
            if (drone_lat[num] and drone_lon[num] and drone_alt[num] and drone_angle[num] and
                    drone_target_lat[num] and drone_target_lon[num]):
                num += 1

            if (time.time() - check_start) > READ_CHECK_TIME:
                self.timeout = True
                break

    def build(self):
        self.title = "DroneMapViewer"
        Window.size = MAP_SIZE_WIDTH, MAP_SIZE_HEIGHT

        if self.timeout:
            root = MapLoadTimeout()
        else:
            self.map = MapDisplay()
            self.info = DroneInfo()
            root = self.map
            root.add_widget(self.info)

            Clock.schedule_interval(self.data_update, 0.1)

        return root

    def data_update(self, dt):
        self.map.map_update()
        self.info.info_update()


if __name__ == '__main__':
    MapCreateApp().run()
