#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file map_create_test.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""

import unittest

import RTC
import OpenRTM_aist
import map_create_mock

import sys
import math

from mock import Mock
from mock import patch, MagicMock

INPORT_SET_NUM = 3
DRONE_NUM = 3

ROUND_DIGIT = 4

class TestMapCreateApp(unittest.TestCase):
    def setUp(self):
        self.mock_DroneNumSet = map_create_mock.DroneNumSet().drone_num(DRONE_NUM)

        if DRONE_NUM == 1:
            self.mock_app = map_create_mock.MapCreateApp(m_drone1_lat = 0, m_drone1_lon = 0, 
                                                                 m_drone1_alt = 0,
                                                                 m_drone1_angle = 0, 
                                                                 m_target1_lat = 0,
                                                                 m_target1_lon = 0)

        if DRONE_NUM == 2:
            self.mock_app = map_create_mock.MapCreateApp(m_drone1_lat = 0, m_drone1_lon = 0, 
                                                                 m_drone1_alt = 0,
                                                                 m_drone1_angle = 0, 
                                                                 m_target1_lat = 0,
                                                                 m_target1_lon = 0,
                                                                 m_drone2_lat = 0, m_drone2_lon = 0, 
                                                                 m_drone2_alt = 0,
                                                                 m_drone2_angle = 0, 
                                                                 m_target2_lat = 0,
                                                                 m_target2_lon = 0)

        if DRONE_NUM == 3:
            self.mock_app = map_create_mock.MapCreateApp(m_drone1_lat = 0, m_drone1_lon = 0, 
                                                                 m_drone1_alt = 0,
                                                                 m_drone1_angle = 0, 
                                                                 m_target1_lat = 0,
                                                                 m_target1_lon = 0,
                                                                 m_drone2_lat = 0, m_drone2_lon = 0, 
                                                                 m_drone2_alt = 0,
                                                                 m_drone2_angle = 0, 
                                                                 m_target2_lat = 0,
                                                                 m_target2_lon = 0,
                                                                 m_drone3_lat = 0, m_drone3_lon = 0, 
                                                                 m_drone3_alt = 0,
                                                                 m_drone3_angle = 0, 
                                                                 m_target3_lat = 0, 
                                                                 m_target3_lon = 0)

        self.mock_MapZoomUpdate = map_create_mock.MapZoomUpdate().map_zoom(18)

        if DRONE_NUM > 0:
            self.set_drone_pos(37.5230, 139.9380,  0.0000, 0)
            self.set_drone_angle(45,  0)
            self.set_target_pos(37.5231, 139.9380, 0)
        if DRONE_NUM > 1:
            self.set_drone_pos(37.5234, 139.9372, 10.0000, 1)
            self.set_drone_angle(180, 1)
            self.set_target_pos(37.5229, 139.9382, 1)
        if DRONE_NUM > 2:
            self.set_drone_pos(37.5226, 139.9379, 15.0000, 2)
            self.set_drone_angle(270, 2)
            self.set_target_pos(37.5229, 139.9377, 2)
        
        self.mock_MapDisplay = map_create_mock.MapDisplay()
        self.mock_DroneInfo = map_create_mock.DroneInfo()
        self.mock_MapLoadTimeout = map_create_mock.MapLoadTimeout()


    def set_drone_pos(self, lat, lon, alt, no):
        self.mock_DronePosUpdate = map_create_mock.DronePosUpdate().drone_move(lat, lon,  alt, no)

    def set_drone_angle(self, angle, no):
        self.mock_DroneAngleUpdate = map_create_mock.DroneAngleUpdate().drone_rotate(angle, no)

    def set_target_pos(self, target_lat, target_lon, no): 
        self.mock_TargetPosUpdate = map_create_mock.TargetPosUpdate().drone_target(target_lat, target_lon, no)

    def test_MapDisplay__init__(self):

        result_zoom = [18]
        result_mapview_lat = [37.5230, 37.5232, 37.5230]
        result_mapview_lon = [139.9380, 139.9376, 139.9377]
        result_mapmarker_lat = [37.5230, 37.5234, 37.5226]
        result_mapmarker_lon = [139.9380 ,139.9372, 139.9379]

        result_line_x1 = [[440.0000],
                          [514.5654, 365.4346],
                          [495.9241, 346.7932, 477.2827]
                         ]
        result_line_y1 = [[330.0000],
                          [282.9916, 377.0084],
                          [330.0000, 424.0166, 235.9834]
                         ]
        result_line_x2 = [[440.0000],
                          [514.5654, 551.8481],
                          [495.9241, 533.2068, 440.0000]
                         ]
        result_line_y2 = [[353.5042],
                          [306.4958, 259.4874],
                          [353.5042, 306.4958, 306.4958]
                         ]


        self.assertEqual(self.mock_MapDisplay.mapview.zoom, result_zoom[0])
        self.assertEqual(round(self.mock_MapDisplay.mapview.lat, ROUND_DIGIT), result_mapview_lat[DRONE_NUM-1])
        self.assertEqual(round(self.mock_MapDisplay.mapview.lon, ROUND_DIGIT), result_mapview_lon[DRONE_NUM-1])
        num = 0
        while num < DRONE_NUM:
            self.assertEqual(round(self.mock_MapDisplay.mapmarker[num].lat, ROUND_DIGIT), result_mapmarker_lat[num])
            self.assertEqual(round(self.mock_MapDisplay.mapmarker[num].lon, ROUND_DIGIT), result_mapmarker_lon[num])
            self.assertEqual(round(self.mock_MapDisplay.map_drone_width[num], ROUND_DIGIT), result_line_x1[DRONE_NUM-1][num])
            self.assertEqual(round(self.mock_MapDisplay.map_drone_height[num], ROUND_DIGIT), result_line_y1[DRONE_NUM-1][num])
            self.assertEqual(round(self.mock_MapDisplay.map_target_width[num], ROUND_DIGIT), result_line_x2[DRONE_NUM-1][num])
            self.assertEqual(round(self.mock_MapDisplay.map_target_height[num], ROUND_DIGIT), result_line_y2[DRONE_NUM-1][num])
            num += 1

    def test_is_all_rtl(self):
        in_state = [0] * INPORT_SET_NUM


        in_drone_num = [1, 1, 1, 2, 2, 2, 2, 2, 2, 2,
                        3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3]

        in_state[0] = ["RTL", "Disarm", "Armed", "RTL", "RTL", "Disarm", "Disarm", "RTL", "Armed", "Armed",
                       "RTL", "RTL", "RTL", "RTL", "Disarm", "Disarm", "Disarm", "Disarm",  "RTL", "Armed",
                       "Armed", "RTL", "Armed"]
        in_state[1] = ["None", "None", "None", "RTL", "Disarm", "RTL", "Disarm", "Armed", "Disarm", "Armed",
                       "RTL", "RTL", "Disarm", "Disarm", "RTL", "RTL", "Disarm", "Disarm", "Disarm", "RTL", 
                       "Armed", "Armed", "Armed"]
        in_state[2] = ["None", "None", "None", "None", "None", "None", "None", "None", "None", "None",
                       "RTL", "Disarm", "RTL", "Disarm", "RTL", "Disarm", "RTL", "Disarm", "Armed", "Disarm", 
                       "Disarm", "Armed", "Armed"]



        all_rtl_result = [True, True, False, True, True, True, True, False, False, False,
                                True, True, True, True, True, True, True, True, False, False,
                                False, False, False]


        test_case = 0
        while test_case < len(all_rtl_result):
            self.mock_DroneNumSet = map_create_mock.DroneNumSet().drone_num(in_drone_num[test_case])

            drone_num = 0
            while drone_num < map_create_mock.drone_num:
                self.mock_DroneStateUpdate = map_create_mock.DroneStateUpdate().drone_state(in_state[drone_num][test_case], drone_num)
                drone_num += 1

            self.mock_MapDisplay._is_all_rtl()

            self.assertEqual(self.mock_MapDisplay.all_rtl, all_rtl_result[test_case])

            test_case += 1

    def test_select_drone_calc_c_pos(self):
        in_state = [0] * INPORT_SET_NUM
        in_calc_c_pos = [0] * INPORT_SET_NUM
        calc_c_pos_result = [0] * INPORT_SET_NUM

        #
        all_rtl = [False, False, False, False, False, False, False, False, False, 
                       True, True, True, True, True, True, True, True, True, True, True, True, True, True]

        in_drone_num = [1, 2, 2, 2, 3, 3, 3, 3, 3,
                        1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3]

        in_state[0] = ["Armed", "RTL", "Armed", "Armed", "RTL", "Armed", "Disarm", "RTL", "Armed", 
                       "RTL", "Disarm", "RTL", "RTL", "Disarm", "Disarm", "RTL", "RTL", "RTL", "RTL", "Disarm", "Disarm", "Disarm", "Disarm"]
        in_state[1] = ["None", "Armed", "Disarm", "Armed", "Disarm", "RTL", "Armed", "Armed", "Armed", 
                       "None", "None", "RTL", "Disarm", "RTL", "Disarm", "RTL", "RTL", "Disarm", "Disarm", "RTL", "RTL", "Disarm", "Disarm"]
        in_state[2] = ["None", "None", "None", "None", "Armed", "Disarm", "Disarm", "Armed", "Armed",
                       "None", "None", "None", "None", "None", "None", "RTL", "Disarm", "RTL", "Disarm", "RTL", "Disarm", "RTL", "Disarm"]

        in_calc_c_pos[0] = [False, False, False, False, False, False, False, False, False,
                                       True, True, True, True, False, False, True, True, True, True, False, False, False, False]

        in_calc_c_pos[1] = [None, False, False, False, False, False, False, False, False,
                                       None, None, True, False, True, True, True, True, False, False, True, True, False, False]

        in_calc_c_pos[2] = [None, None, None, None, False, False, False, False, False,
                                       None, None, None, None, None, None, True, False, True, False, True, False, True, True]

        #result
        calc_c_pos_result[0] = [True, False, True, True, False, True, False, False, True,
                                           True, True, True, True, False, False, True, True, True, True, False, False, False, False]

        calc_c_pos_result[1] = [None, True, False, True, False, False, True, True, True,
                                       None, None, True, False, True, True, True, True, False, False, True, True, False, False]

        calc_c_pos_result[2] = [None, None, None, None, True, False, False, True, True,
                                       None, None, None, None, None, None, True, False, True, False, True, False, True, True]


        test_case = 0
        while test_case < len(calc_c_pos_result[0]):
            self.mock_MapDisplay.all_rtl = all_rtl[test_case]
            self.mock_DroneNumSet = map_create_mock.DroneNumSet().drone_num(in_drone_num[test_case])

            drone_num = 0
            while drone_num < map_create_mock.drone_num:
                self.mock_DroneStateUpdate = map_create_mock.DroneStateUpdate().drone_state(in_state[drone_num][test_case], drone_num)
                self.mock_MapDisplay._calc_c_pos[drone_num] = in_calc_c_pos[drone_num][test_case]
                drone_num += 1

            self.mock_MapDisplay.select_drone_calc_c_pos()

            drone_num = 0
            while drone_num < map_create_mock.drone_num:
                self.assertEqual(self.mock_MapDisplay._calc_c_pos[drone_num], calc_c_pos_result[drone_num][test_case])
                drone_num += 1

            test_case += 1

    def test_MapDisplay_map_update(self):
        map_center = [37.5230, 139.9380] 
        result_center = [37.5230, 139.9380]

        self.mock_MapDisplay.map_center_lat = map_center[0]
        self.mock_MapDisplay.map_center_lon = map_center[1]
        self.assertEqual(round(self.mock_MapDisplay.map_center_lat, ROUND_DIGIT), result_center[0])   
        self.assertEqual(round(self.mock_MapDisplay.map_center_lon, ROUND_DIGIT), result_center[1])

    def test_MapDisplay_map_center_change(self):
        in_drone_width = [0] * INPORT_SET_NUM
        in_drone_width[0] = [99,100,101,440,779,780,781]
        in_drone_width[1] = [99,100,101,440,779,780,781]
        in_drone_width[2] = [99,100,101,440,779,780,781]

        in_drone_height = [0] * INPORT_SET_NUM
        in_drone_height[0] = [99,100,101,330,559,560,561]
        in_drone_height[1] = [99,100,101,330,559,560,561]
        in_drone_height[2] = [99,100,101,330,559,560,561]


        #result
        result = [True, False, False, False, False, False, True]

        num = 0
        while num < DRONE_NUM:
            self.mock_MapDisplay._calc_c_pos[num] = True#in_calc_c_pos[num][check_num]

            self.mock_MapDisplay.map_drone_width[num] = in_drone_width[num][3]
            self.mock_MapDisplay.map_drone_height[num] = in_drone_height[num][3]

            num += 1

        num = 0
        while num < DRONE_NUM:
            check_num = 0
            while check_num < 7:

                self.mock_MapDisplay.map_drone_width[num] = in_drone_width[num][check_num]
                ret = self.mock_MapDisplay.map_center_change()
                self.assertEqual(ret, result[check_num])
                self.mock_MapDisplay.map_drone_width[num] = in_drone_width[num][3]


                self.mock_MapDisplay.map_drone_height[num] = in_drone_height[num][check_num]
                ret = self.mock_MapDisplay.map_center_change()
                self.assertEqual(ret, result[check_num])
                self.mock_MapDisplay.map_drone_height[num] = in_drone_height[num][3]
        
                check_num += 1
            num += 1

        #---drone_state_rtl---
        in_drone_num = [2, 2, 3, 3, 3, 3, 3, 3]

        in_drone_state = [0] * INPORT_SET_NUM
        in_drone_state = [0] * INPORT_SET_NUM
        in_drone_state[0] = ["Disarm", "Armed", "RTL", "Armed", "Armed", "Disarm", "RTL", "Armed"]
        in_drone_state[1] = ["Armed", "RTL", "Armed", "Disarm", "Armed", "Armed", "RTL", "Disarm"]
        in_drone_state[2] = ["None", "None", "Armed", "Armed", "RTL", "Disarm", "Armed", "RTL"]

        in_calc_c_pos = [0] * INPORT_SET_NUM
        in_calc_c_pos[0] = [False, True, False, True, True, False, False, True]
        in_calc_c_pos[1] = [True, False, True, False, True, True, False, False]
        in_calc_c_pos[2] = [None, None, True, True, False, False, True, False]



        in_drone_width[0] = [440, 440, 781, 440, 440, 99, 99, 440]
        in_drone_width[1] = [440, 99, 440, 99, 440, 440, 781, 99]
        in_drone_width[2] = [440, 440, 440, 440, 99, 781, 440, 781]

        in_drone_height = [0] * INPORT_SET_NUM
        in_drone_height[0] = [99, 330, 330, 330, 330, 330, 330, 330]
        in_drone_height[1] = [330, 330, 330, 330, 330, 330, 330, 330]
        in_drone_height[2] = [330, 330, 330, 330, 330, 330, 330, 330]

        map_center_change_result = [False, False, False, False, False, False, False, False]

        check_num = 0
        while check_num < len(map_center_change_result):
            self.mock_DroneNumSet = map_create_mock.DroneNumSet().drone_num(in_drone_num[check_num])
            self.mock_DroneStateUpdate = map_create_mock.DroneStateUpdate().drone_state(in_drone_state[0][check_num], 0)
            self.mock_DroneStateUpdate = map_create_mock.DroneStateUpdate().drone_state(in_drone_state[1][check_num], 1)
            self.mock_DroneStateUpdate = map_create_mock.DroneStateUpdate().drone_state(in_drone_state[2][check_num], 2)

            num = 0
            while num < map_create_mock.drone_num:
                self.mock_MapDisplay._calc_c_pos[num] = in_calc_c_pos[num][check_num]
                self.mock_MapDisplay.map_drone_width[num] = in_drone_width[num][check_num]
                self.mock_MapDisplay.map_drone_height[num] = in_drone_height[num][check_num]
                num += 1

            ret = self.mock_MapDisplay.map_center_change()

            self.assertEqual(ret, map_center_change_result[check_num])
            check_num += 1

    def test_MapDisplay_map_center_calc(self):

        drone_lat = [0] * INPORT_SET_NUM
        drone_lat[0] = [37.5230]
        drone_lat[1] = [37.5234]
        drone_lat[2] = [37.5226]

        drone_lon = [0] * INPORT_SET_NUM
        drone_lon[0] = [139.9380]
        drone_lon[1] = [139.9372]
        drone_lon[2] = [139.9379]


        in_drone_num = [1, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3]

        in_calc_c_pos = [0] * INPORT_SET_NUM
        in_calc_c_pos[0] = [True, True, True, False, True, True, True, False, True, False, False]
        in_calc_c_pos[1] = [None, True, False, True, True, True, False, True, False, True, False]
        in_calc_c_pos[2] = [None, None, None, None, True, False, True, True, False, False, True]

        map_center_lat_result = [37.5230, 37.5232, 37.5230, 37.5234, 37.5230, 37.5232, 37.5228, 37.5230, 37.5230, 37.5234, 37.5226]
        map_center_lon_result = [139.9380, 139.9376, 139.9380, 139.9372, 139.9377, 139.9376, 139.9380, 139.9375, 139.9380, 139.9372, 139.9379]


        check_num = 0
        while check_num < len(map_center_lat_result):
            self.mock_DroneNumSet = map_create_mock.DroneNumSet().drone_num(in_drone_num[check_num])

            num = 0
            while num < map_create_mock.drone_num:
                self.mock_MapDisplay._calc_c_pos[num] = in_calc_c_pos[num][check_num]
                num += 1

            self.mock_MapDisplay.map_center_calc()

            self.assertEqual(round(self.mock_MapDisplay.map_center_lat, 4), map_center_lat_result[check_num])
            self.assertEqual(round(self.mock_MapDisplay.map_center_lon, 4), map_center_lon_result[check_num])
            check_num += 1

    def test_MapDisplay_icon_update(self):
        drone_lat = [0] * INPORT_SET_NUM
        drone_lat[0] = [37.5230]
        drone_lat[1] = [37.5234]
        drone_lat[2] = [37.5226]

        drone_lon = [0] * INPORT_SET_NUM
        drone_lon[0] = [139.9380]
        drone_lon[1] = [139.9372]
        drone_lon[2] = [139.9379]

        drone_angle = [0] * INPORT_SET_NUM
        drone_angle[0] = [45]
        drone_angle[1] = [180]
        drone_angle[2] = [270]

        #result
        result_mapmarker_lat = [37.5230, 37.5234, 37.5226]
        result_mapmarker_lon = [139.9380 ,139.9372, 139.9379]
        result_line_x1 = [[440.0000],
                          [514.5654, 365.4346],
                          [495.9241, 346.7932, 477.2827]
                         ]
        result_line_y1 = [[330.0000],
                          [282.9916, 377.0084],
                          [330.0000, 424.0166, 235.9834]
                         ]
        result_line_x2 = [[440.0000],
                          [514.5654, 551.8481],
                          [495.9241, 533.2068, 440.0000]
                         ]
        result_line_y2 = [[353.5042],
                          [306.4958, 259.4874],
                          [353.5042, 306.4958, 306.4958]
                         ]

        num = 0
        while num < DRONE_NUM:
            self.assertEqual(round(self.mock_MapDisplay.mapmarker[num].lat, ROUND_DIGIT), result_mapmarker_lat[num])
            self.assertEqual(round(self.mock_MapDisplay.mapmarker[num].lon, ROUND_DIGIT), result_mapmarker_lon[num])
            self.assertEqual(round(self.mock_MapDisplay.map_drone_width[num], ROUND_DIGIT), result_line_x1[DRONE_NUM-1][num])
            self.assertEqual(round(self.mock_MapDisplay.map_drone_height[num], ROUND_DIGIT), result_line_y1[DRONE_NUM-1][num])
            self.assertEqual(round(self.mock_MapDisplay.map_target_width[num], ROUND_DIGIT), result_line_x2[DRONE_NUM-1][num])
            self.assertEqual(round(self.mock_MapDisplay.map_target_height[num], ROUND_DIGIT), result_line_y2[DRONE_NUM-1][num])
            num += 1
    
    def test_MapDisplay_target_update(self):
    
        self._virtual_leader = RTC.DroneMap(RTC.Time(0,0),
                                               RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                           0.0, 0, RTC.GPS_FIX_NORMAL),
                                               RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                           0.0, 0, RTC.GPS_FIX_NORMAL),
                                               RTC.TimedFloat(RTC.Time(0,0),0),
                                               RTC.TimedString(RTC.Time(0,0),0))

        virtual_leader_GPS = [False, False, False, True, True, True]

        virtual_leader_pos_lat = [0, 37.5230, 37.5232, 37.5230]
        virtual_leader_pos_lon = [0, 139.9380, 139.9376, 139.9377]
        virtual_leader_target_lat = [0, 37.5232, 37.5233, 37.5232]
        virtual_leader_target_lon = [0, 139.9379, 139.9376, 139.9380]

        map_center_lat = [0, 37.5230, 37.5232, 37.5230]
        map_center_lon = [0, 139.9380, 139.9376, 139.9377]

        drone_lat = [0] * INPORT_SET_NUM
        drone_lat[0] = [37.5230]
        drone_lat[1] = [37.5234]
        drone_lat[2] = [37.5226]

        drone_lon = [0] * INPORT_SET_NUM
        drone_lon[0] = [139.9380]
        drone_lon[1] = [139.9372]
        drone_lon[2] = [139.9379]

        drone_target_lat = [37.5231, 37.5229, 37.5229]
        drone_target_lon = [139.9380, 139.9382, 139.9377]

        #result
        result_map_leader_pos_width = [0, 440, 440, 440, 0, 0]
        result_map_leader_pos_height = [0, 330, 330, 330, 0, 0]
        result_map_leader_target_width = [0, 440, 421.3586, 440, 0, 0]
        result_map_leader_target_height =[0, 330, 377.0083, 353.5042, 0, 0]


        result_map_drone_width = [[440.0000],
                                  [514.5654, 365.4346],
                                  [495.9241, 346.7932, 477.2827]
                                 ]
        result_map_drone_height = [[330.0000],
                                   [282.9916, 377.0084],
                                   [330.0000, 424.0166, 235.9834]
                                  ]
        result_map_target_width = [[440.0000],
                                   [514.5654, 551.8481],
                                   [495.9241, 533.2068, 440.0000]
                                  ]
        result_map_target_height = [[353.5042],
                                    [306.4958, 259.4874],
                                    [353.5042, 306.4958, 306.4958]
                                   ]


        for i in range(4):

            print i#test_case
            #self.mock_MapDisplay.virtual_leader_GPS = virtual_leader_GPS[i]
            #self.mock_MapDisplay.virtual_leader_pos_lat = virtual_leader_pos_lat[i]
            #self.mock_MapDisplay.virtual_leader_pos_lon = virtual_leader_pos_lon[i]
            #self.mock_MapDisplay.virtual_leader_target_lat = virtual_leader_target_lat[i]
            #self.mock_MapDisplay.virtual_leader_target_lon = virtual_leader_target_lon[i]

            #self.mock_MapDisplay.target_update()
            self.mock_MapDisplay.map_center_lat = map_center_lat[i]
            self.mock_MapDisplay.map_center_lon = map_center_lon[i]

            self.assertEqual(round(self.mock_MapDisplay.map_leader_pos_width, ROUND_DIGIT), result_map_leader_pos_width[i])
            self.assertEqual(round(self.mock_MapDisplay.map_leader_pos_height, ROUND_DIGIT), result_map_leader_pos_height[i])
            self.assertEqual(round(self.mock_MapDisplay.map_leader_target_width, ROUND_DIGIT),
                                                            result_map_leader_target_width[i])
            self.assertEqual(round(self.mock_MapDisplay.map_leader_target_height, ROUND_DIGIT),
                                                            result_map_leader_target_height[i])

            self._virtual_leader.droneGPS.latitude = virtual_leader_pos_lat[i]
            self._virtual_leader.droneGPS.longitude = virtual_leader_pos_lon[i]
            self._virtual_leader.targetGPS.latitude = virtual_leader_target_lat[i]
            self._virtual_leader.targetGPS.longitude = virtual_leader_target_lon[i]
            map_create_mock.VirtualLeaderUpdate().virtual_leader(self._virtual_leader)
            self.mock_MapDisplay.target_update()
            """
            num = 0
            while num < DRONE_NUM:
                self.assertEqual(round(self.mock_MapDisplay.map_drone_width[num], ROUND_DIGIT), result_map_drone_width[DRONE_NUM-1][num])
                self.assertEqual(round(self.mock_MapDisplay.map_drone_height[num], ROUND_DIGIT), result_map_drone_height[DRONE_NUM-1][num])
                self.assertEqual(round(self.mock_MapDisplay.map_target_width[num], ROUND_DIGIT), result_map_target_width[DRONE_NUM-1][num])
                self.assertEqual(round(self.mock_MapDisplay.map_target_height[num], ROUND_DIGIT), result_map_target_height[DRONE_NUM-1][num])
                num += 1
            """
    def test_MapDisplay_locus_update(self):

        self._locus = RTC.DroneMap(RTC.Time(0,0),
                                               RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                           0.0, 0, RTC.GPS_FIX_NORMAL),
                                               RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                           0.0, 0, RTC.GPS_FIX_NORMAL),
                                               RTC.TimedFloat(RTC.Time(0,0),0),
                                               RTC.TimedString(RTC.Time(0,0),0))

        virtual_leader_pos_lat = [37.510, 37.520, 37.530, 37.540, 37.550, 37.560, 37.570, 37.580, 37.590, 37.600]
        virtual_leader_pos_lon = [139.9310, 139.9320, 139.9330, 139.9340, 139.9350, 139.9360, 139.9370, 139.9380, 139.9390, 139.9400]

        #result
        result_locus_lat= [[None, None, None, None, None, None, None, None, None, None],
                           [37.510, None, None, None, None, None, None, None, None, None],
                           [37.520, 37.510, None, None, None, None, None, None, None, None],
                           [37.530, 37.520, 37.510, None, None, None, None, None, None, None],
                           [37.540, 37.530, 37.520, 37.510, None, None, None, None, None, None],
                           [37.550, 37.540, 37.530, 37.520, 37.510, None, None, None, None, None],
                           [37.560, 37.550, 37.540, 37.530, 37.520, 37.510, None, None, None, None],
                           [37.570, 37.560, 37.550, 37.540, 37.530, 37.520, 37.510, None, None, None],
                           [37.580, 37.570, 37.560, 37.550, 37.540, 37.530, 37.520, 37.510, None, None],
                           [37.590, 37.580, 37.570, 37.560, 37.550, 37.540, 37.530, 37.520, 37.510, None],
                           [37.600, 37.590, 37.580, 37.570, 37.560, 37.550, 37.540, 37.530, 37.520, 37.510]
                          ]
        result_locus_lon = [[None, None, None, None, None, None, None, None, None, None],
                            [139.9310, None, None, None, None, None, None, None, None, None],
                            [139.9320, 139.9310, None, None, None, None, None, None, None, None],
                            [139.9330, 139.9320, 139.9310, None, None, None, None, None, None, None],
                            [139.9340, 139.9330, 139.9320, 139.9310, None, None, None, None, None, None],
                            [139.9350, 139.9340, 139.9330, 139.9320, 139.9310, None, None, None, None, None],
                            [139.9360, 139.9350, 139.9340, 139.9330, 139.9320, 139.9310, None, None, None, None],
                            [139.9370, 139.9360, 139.9350, 139.9340, 139.9330, 139.9320, 139.9310, None, None, None],
                            [139.9380, 139.9370, 139.9360, 139.9350, 139.9340, 139.9330, 139.9320, 139.9310, None, None],
                            [139.9390, 139.9380, 139.9370, 139.9360, 139.9350, 139.9340, 139.9330, 139.9320, 139.9310, None],
                            [139.9400, 139.9390, 139.9380, 139.9370, 139.9360, 139.9350, 139.9340, 139.9330, 139.9320, 139.9310]
                           ]

        for test_case in range(10):
            for num in range(10):
                self.assertEqual(self.mock_MapDisplay.locus_lat[num], result_locus_lat[test_case][num])
                self.assertEqual(self.mock_MapDisplay.locus_lon[num], result_locus_lon[test_case][num])

            self._locus.droneGPS.latitude = virtual_leader_pos_lat[test_case]
            self._locus.droneGPS.longitude = virtual_leader_pos_lon[test_case]
            map_create_mock.VirtualLeaderUpdate().virtual_leader(self._locus)
            self.mock_MapDisplay.locus_update()

    def test_MapDroneInfo__init__(self):

        drone_lat = [37.5230, 37.5234, 37.5226]
        drone_lon = [139.9380, 139.9372, 139.9379]
        drone_alt = [0.0000, 10.0000, 15.0000]

        #result
        result_name_text = ["Drone1", "Drone2", "Drone3"]
        result_lat_text = ["lat:  37.523000", "lat:  37.523400", "lat:  37.522600"]
        result_lon_text = ["lon: 139.938000", "lon: 139.937200", "lon: 139.937900"]
        result_alt_text = ["alt:  0.000000", "alt:  10.000000", "alt:  15.000000"]

        num = 0
        while num < DRONE_NUM:
            self.assertEqual(self.mock_DroneInfo.info_name[num].text, result_name_text[num])
            self.assertEqual(self.mock_DroneInfo.info_lat[num].text, result_lat_text[num])
            self.assertEqual(self.mock_DroneInfo.info_lon[num].text, result_lon_text[num])
            self.assertEqual(self.mock_DroneInfo.info_alt[num].text, result_alt_text[num])
            num += 1
 
    def test_MapDroneInfo_info_update(self):

        drone_lat = [37.5230, 37.5234, 37.5226]
        drone_lon = [139.9380, 139.9372, 139.9379]
        drone_alt = [0.0000, 10.0000, 15.0000]

        #result
        result_lat_text = ["lat:  37.523000", "lat:  37.523400", "lat:  37.522600"]
        result_lon_text = ["lon: 139.938000", "lon: 139.937200", "lon: 139.937900"]
        result_alt_text = ["alt:  0.000000", "alt:  10.000000", "alt:  15.000000"]

        num = 0
        while num < DRONE_NUM:
            self.assertEqual(self.mock_DroneInfo.info_lat[num].text, result_lat_text[num])
            self.assertEqual(self.mock_DroneInfo.info_lon[num].text, result_lon_text[num])
            self.assertEqual(self.mock_DroneInfo.info_alt[num].text, result_alt_text[num])
            num += 1

    def test_MapLoadTimeout__init__(self):
        result_timeout_text = "Map Loaded Timeout"

        self.assertEqual(self.mock_MapLoadTimeout.timeout_msg.text, result_timeout_text)


    def test_MapCreateApp__init__(self):

        drone_lat = [0] * INPORT_SET_NUM
        drone_lat[0] = [False, True, False, True, True, False, True, True, True]
        drone_lat[1] = [None, None, None, False, True, None, False, True, True]                 
        drone_lat[2] = [None, None, None, None, None, None, None, False, True]

        drone_lon = [0] * INPORT_SET_NUM
        drone_lon[0] = [False, True, False, True, True, False, True, True, True]
        drone_lon[1] = [None, None, None, False, True, None, False, True, True]
        drone_lon[2] = [None, None, None, None, None, None, None, False, True]

        drone_alt = [0] * INPORT_SET_NUM
        drone_alt[0] = [False, True, False, True, True, False, True, True, True]
        drone_alt[1] = [None, None, None, False, True, None, False, True, True]
        drone_alt[2] = [None, None, None, None, None, None, None, False, True]

        drone_angle = [0] * INPORT_SET_NUM
        drone_angle[0] = [False, True, False, True, True, False, True, True, True]
        drone_angle[1] = [None, None, None, False, True, None, False, True, True]
        drone_angle[2] = [None, None, None, None, None, None, None, False, True]

        drone_target_lat = [0] * INPORT_SET_NUM
        drone_target_lat[0] = [False, True, False, True, True, False, True, True, True]
        drone_target_lat[1] = [None, None, None, False, True, None, False, True, True]
        drone_target_lat[2] = [None, None, None, None, None, None, None, False, True]

        drone_target_lon = [0] * INPORT_SET_NUM
        drone_target_lon[0] = [False, True, False, True, True, False, True, True, True]
        drone_target_lon[1] = [None, None, None, False, True, None, False, True, True]
        drone_target_lon[2] = [None, None, None, None, None, None, None, False, True]

        result_timeout = [0] * INPORT_SET_NUM
        result_timeout = [True, False, True, True, False, True, True, True, False]


        if DRONE_NUM == 1:
            elem = 0
        elif DRONE_NUM == 2:
            elem = 2
        elif DRONE_NUM == 3:
            elem = 5
        num = elem
        while num <= (elem + DRONE_NUM):
            if DRONE_NUM == 1:
                self.mock_app = map_create_mock.MapCreateApp(m_drone1_lat = drone_lat[0][elem], m_drone1_lon = drone_lon[0][elem], 
                                                                     m_drone1_alt = drone_alt[0][elem],
                                                                     m_drone1_angle = drone_angle[0][elem], 
                                                                     m_target1_lat = drone_target_lat[0][elem],
                                                                     m_target1_lon = drone_target_lon[0][elem])

            if DRONE_NUM == 2:
                self.mock_app = map_create_mock.MapCreateApp(m_drone1_lat = drone_lat[0][elem], m_drone1_lon = drone_lon[0][elem], 
                                                                     m_drone1_alt = drone_alt[0][elem],
                                                                     m_drone1_angle = drone_angle[0][elem], 
                                                                     m_target1_lat = drone_target_lat[0][elem],
                                                                     m_target1_lon = drone_target_lon[0][elem],
                                                                     m_drone2_lat = drone_lat[1][elem], m_drone2_lon = drone_lon[1][elem], 
                                                                     m_drone2_alt = drone_alt[1][elem],
                                                                     m_drone2_angle = drone_angle[1][elem], 
                                                                     m_target2_lat = drone_target_lat[1][elem],
                                                                     m_target2_lon = drone_target_lon[1][elem])

            if DRONE_NUM == 3:
                self.mock_app = map_create_mock.MapCreateApp(m_drone1_lat = drone_lat[0][elem], m_drone1_lon = drone_lon[0][elem], 
                                                                     m_drone1_alt = drone_alt[0][elem],
                                                                     m_drone1_angle = drone_angle[0][elem], 
                                                                     m_target1_lat = drone_target_lat[0][elem],
                                                                     m_target1_lon = drone_target_lon[0][elem],
                                                                     m_drone2_lat = drone_lat[1][elem], m_drone2_lon = drone_lon[1][elem], 
                                                                     m_drone2_alt = drone_alt[1][elem],
                                                                     m_drone2_angle = drone_angle[1][elem], 
                                                                     m_target2_lat = drone_target_lat[1][elem],
                                                                     m_target2_lon = drone_target_lon[1][elem],
                                                                     m_drone3_lat = drone_lat[2][elem], m_drone3_lon = drone_lon[2][elem], 
                                                                     m_drone3_alt = drone_alt[2][elem],
                                                                     m_drone3_angle = drone_angle[2][elem], 
                                                                     m_target3_lat = drone_target_lat[2][elem], 
                                                                     m_target3_lon = drone_target_lon[2][elem])

            self.assertEqual(self.mock_app.timeout, result_timeout[elem])

            num += 1

if __name__ == '__main__':
    unittest.main()
