#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DroneViewer.py
 @brief Display a variety of info of the drone
 @date $Date$

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import sys
import time

# Import RTM module
import RTC
import OpenRTM_aist

# Import viewer
import drone_view as view
import drone_distance_cal as dr_cal

# log output
from logging import getLogger, StreamHandler, DEBUG
logger = getLogger(__name__)
handler = StreamHandler()
handler.setLevel(DEBUG)
logger.setLevel(DEBUG)
logger.addHandler(handler)

sys.path.append(".")

# error message
ERR_MSG_01 = "WARNING: It exceeds the display area"

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
droneviewer_spec = ["implementation_id", "DroneViewer",
                    "type_name",         "DroneViewer",
                    "description",       "Display a variety of info of the drone",
                    "version",           "1.4.0",
                    "vendor",            "EAC",
                    "category",          "Viewer",
                    "activity_type",     "STATIC",
                    "max_instance",      "1",
                    "language",          "Python",
                    "lang_type",         "SCRIPT",
                    "conf.default.AltMaxVal", "50",
                    "conf.__widget__.AltMaxVal", "spin",
                    "conf.__constraints__.AltMaxVal", "0<x<=1000",
                    ""]
# </rtc-template>

##
# @class DroneViewer
# @brief Display a variety of info of the drone
#
# Overview: Refer to the "機能仕様書_DroneViewer.pdf" for details.
#
# In / Out: Refer to the "機能仕様書_DroneViewer.pdf" for details.
#
# Algorythm: Refer to the "機能仕様書_DroneViewer.pdf" for details.
#
#


class DroneViewer(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_inIOComposite = RTC.DroneView(RTC.Time(0, 0),
                                            RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                        100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                        0.0, 0, RTC.GPS_FIX_NORMAL),
                                            RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                        100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                        0.0, 0, RTC.GPS_FIX_NORMAL),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0]),
                                            RTC.TimedULong(RTC.Time(0, 0), 0),
                                            RTC.TimedOrientation3D(RTC.Time(0, 0), 
                                                        RTC.Orientation3D(0.0, 0.0, 0.0)),
                                            RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedString(RTC.Time(0, 0), ""),
                                            RTC.TimedString(RTC.Time(0, 0), ""),
                                            RTC.TimedString(RTC.Time(0, 0), ""))
        """
        Drone info
        """
        self._inIOCompositeIn = OpenRTM_aist.InPort("inIOComposite", self._d_inIOComposite)
        self._d_inSettingSpeed = RTC.TimedFloat(RTC.Time(0, 0), 0.0)
        """
         - Unit: m/s
        """
        self._inSettingSpeedIn = OpenRTM_aist.InPort("inSettingSpeed", self._d_inSettingSpeed)
        self._d_inMoveMode = RTC.TimedString(RTC.Time(0, 0), "")
        """
        """
        self._inMoveModeIn = OpenRTM_aist.InPort("inMoveMode", self._d_inMoveMode)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
        """
		 - Name:  AltMaxVal
		 - DefaultValue: 50
		"""
        self._AltMaxVal = [50]

        # </rtc-template>

    ##
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onInitialize(self):
        # Bind variables and configuration variable
        self.bindParameter("AltMaxVal", self._AltMaxVal, "50")

        # Set InPort buffers
        self.addInPort("inIOComposite", self._inIOCompositeIn)
        self.addInPort("inSettingSpeed", self._inSettingSpeedIn)
        self.addInPort("inMoveMode", self._inMoveModeIn)

        # Set OutPort buffers

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        return RTC.RTC_OK

    #	##
    #	#
    #	# The finalize action (on ALIVE->END transition)
    #	# formaer rtc_exiting_entry()
    #	#
    #	# @return RTC::ReturnCode_t
    #
    #	#
    # def onFinalize(self):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The startup action when ExecutionContext startup
    #	# former rtc_starting_entry()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onStartup(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The shutdown action when ExecutionContext stop
    #	# former rtc_stopping_entry()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onShutdown(self, ec_id):
    #
    #	return RTC.RTC_OK

        ##
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onActivated(self, ec_id):

        self._init_info()

        return RTC.RTC_OK

    #	##
    #	#
    #	# The deactivated action (Active state exit action)
    #	# former rtc_active_exit()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onDeactivated(self, ec_id):
    #
    #	return RTC.RTC_OK

        ##
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onExecute(self, ec_id):

        self._read_info()
        self._set_info()

        return RTC.RTC_OK

    #	##
    #	#
    #	# The aborting action when main logic error occurred.
    #	# former rtc_aborting_entry()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onAborting(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The error action in ERROR state
    #	# former rtc_error_do()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onError(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The reset action that is invoked resetting
    #	# This is same but different the former rtc_init_entry()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onReset(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The state update action that is invoked after onExecute() action
    #	# no corresponding operation exists in OpenRTm-aist-0.2.0
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#

    #	#
    # def onStateUpdate(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The action that is invoked when execution context's rate is changed
    #	# no corresponding operation exists in OpenRTm-aist-0.2.0
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onRateChanged(self, ec_id):
    #
    #	return RTC.RTC_OK
    def _init_info(self):
        
        self._home_lat = 100.0
        self._home_lon = 200.0
        self._home_alt = 0.0
        self._drone_lat = 100.0
        self._drone_lon = 200.0
        self._drone_alt = 0.0
        self._targetSpeed = 0.0
        self._battV = 0.0
        self._battA = 0.0
        self._fltTime = 0
        self._gyro_r = 0.0
        self._gyro_p = 0.0
        self._gyro_y = 0.0
        self._accel_x = 0.0
        self._accel_y = 0.0
        self._accel_z = 0.0
        self._mag = 0.0
        self._temp = 0.0
        self._pres = 0.0
        self._fltMode = ""
        self._state = ""
        self._sysErr = ""
        self._settingSpd = 0.0
        self._mvMode = ""

    
    def _read_info(self):
    
        self._is_read_drone = False
        if self._inIOCompositeIn.isNew():
            self._d_inIOComposite = self._inIOCompositeIn.read()
            self._is_read_drone = True

        if self._inSettingSpeedIn.isNew():
            self._d_inSettingSpeed = self._inSettingSpeedIn.read()

        if self._inMoveModeIn.isNew():
            self._d_inMoveMode = self._inMoveModeIn.read()
        

    def _set_info(self):

        if self._is_read_drone:
            self._home_lat = self._d_inIOComposite.homeGPS.latitude
            self._home_lon = self._d_inIOComposite.homeGPS.longitude
            self._home_alt = self._d_inIOComposite.homeGPS.altitude
            self._drone_lat = self._d_inIOComposite.droneGPS.latitude
            self._drone_lon = self._d_inIOComposite.droneGPS.longitude
            self._drone_alt = self._d_inIOComposite.droneGPS.altitude
            self._targetSpeed = self._d_inIOComposite.targetSpeed.data
            self._battV = self._d_inIOComposite.batt.data[0]
            self._battA = self._d_inIOComposite.batt.data[1]
            self._fltTime = self._d_inIOComposite.fltTime.data
            self._gyro_r = self._d_inIOComposite.gyro.data.r
            self._gyro_p = self._d_inIOComposite.gyro.data.p
            self._gyro_y = self._d_inIOComposite.gyro.data.y
            self._accel_x = self._d_inIOComposite.accel.data[0]
            self._accel_y = self._d_inIOComposite.accel.data[1]
            self._accel_z = self._d_inIOComposite.accel.data[2]
            self._mag = self._d_inIOComposite.mag.data
            self._temp = self._d_inIOComposite.temp.data
            self._pres = self._d_inIOComposite.pres.data
            self._fltMode = self._d_inIOComposite.fltMode.data
            self._state = self._d_inIOComposite.state.data
            self._sysErr = self._d_inIOComposite.sysErr.data

        self._settingSpd = self._d_inSettingSpeed.data
        self._mvMode = self._d_inMoveMode.data

        self._set_flt_info(self._fltMode, self._mvMode, self._state, self._battV, self._battA, self._fltTime)
        self._set_gyro_info(self._gyro_r, self._gyro_p, self._gyro_y)
        self._set_mag_info(self._mag)
        self._set_err_info(self._sysErr)
        self._set_spd_info(self._accel_x, self._accel_y, self._accel_z, self._targetSpeed, self._settingSpd)
        self._set_baro_info(self._temp, self._pres)
        self._set_gps_info(self._home_lat, self._home_lon, self._home_alt, self._drone_lat, self._drone_lon, self._drone_alt)
        self._set_alt_info(self._drone_alt)

    def _set_flt_info(self, fltMode, mvMode, state, battV, battA, fltTime):
        
        send = view.FlightInfo()
        send.set_fltMode(fltMode)
        send.set_mvMode(mvMode)
        send.set_state(state)
        send.set_batt(battV, battA)
        min = fltTime / 60
        sec = fltTime % 60
        time_str = "{0:d}:{1:02d}".format(min, sec)
        send.set_fltTime(time_str)

    def _set_gyro_info(self, roll, pitch, yaw):
        
        send = view.GyroInfo()
        send.set_gyro(roll, pitch, yaw)

    def _set_mag_info(self, mag):
        
        send = view.GyroInfo()
        send.set_mag(mag)

    def _set_spd_info(self, x, y, z, target, setting):
        
        send = view.SpeedInfo()
        send.set_accel(x, y, z)
        send.set_tgtSpd(target)
        send.set_settingSpd(setting)

    def _set_baro_info(self, temp, pres):
        
        send = view.BaroInfo()
        send.set_baro(temp, pres)

    def _set_err_info(self, msg):
        
        send = view.ErrInfo()
        send.set_sysErr(msg)

    def _set_gps_info(self, h_lat, h_lon, h_alt, dr_lat, dr_lon, dr_alt):

        send = view.GPSInfo()
        d = dr_cal.DroneMathmetics()
        dist = d.distance_math(h_lon, dr_lon, h_lat, dr_lat, 0, dr_alt)
        send.set_home_pos(h_lat, h_lon, h_alt)
        send.set_drone_pos(dr_lat, dr_lon, dr_alt)
        send.set_dist(dist)

    def _set_alt_info(self, dr_alt):

        altitude = self._round_altitude(dr_alt)
        self._set_max()
        self._area_check(altitude)
        self._set_pos(altitude)
    
    def _set_max(self):
        
        send = view.HeightValue()
        send.set_max_val(self._AltMaxVal[0])

    def _set_pos(self, altitude):
        
        send = view.HeightValue()
        send.set_pos(altitude)

    def _area_check(self, altitude):
        
        ruler = view.Scale()
        min = ruler.min_value(self._AltMaxVal[0])
        if altitude > self._AltMaxVal[0] or altitude < min:
            logger.warning(ERR_MSG_01)

    def _round_altitude(self, altitude):

        ret = round(altitude, 3)

        return ret


def DroneViewerInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=droneviewer_spec)
    manager.registerFactory(profile,
                            DroneViewer,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    DroneViewerInit(manager)

    # Create a component
    comp = manager.createComponent("DroneViewer")


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager(True)
    viewer = view.drone_view().run()


if __name__ == "__main__":
    main()
