#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DroneViewer_test.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import unittest
import RTC
import OpenRTM_aist
import drone_distance_cal
import DroneViewer

from mock import Mock

class TestDroneViewer(unittest.TestCase):

    def setUp(self):

        self.m = DroneViewer.DroneViewer(0)

        self.xx_logger = Mock()
        DroneViewer.logger.warning = self.xx_logger.warning
        DroneViewer.logger.error = self.xx_logger.error
        DroneViewer.logger.info = self.xx_logger.info
        DroneViewer.logger.debug = self.xx_logger.debug

    def test_read_info(self):

        xx = Mock()
        self.m._inIOCompositeIn.isNew = xx.isNew
        self.m._inIOCompositeIn.read = xx.read

        yy = Mock()
        self.m._inSettingSpeedIn.isNew = yy.isNew
        self.m._inSettingSpeedIn.read = yy.read

        zz = Mock()
        self.m._inMoveModeIn.isNew = zz.isNew
        self.m._inMoveModeIn.read = zz.read

        init_io_val = RTC.DroneView(RTC.Time(0, 0),
                                            RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                        100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                        0.0, 0, RTC.GPS_FIX_NORMAL),
                                            RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                        100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                        0.0, 0, RTC.GPS_FIX_NORMAL),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0]),
                                            RTC.TimedULong(RTC.Time(0, 0), 0),
                                            RTC.TimedOrientation3D(RTC.Time(0, 0), 
                                                        RTC.Orientation3D(0.0, 0.0, 0.0)),
                                            RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                            RTC.TimedString(RTC.Time(0, 0), ""),
                                            RTC.TimedString(RTC.Time(0, 0), ""),
                                            RTC.TimedString(RTC.Time(0, 0), ""))
 
        io_val1 = RTC.DroneView(RTC.Time(0, 0),
                                        RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                    37.5223693848, 139.9386749268, 208.09, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                    37.5223941, 139.9386752, 10.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.TimedFloat(RTC.Time(0, 0), 3.0),
                                        RTC.TimedFloatSeq(RTC.Time(0, 0), [15.75, 0.0]),
                                        RTC.TimedULong(RTC.Time(0, 0), 19),
                                        RTC.TimedOrientation3D(RTC.Time(0, 0), 
                                                    RTC.Orientation3D(-0.000740, -0.048703, -0.19275)),
                                        RTC.TimedFloatSeq(RTC.Time(0, 0), [0.61, -0.03, 0.0]),
                                        RTC.TimedFloat(RTC.Time(0, 0), 358.0),
                                        RTC.TimedFloat(RTC.Time(0, 0), 28.51),
                                        RTC.TimedFloat(RTC.Time(0, 0), 987.38),
                                        RTC.TimedString(RTC.Time(0, 0), "GUIDED"),
                                        RTC.TimedString(RTC.Time(0, 0), "In_Move"),
                                        RTC.TimedString(RTC.Time(0, 0), ""))

        init_ts_val = self._d_inSettingSpeed = RTC.TimedFloat(RTC.Time(0, 0), 0.0)
        ts_val1 = self._d_inSettingSpeed = RTC.TimedFloat(RTC.Time(0, 0), 3.0)

        init_mv_val = self._d_inMoveMode = RTC.TimedString(RTC.Time(0, 0), "")
        mv_val1 = self._d_inMoveMode = RTC.TimedString(RTC.Time(0, 0), "Absolute")


        in_is_new = [False, True]
        in_io = [init_io_val, io_val1]
        in_ts = [init_ts_val, ts_val1]
        in_mv = [init_mv_val, mv_val1]
        result = [False, True]

        self.m._init_info()
        for cnt in range(len(in_is_new)):
            xx.isNew.return_value = in_is_new[cnt]
            xx.read.return_value = in_io[cnt]
            yy.read.return_value = in_ts[cnt]
            zz.read.return_value = in_mv[cnt]
            self.m._read_info()
            self.m._set_info()
            
            self.assertEqual(self.m._is_read_drone, result[cnt])

    def test_set_info(self):
    
        in_is_read_dr = [False, True]
        in_h_lat = [100.0, 37.5223693848]
        in_h_lon = [200.0, 139.9386749268]
        in_h_alt = [0.0, 208.09]
        in_d_lat = [100.0, 37.5223941]
        in_d_lon = [200.0, 139.9386752]
        in_d_alt = [0.0, 10.0]
        in_ts = [0.0, 3.0]
        in_battV = [0.0, 15.75]
        in_battA = [0.0, 0.0]
        in_flttime = [0, 19]
        in_gyro_r = [0.0, -0.000740]
        in_gyro_p = [0.0, -0.048703]
        in_gyro_y = [0.0, -0.19275]
        in_acc_x = [0.0, 0.61]
        in_acc_y = [0.0, -0.03]
        in_acc_z = [0.0, 0.0]
        in_mag = [0.0, 358.0]
        in_temp = [0.0, 28.51]
        in_pres = [0.0, 987.38]
        in_fltmd = ["", "GUIDED"]
        in_state = ["", "In_Move"]
        in_err = ["", ""]
        in_ss = [0.0, 3.0]
        in_mv = ["", "Absolute"]

        self.m._init_info()

        for cnt in range(len(in_is_read_dr)):
            self.m._is_read_drone = in_is_read_dr[cnt]
            self.m._d_inIOComposite.homeGPS.latitude = in_h_lat[cnt]
            self.m._d_inIOComposite.homeGPS.longitude = in_h_lon[cnt]
            self.m._d_inIOComposite.homeGPS.altitude = in_h_alt[cnt]
            self.m._d_inIOComposite.droneGPS.latitude = in_d_lat[cnt]
            self.m._d_inIOComposite.droneGPS.longitude = in_d_lon[cnt]
            self.m._d_inIOComposite.droneGPS.altitude = in_d_alt[cnt]
            self.m._d_inIOComposite.targetSpeed.data = in_ts[cnt]
            self.m._d_inIOComposite.batt.data[0] = in_battV[cnt]
            self.m._d_inIOComposite.batt.data[1] = in_battA[cnt]
            self.m._d_inIOComposite.fltTime.data = in_flttime[cnt]
            self.m._d_inIOComposite.gyro.data.r = in_gyro_r[cnt]
            self.m._d_inIOComposite.gyro.data.p = in_gyro_p[cnt]
            self.m._d_inIOComposite.gyro.data.y = in_gyro_y[cnt]
            self.m._d_inIOComposite.accel.data[0] = in_acc_x[cnt]
            self.m._d_inIOComposite.accel.data[1] = in_acc_y[cnt]
            self.m._d_inIOComposite.accel.data[2] = in_acc_z[cnt]
            self.m._d_inIOComposite.mag.data = in_mag[cnt]
            self.m._d_inIOComposite.temp.data = in_temp[cnt]
            self.m._d_inIOComposite.pres.data = in_pres[cnt]
            self.m._d_inIOComposite.fltMode.data = in_fltmd[cnt]
            self.m._d_inIOComposite.state.data = in_state[cnt]
            self.m._d_inIOComposite.sysErr.data = in_err[cnt]
            self.m._d_inSettingSpeed.data = in_ss[cnt]
            self.m._d_inMoveMode.data = in_mv[cnt]

            self.m._set_info()
            
            self.assertEqual(self.m._home_lat, in_h_lat[cnt])
            self.assertEqual(self.m._home_lon, in_h_lon[cnt])
            self.assertEqual(self.m._home_alt, in_h_alt[cnt])
            self.assertEqual(self.m._drone_lat, in_d_lat[cnt])
            self.assertEqual(self.m._drone_lon, in_d_lon[cnt])
            self.assertEqual(self.m._drone_alt, in_d_alt[cnt])
            self.assertEqual(self.m._targetSpeed, in_ts[cnt])
            self.assertEqual(self.m._battV, in_battV[cnt])
            self.assertEqual(self.m._battA, in_battA[cnt])
            self.assertEqual(self.m._fltTime, in_flttime[cnt])
            self.assertEqual(self.m._gyro_r, in_gyro_r[cnt])
            self.assertEqual(self.m._gyro_p, in_gyro_p[cnt])
            self.assertEqual(self.m._gyro_y, in_gyro_y[cnt])
            self.assertEqual(self.m._accel_x, in_acc_x[cnt])
            self.assertEqual(self.m._accel_y, in_acc_y[cnt])
            self.assertEqual(self.m._accel_z, in_acc_z[cnt])
            self.assertEqual(self.m._mag, in_mag[cnt])
            self.assertEqual(self.m._temp, in_temp[cnt])
            self.assertEqual(self.m._pres, in_pres[cnt])
            self.assertEqual(self.m._fltMode, in_fltmd[cnt])
            self.assertEqual(self.m._state, in_state[cnt])
            self.assertEqual(self.m._sysErr, in_err[cnt])
            self.assertEqual(self.m._settingSpd, in_ss[cnt])
            self.assertEqual(self.m._mvMode, in_mv[cnt])


    """
    def test_is_read_target_pos(self):

        pos = [RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0), -90.1, -180.1, -0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0, RTC.GPS_FIX_NORMAL),
                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0), -90.1, -180.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0, RTC.GPS_FIX_NORMAL),
                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0), -90.1, 139.938675, 13.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0, RTC.GPS_FIX_NORMAL),
                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0), 37.522794, 139.938675, 13.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0, RTC.GPS_FIX_NORMAL)
        ]
        in_lat = [-90.1, -90.1, -90.1, 37.522794]
        in_lon = [-180.1, -180.1, 139.938675, 139.938675]
        in_alt = [-0.1, 0.0, 13.4, 13.4]
        result = [False, False, False, True]

        for cnt in range(len(pos)):
            self.m._d_inTarget.pos = pos[cnt]
            rtn = self.m._is_read_target_pos()
            self.assertEqual(self.m._d_inTarget.pos.latitude, in_lat[cnt])
            self.assertEqual(self.m._d_inTarget.pos.longitude, in_lon[cnt])
            self.assertEqual(self.m._d_inTarget.pos.altitude, in_alt[cnt])
            self.assertEqual(rtn, result[cnt])

    def test_is_read_target_yaw(self):

        in_yaw_deg = [-1.0, 0.0, 1.0, 90.0, 359.0, 360.0, 361.0]
        result = [False, True, True, True, True, True, False]
        self.m._d_inTarget.yaw = RTC.TimedShortSeq(RTC.Time(0,0),[0.0, 0.0])

        for cnt in range(len(in_yaw_deg)):
            self.m._d_inTarget.yaw.data[1] = in_yaw_deg[cnt]
            rtn = self.m._is_read_target_yaw()
            
            self.assertEqual(self.m._d_inTarget.yaw.data[1], in_yaw_deg[cnt])
            self.assertEqual(rtn, result[cnt])

    def test_arv_judge(self):

        self.m._takeoff_alt = 10.0

        in_sts = [DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_TAKEOFF, DroneIO.STS_TAKEOFF]
        in_drone_alt = [0.0, 9.8, 9.9, 10.0]
        out_sts = [DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING]
        
        for cnt in range(len(in_sts)):
            self.m._drone_sts = in_sts[cnt]
            self.m._d_outGPSDrone.altitude = in_drone_alt[cnt]
            self.m._arv_judge()
            self.assertEqual(self.m._drone_sts, out_sts[cnt])
        
    def test_is_gcs_comm_err(self):

        data = [29, 30, 31]
        result = [False, True, False]

        for i, dat in enumerate(data):
            self.m._comm_err_timer = dat
            rtn = self.m._is_gcs_comm_err()
            self.assertEqual(rtn, result[i])

    def test_takeoff(self):

        data = [5, 10]
        result = [DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF]

        for i, dat in enumerate(data):
            self.m._d_inTarget.pos.altitude = dat
            self.m._drone_sts = DroneIO.STS_ARMED
            #self.m._takeoff()
            #self.assertEqual(self.m._drone_sts, result[i])

    def test_is_rtl_req(self):

        in_lat = [37.522614, 37.522615, 37.522614, 37.522615, 37.522614, 37.522615, 37.522614, 37.522615]
        in_lon = [139.938675, 139.938675, 139.938676, 139.938676, 139.938675, 139.938675, 139.938676, 139.938676]
        in_alt = [208.1, 208.1, 208.1, 208.1, 208.2, 208.2, 208.2, 208.2]
        result = [True, False, False, False, False, False, False, False]

        self.m._d_outGPSHome.latitude = 37.522614
        self.m._d_outGPSHome.longitude = 139.938675
        self.m._d_outGPSHome.altitude = 208.1

        for cnt in range(len(result)):
           self.m._d_inTarget.pos.latitude = in_lat[cnt] 
           self.m._d_inTarget.pos.longitude = in_lon[cnt] 
           self.m._d_inTarget.pos.altitude = in_alt[cnt]
           rtn = self.m._is_rtl_req()
           self.assertEqual(rtn, result[cnt])
 
    def test_is_batt_state(self):

        xx = Mock()
        self.m._vehicle = xx

        self.m._batt_thr = (DroneIO.BATT_LIPO_VPC - (DroneIO.BATT_LIPO_VPC_PERCENT -
                                           (self.m._RTLLowBatt[0] / 100.0))) * self.m._BattCell[0]
        
        in_batt_chk = ['off', 'on', 'on', 'on']
        in_curr_batt = [13.9, 13.9, 14.0, 14.1]
        out_rtn = [True, False, True, True]

        for cnt in range(len(in_batt_chk)):
            self.m._BattChk[0] = in_batt_chk[cnt]
            self.m._vehicle.battery.voltage = in_curr_batt[cnt]
            rtn = self.m._is_batt_state()
            self.assertEqual(rtn, out_rtn[cnt])
    
    def test_continuous_manu_rtl(self):

        in_rtl_stg = [DroneIO.RTL_STAGE_1ST, DroneIO.RTL_STAGE_1ST, DroneIO.RTL_STAGE_2ND, DroneIO.RTL_STAGE_2ND, 
        DroneIO.RTL_STAGE_2ND, DroneIO.RTL_STAGE_2ND, DroneIO.RTL_STAGE_3RD]
        in_is_arv_tgt_alt = [False, True, True, True, True, True, True]
        in_read_pos = [False, False, False, True, False, True, False]
        in_is_rtl_req = [False, False, False, False, True, True, True]

        for cnt in range(len(in_rtl_stg)):
            self.m._rtl_stage = in_rtl_stg[cnt] 

            if self.m._rtl_stage == DroneIO.RTL_STAGE_1ST:
                rtn_is_arv_tgt_alt = self.m._is_arv_tgt_alt()
                rtn_is_arv_tgt_alt = in_is_arv_tgt_alt[cnt]
                if rtn_is_arv_tgt_alt is True:
                    self.m._rtl_stage = DroneIO.RTL_STAGE_2ND
                    point = LocationGlobalRelative(
                        self.m._d_outGPSHome.latitude,
                        self.m._d_outGPSHome.longitude,
                        self.m._tgt_alt)
                    #self.m._vehicle.simple_goto(point)

            elif self.m._rtl_stage == DroneIO.RTL_STAGE_2ND:
                read_pos = self.m._is_read_info()
                read_pos = in_read_pos[cnt]
                if read_pos:
                    rtn_is_rtl_req = self.m._is_rtl_req()
                    rtn_is_rtl_req = in_is_rtl_req[cnt]
                    if rtn_is_rtl_req:
                        self.m._rtl_stage = DroneIO.RTL_STAGE_3RD
                        #self.m._start_rtl()

            else:
                pass

    def test_is_arv_tgt_alt(self):

        self.m._tgt_alt = 10.0
        in_drone_alt = [9.4, 9.5, 9.6, 10.0, 10.4, 10.5, 10.6]
        out_rtn = [False, True, True, True, True, True, False]

        for cnt in range(len(in_drone_alt)):
            self.m._d_outGPSDrone.altitude = in_drone_alt[cnt]
            rtn = self.m._is_arv_tgt_alt()
            self.assertEqual(rtn, out_rtn[cnt])

    def test_move_to_target_pos(self):
        in_curr_lat = [37.5225155, 37.5225155, 37.5225155, 37.5225155, 37.5225155, 37.5225155, 37.5225155, 37.5225155]
        in_curr_lon = [139.9385735, 139.9385735, 139.9385735, 139.9385735, 139.9385735, 139.9385735, 139.9385735, 139.9385735]
        # in_curr_alt = [4.89, 4.9, 4.91, 4.9, 4.9, 4.9, 4.9, 4.9]
        in_curr_alt = [4.79, 4.8, 4.81, 4.8, 4.8, 4.8, 4.8, 4.8]
        in_tar_lat = [37.5225335, 37.5225335, 37.5225335, 37.5225336, 37.5229265, 37.5225335, 37.5225336, 37.5229265]
        in_tar_lon = [139.9385734, 139.9385734, 139.9385734, 139.9385734, 139.9385734, 139.9385734, 139.9385734, 139.9385734]
        in_tar_alt = [5.0, 5.0, 5.0, 5.0, 5.0, 6.0, 6.0, 6.0]
        out_state = [DroneIO.STS_ALT_CHG, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING, DroneIO.STS_IN_MOVE, DroneIO.STS_IN_MOVE, DroneIO.STS_ALT_CHG, DroneIO.STS_IN_MOVE, DroneIO.STS_IN_MOVE]

        vv = Mock()
        self.m._vehicle = vv.vehicle

        for cnt in range(len(in_curr_lat)):
            self.m._d_outGPSDrone.latitude = in_curr_lat[cnt]
            self.m._d_outGPSDrone.longitude = in_curr_lon[cnt]
            self.m._d_outGPSDrone.altitude = in_curr_alt[cnt]
            self.m._d_inTarget.pos.latitude = in_tar_lat[cnt]
            self.m._d_inTarget.pos.longitude = in_tar_lon[cnt]
            self.m._d_inTarget.pos.altitude = in_tar_alt[cnt]
            self.m._move_to_target_pos()
            self.assertEqual(self.m._drone_sts, out_state[cnt])

    def test_move_to_target_yaw(self):
        in_yaw_dir = [1, -1, 0]
        out_rtn = [True, True, False]

        self.m._d_inTarget.yaw.data = [0, 0]
        
        for cnt in range(len(in_yaw_dir)):
            rtn = False
            self.m._d_inTarget.yaw.data[0] = in_yaw_dir[cnt]
            if self.m._d_inTarget.yaw.data[0] in [DroneIO.YAW_TURN_RIGHT, DroneIO.YAW_TURN_LEFT]:
                #self.m._condition_yaw()
                rtn = True
            
            self.assertEqual(rtn, out_rtn[cnt])

    def test_write_drone_ctrl(self):

        self.m._d_outDroneCtrl = RTC.DroneCtrl(RTC.Time(0,0),
                                             0,
                                             RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.TimedFloatSeq(RTC.Time(0,0),[0.0, 0.0, 0.0]),
                                             RTC.TimedFloat(RTC.Time(0,0),0),
                                             RTC.TimedString(RTC.Time(0,0),0))
        
        in_gps_home = [False, True, True]
        in_gps_drone = [False, False, True]
        in_read_mag = [False, False, True]
        in_read_accel = [False, False, True]

        in_gps_home_lat = [37.522614, 37.522614, 37.522614]
        in_gps_home_lon = [139.938675, 139.938675, 139.938675]
        in_gps_home_alt = [208.1, 208.1, 208.1]
        in_gps_drone_lat = [37.522794, 37.522794, 37.522794]
        in_gps_drone_lon = [139.938675, 139.938675, 139.938675]
        in_gps_drone_alt = [13.4, 13.4, 13.4]
        in_mag = [349.0, 349.0, 349.0]
        in_accel = [[2.38, 1.57, 0.1], [2.38, 1.57, 0.1], [2.38, 1.57, 0.1]]
        in_state = [DroneIO.STS_HOVERING, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING]

        out_id = [1, 2, 3]
        out_gps_home_lat = [0.0, 0.0, 37.522614]
        out_gps_home_lon = [0.0, 0.0, 139.938675]
        out_gps_home_alt = [0.0, 0.0, 208.1]
        out_gps_drone_lat = [0.0, 0.0, 37.522794]
        out_gps_drone_lon = [0.0, 0.0, 139.938675]
        out_gps_drone_alt = [0.0, 0.0, 13.4]
        out_mag = [0.0, 0.0, 349.0]
        out_accel = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [2.38, 1.57, 0.1]]
        out_state = [DroneIO.STS_HOVERING, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING]

        for cnt in range(len(in_gps_home)):
            self.m._gps_home = in_gps_home[cnt]
            self.m._gps_drone = in_gps_drone[cnt]
            self.m._mag = in_read_mag[cnt]
            self.m._accel = in_read_accel[cnt]
            self.m._d_outGPSHome.latitude = in_gps_home_lat[cnt]
            self.m._d_outGPSHome.longitude = in_gps_home_lon[cnt]
            self.m._d_outGPSHome.altitude = in_gps_home_alt[cnt]
            self.m._d_outGPSDrone.latitude = in_gps_drone_lat[cnt]
            self.m._d_outGPSDrone.longitude = in_gps_drone_lon[cnt]
            self.m._d_outGPSDrone.altitude = in_gps_drone_alt[cnt]
            self.m._d_outMag.data = in_mag[cnt]
            self.m._d_outAccel.data = in_accel[cnt]
            self.m._d_outState.data = in_state[cnt]
            self.m._write_drone_ctrl()

            self.assertEqual(self.m._d_outDroneCtrl.id, out_id[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.homeGPS.latitude, out_gps_home_lat[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.homeGPS.longitude, out_gps_home_lon[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.homeGPS.altitude, out_gps_home_alt[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.droneGPS.latitude, out_gps_drone_lat[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.droneGPS.longitude, out_gps_drone_lon[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.droneGPS.altitude, out_gps_drone_alt[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.droneGPS.altitude, out_gps_drone_alt[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.mag.data, out_mag[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.accel.data, out_accel[cnt])
            self.assertEqual(self.m._d_outDroneCtrl.state.data, out_state[cnt])

    def test_write_map(self):
    
        self._d_outDroneMap = RTC.DroneMap(RTC.Time(0,0),
                                           RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                       0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                       0.0, 0, RTC.GPS_FIX_NORMAL),
                                           RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                                       0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                       0.0, 0, RTC.GPS_FIX_NORMAL),
                                           RTC.TimedFloat(RTC.Time(0,0),0),
                                           RTC.TimedString(RTC.Time(0,0),0))
        
        in_gps_drone = [False, True, True]
        in_read_mag = [False, False, True]

        in_gps_drone_lat = [37.522794, 37.522794, 37.522794]
        in_gps_drone_lon = [139.938675, 139.938675, 139.938675]
        in_gps_drone_alt = [13.4, 13.4, 13.4]
        in_gps_target_lat = [37.522800, 37.522800, 37.522800]
        in_gps_target_lon = [139.938675, 139.938675, 139.938675]
        in_gps_target_alt = [13.4, 13.4, 13.4]
        in_mag = [349.0, 349.0, 349.0]
        in_state = [DroneIO.STS_HOVERING, DroneIO.STS_HOVERING, DroneIO.STS_HOVERING]

        out_gps_drone_lat = [0.0, 0.0, 37.522794]
        out_gps_drone_lon = [0.0, 0.0, 139.938675]
        out_gps_drone_alt = [0.0, 0.0, 13.4]
        out_gps_target_lat = [0.0, 0.0, 37.522800]
        out_gps_target_lon = [0.0, 0.0, 139.938675]
        out_gps_target_alt = [0.0, 0.0, 13.4]
        out_mag = [0.0, 0.0, 349.0]
        out_state = [0, 0, DroneIO.STS_HOVERING]

        for cnt in range(len(in_gps_drone)):
            self.m._gps_drone = in_gps_drone[cnt]
            self.m._mag = in_read_mag[cnt]
            self.m._d_outGPSDrone.latitude = in_gps_drone_lat[cnt]
            self.m._d_outGPSDrone.longitude = in_gps_drone_lon[cnt]
            self.m._d_outGPSDrone.altitude = in_gps_drone_alt[cnt]
            self.m._d_inTarget.pos.latitude = in_gps_target_lat[cnt]
            self.m._d_inTarget.pos.longitude = in_gps_target_lon[cnt]
            self.m._d_inTarget.pos.altitude = in_gps_target_alt[cnt]
            self.m._d_outMag.data = in_mag[cnt]
            self.m._d_outState.data = in_state[cnt]
            self.m._write_map()

            self.assertEqual(self.m._d_outDroneMap.droneGPS.latitude, out_gps_drone_lat[cnt])
            self.assertEqual(self.m._d_outDroneMap.droneGPS.longitude, out_gps_drone_lon[cnt])
            self.assertEqual(self.m._d_outDroneMap.droneGPS.altitude, out_gps_drone_alt[cnt])
            self.assertEqual(self.m._d_outDroneMap.targetGPS.latitude, out_gps_target_lat[cnt])
            self.assertEqual(self.m._d_outDroneMap.targetGPS.longitude, out_gps_target_lon[cnt])
            self.assertEqual(self.m._d_outDroneMap.targetGPS.altitude, out_gps_target_alt[cnt])
            self.assertEqual(self.m._d_outDroneMap.mag.data, out_mag[cnt])
            self.assertEqual(self.m._d_outDroneMap.state.data, out_state[cnt])

    def test_write_db(self):
    
        vv = Mock()
        self.m._vehicle = vv.vehicle
        
        in_read_gps_home = [False, True]
        in_read_gps_drone = [False, True]
        in_read_battV = [False, True]
        in_read_battA = [False, True]
        in_read_gyro = [False, True]
        in_read_accel = [False, True]
        in_read_mag = [False, True]
        in_read_temp = [False, True]
        in_read_pres = [False, True]

        in_robotID= ["drone1", "drone1"]
        in_gps_home_lat = [100.0, 37.522530]
        in_gps_home_lon = [200.0, 139.938615]
        in_gps_home_alt = [0.0, 216.650000]
        in_gps_drone_lat = [100.0, 37.522532]
        in_gps_drone_lon = [200.0, 139.938613]
        in_gps_drone_alt = [0.0, 9.990000]
        in_battV = [0.0, 15.559000]
        in_battA = [0.0, 0.0]
        in_flttime = [0, 16]
        in_gyro_r = [0, -0.00975905917585]
        in_gyro_p = [0, 0.0214286409318]
        in_gyro_y = [0, 1.79124319553]
        in_accel = [[0.0, 0.0, 0.0], [-0.06, -0.08, 0.01]]
        in_mag = [0.0, 102.0]
        in_temp = [0.0, 38.48]
        in_pres = [0.0, 980.204772949]
        in_fltMode = ["GUIDED", "GUIDED"]
        in_state = ["Hovering", "Hovering"]
        in_sysErr = ["", ""]
        in_rawIMU = [[0, 0, 0, 0, 0, 0, 0, 0, 0], [35, 8, -1011, -17, -9, 41, -193, -451, 603]]
        in_rawIMU_xacc = [0, 35]
        in_rawIMU_yacc = [0, 8]
        in_rawIMU_zacc = [0, -1011]
        in_rawIMU_xgyro = [0, -17]
        in_rawIMU_ygyro = [0, -9]
        in_rawIMU_zgyro = [0, 41]
        in_rawIMU_xmag = [0, -193]
        in_rawIMU_ymag = [0, -451]
        in_rawIMU_zmag = [0, 603]
        in_correctIMU = [[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [-0.06, -0.08, 0.01, -0.00975905917585, 0.0214286409318, 1.79124319553, 102.0]]

        self.m._d_outDroneDB = RTC.DroneDB(RTC.Time(0, 0),
                                        RTC.TimedString(RTC.Time(0, 0), ""),
                                        RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                    100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                    100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedULong(RTC.Time(0, 0), 0),
                                        RTC.TimedOrientation3D(RTC.Time(0, 0),RTC.Orientation3D(0.0, 0.0, 0.0)),
                                        RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedString(RTC.Time(0, 0), ""),
                                        RTC.TimedString(RTC.Time(0, 0), ""),
                                        RTC.TimedString(RTC.Time(0, 0), ""),
                                        RTC.TimedLongSeq(RTC.Time(0, 0), [0, 0, 0, 0, 0, 0, 0, 0, 0]),
                                        RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]))

        for cnt in range(len(in_read_gps_home)):
            self.m._gps_home = in_read_gps_home[cnt]
            self.m._gps_drone = in_read_gps_drone[cnt]
            self.m._battV = in_read_battV[cnt]
            self.m._battA = in_read_battA[cnt]
            self.m._gyro = in_read_gyro[cnt]
            self.m._accel = in_read_accel[cnt]
            self.m._mag = in_read_mag[cnt]
            self.m._temp = in_read_temp[cnt]
            self.m._pres = in_read_pres[cnt]
            
            self.m._RobotID[0] = in_robotID[cnt]
            self.m._d_outGPSHome.latitude = in_gps_home_lat[cnt]
            self.m._d_outGPSHome.longitude = in_gps_home_lon[cnt]
            self.m._d_outGPSHome.altitude = in_gps_home_alt[cnt]
            self.m._d_outGPSDrone.latitude = in_gps_drone_lat[cnt]
            self.m._d_outGPSDrone.longitude = in_gps_drone_lon[cnt]
            self.m._d_outGPSDrone.altitude = in_gps_drone_alt[cnt]
            self.m._d_outBattV.data = in_battV[cnt]
            self.m._d_outBattA.data = in_battA[cnt]
            self.m._d_outFltTime.sec = in_flttime[cnt]
            self.m._d_outGyro.data.r = in_gyro_r[cnt]
            self.m._d_outGyro.data.p = in_gyro_p[cnt]
            self.m._d_outGyro.data.y = in_gyro_y[cnt]
            self.m._d_outAccel.data = in_accel[cnt]
            self.m._d_outMag.data = in_mag[cnt]
            self.m._d_outTemp.data = in_temp[cnt]
            self.m._d_outPres.data = in_pres[cnt]
            self.m._d_outFltMode.data = in_fltMode[cnt]
            self.m._d_outState.data = in_state[cnt]
            self.m._d_outSysErr.data = in_sysErr[cnt]
            # self.m._vehicle.raw_imu = in_rawIMU[cnt]
            self.m._vehicle.raw_imu.xacc = in_rawIMU_xacc[cnt]
            self.m._vehicle.raw_imu.yacc = in_rawIMU_yacc[cnt]
            self.m._vehicle.raw_imu.zacc = in_rawIMU_zacc[cnt]
            self.m._vehicle.raw_imu.xgyro = in_rawIMU_xgyro[cnt]
            self.m._vehicle.raw_imu.ygyro = in_rawIMU_ygyro[cnt]
            self.m._vehicle.raw_imu.zgyro = in_rawIMU_zgyro[cnt]
            self.m._vehicle.raw_imu.xmag = in_rawIMU_xmag[cnt]
            self.m._vehicle.raw_imu.ymag = in_rawIMU_ymag[cnt]
            self.m._vehicle.raw_imu.zmag = in_rawIMU_zmag[cnt]
            
            self.m._write_db()

            self.assertEqual(self.m._d_outDroneDB.robotID.data, in_robotID[cnt])
            self.assertEqual(self.m._d_outDroneDB.homeGPS.latitude, in_gps_home_lat[cnt])
            self.assertEqual(self.m._d_outDroneDB.homeGPS.longitude, in_gps_home_lon[cnt])
            self.assertEqual(self.m._d_outDroneDB.homeGPS.altitude, in_gps_home_alt[cnt])
            self.assertEqual(self.m._d_outDroneDB.droneGPS.latitude, in_gps_drone_lat[cnt])
            self.assertEqual(self.m._d_outDroneDB.droneGPS.longitude, in_gps_drone_lon[cnt])
            self.assertEqual(self.m._d_outDroneDB.droneGPS.altitude, in_gps_drone_alt[cnt])
            self.assertEqual(self.m._d_outDroneDB.battV.data, in_battV[cnt])
            self.assertEqual(self.m._d_outDroneDB.battA.data, in_battA[cnt])
            self.assertEqual(self.m._d_outDroneDB.fltTime.data, in_flttime[cnt])
            self.assertEqual(self.m._d_outDroneDB.gyro.data.r, in_gyro_r[cnt])
            self.assertEqual(self.m._d_outDroneDB.gyro.data.p, in_gyro_p[cnt])
            self.assertEqual(self.m._d_outDroneDB.gyro.data.y, in_gyro_y[cnt])
            self.assertEqual(self.m._d_outDroneDB.accel.data, in_accel[cnt])
            self.assertEqual(self.m._d_outDroneDB.mag.data, in_mag[cnt])
            self.assertEqual(self.m._d_outDroneDB.temp.data, in_temp[cnt])
            self.assertEqual(self.m._d_outDroneDB.pres.data, in_pres[cnt])
            self.assertEqual(self.m._d_outDroneDB.fltMode.data, in_fltMode[cnt])
            self.assertEqual(self.m._d_outDroneDB.state.data, in_state[cnt])
            self.assertEqual(self.m._d_outDroneDB.sysErr.data, in_sysErr[cnt])
            self.assertEqual(self.m._d_outDroneDB.rawIMU.data, in_rawIMU[cnt])
            self.assertEqual(self.m._d_outDroneDB.correctIMU.data, in_correctIMU[cnt])


    def test_is_valid_lat(self):

        data = [-90.1, -90.0, -89.9, 0.0, 89.9, 90.0, 90.1]
        result = [False, True, True, True, True, True, False]

        for i, dat in enumerate(data):
            rtn = self.m._is_valid_lat(dat)
            self.assertEqual(rtn, result[i])

    def test_is_valid_lon(self):

        data = [-180.1, -180.0, -179.9, 0.0, 179.9, 180.0, 180.1]
        result = [False, True, True, True, True, True, False]

        for i, dat in enumerate(data):
            rtn = self.m._is_valid_lon(dat)
            self.assertEqual(rtn, result[i])

    def test_is_valid_alt(self):

        in_sts = [DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_RTL, DroneIO.STS_IN_MOVE, DroneIO.STS_ALT_CHG, DroneIO.STS_HOVERING, DroneIO.STS_YAW_CHG,
                DroneIO.STS_ARMED, DroneIO.STS_TAKEOFF, DroneIO.STS_RTL, DroneIO.STS_IN_MOVE, DroneIO.STS_ALT_CHG, DroneIO.STS_HOVERING, DroneIO.STS_YAW_CHG]
        in_alt = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0]
        result = [True, False, False, False, False, False, False,
                True, True, True, True, True, True, True]

        for cnt in range(len(in_sts)):
            self.m._drone_sts = in_sts[cnt]
            rtn = self.m._is_valid_alt(in_alt[cnt])
            self.assertEqual(rtn, result[cnt])

    def test_is_valid_yaw_deg(self):

        data = [-0.1, 0.0, 0.1, 90.0, 359.9, 360.0, 360.1]
        result = [False, True, True, True, True, True, False]

        for i, dat in enumerate(data):
            rtn = self.m._is_valid_yaw_deg(dat)
            self.assertEqual(rtn, result[i])
    """

if __name__ == '__main__':
    unittest.main()
