"""
 @file drone_view.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
from kivy.app import App
from kivy.uix.widget import Widget
from kivy.graphics import Color, Line, Triangle
from kivy.uix.label import Label
from kivy.properties import ObjectProperty, StringProperty, NumericProperty
from kivy.core.window import Window
from kivy.clock import Clock
import math

# window size
WIN_X = 700
WIN_Y = 500

# view size
VIEW_Y = 230


ROLL = 0
PITCH = 0
YAW = 0
YAW_Line = []
YAW_Line = 365 * [0]
YAW_Label = []
YAW_Label = 10 * [0]
ROLL_Label = 0
ROLL_Line = []
ROLL_Line = 365 * [0]
PITCH_Line1 = []
PITCH_Line1 = 365 * [0]
PITCH_Label1 = []
PITCH_Label1 = 365 * [0]
PITCH_Line2 = []
PITCH_Line2 = 365 * [0]
PITCH_Label2 = []
PITCH_Label2 = 365 * [0]
ROLL_Label = []
ROLL_Label = 365 * [0]
ROLL_Label_ = 0

# global variable change class


class FlightInfo:

    fltMode = "---"
    mvMode = "---"
    state = "---"
    battV = 0.0
    battA = 0.0
    fltTime = 0

    # Flight Mode
    @classmethod
    def set_fltMode(cls, val):
        cls.fltMode = val

    @classmethod
    def get_fltMode(cls):
        return cls.fltMode

    # Move Mode
    @classmethod
    def set_mvMode(cls, val):
        cls.mvMode = val

    @classmethod
    def get_mvMode(cls):
        return cls.mvMode

    # State
    @classmethod
    def set_state(cls, val):
        cls.state = val

    @classmethod
    def get_state(cls):
        return cls.state

    # battery - voltage
    @classmethod
    def set_batt(cls, voltage, current):
        cls.battV = voltage
        cls.battA = current

    @classmethod
    def get_batt(cls):
        return cls.battV, cls.battA

    # Flight Time
    @classmethod
    def set_fltTime(cls, val):
        cls.fltTime = val

    @classmethod
    def get_fltTime(cls):
        return cls.fltTime


class HeightValue:

    # max value&position value
    max_val = 50
    pos = 0

    # max value setter getter
    @classmethod
    def set_max_val(cls, max_val):
        cls.max_val = max_val

    @classmethod
    def get_max_val(cls):
        return cls.max_val

    # position value setter getter
    @classmethod
    def set_pos(cls, pos):
        cls.pos = pos

    @classmethod
    def get_pos(cls):
        return cls.pos


class GPSInfo:

    dist = 0.0
    home_lat = 100.0
    home_lon = 200.0
    home_alt = 0.0
    drone_lat = 100.0
    drone_lon = 200.0
    drone_alt = 0.0

    @classmethod
    def set_dist(cls, val):
        cls.dist = val

    @classmethod
    def get_dist(cls):
        return cls.dist

    @classmethod
    def set_home_pos(cls, lat, lon, alt):
        cls.home_lat = lat
        cls.home_lon = lon
        cls.home_alt = alt

    @classmethod
    def get_home_pos(cls):
        return cls.home_lat, cls.home_lon, cls.home_alt

    @classmethod
    def set_drone_pos(cls, lat, lon, alt):
        cls.drone_lat = lat
        cls.drone_lon = lon
        cls.drone_alt = alt

    @classmethod
    def get_drone_pos(cls):
        return cls.drone_lat, cls.drone_lon, cls.drone_alt


class GyroInfo:

    gyro_r = 0.0
    gyro_p = 0.0
    gyro_y = 0.0

    mag = 0

    @classmethod
    def set_gyro(cls, r, p, y):
        cls.gyro_r = r
        cls.gyro_p = p
        cls.gyro_y = y

    @classmethod
    def get_gyro(cls):
        return cls.gyro_r, cls.gyro_p, cls.gyro_y

    @classmethod
    def set_mag(cls, mag):
        cls.mag = mag

    @classmethod
    def get_mag(cls):
        return cls.mag


class SpeedInfo:

    accX = 0.0
    accY = 0.0
    accZ = 0.0
    targetSpd = 0.0
    settingSpd = 0.0

    @classmethod
    def set_accel(cls, x, y, z):
        cls.accX = x
        cls.accY = y
        cls.accZ = z

    @classmethod
    def get_accel(cls):
        return cls.accX, cls.accY, cls.accZ

    @classmethod
    def set_tgtSpd(cls, target):
        cls.targetSpd = target

    @classmethod
    def get_tgtSpd(cls):
        return cls.targetSpd

    @classmethod
    def set_settingSpd(cls, setting):
        cls.settingSpd = setting

    @classmethod
    def get_settingSpd(cls):
        return cls.settingSpd


class ErrInfo:
    
    sysErr = ""

    @classmethod
    def set_sysErr(cls, msg):
        cls.sysErr = msg

    @classmethod
    def get_sysErr(cls):
        return cls.sysErr


class BaroInfo:
    
    temp = 0.0
    pres = 0.0

    @classmethod
    def set_baro(cls, temp, pres):
        cls.temp = temp
        cls.pres = pres

    @classmethod
    def get_baro(cls):
        return cls.temp, cls.pres


class Arrow:
    
    # arrow move
    def arrow_move(self, max, min, pos):
        ret = math.fabs(pos + math.fabs(min)) / \
            (math.fabs(max) + math.fabs(min))
        return ret

    # arrow margin bottom
    def margin_bottom(self):
        # ret = ((WIN_Y - VIEW_Y) / 2) + 50
        ret = (WIN_Y - VIEW_Y) / 2
        return ret

    # get position value
    def position_value(self, pos):
        ret = pos
        return ret


class Scale:

    # get max value
    def max_value(self, max_val):
        ret = max_val
        return ret

    # get min value
    def min_value(self, max_val):
        ret = max_val / -2
        return ret

    # get middle value
    def middle_value(self, max_val):
        ret = max_val / 2
        return ret


# view class
class RootWidget(Widget):
#class RootWidget(TabbedPanel):
    
    # parts
    fltMode = StringProperty('---')
    mvMode = StringProperty('---')
    state = StringProperty('---')
    battV = StringProperty('0.0')
    battA = StringProperty('0.0')
    fltTime = StringProperty('0:00')
    sysErr = StringProperty('')

    dist = StringProperty('0.0')
    home_lat = StringProperty('0.0')
    home_lon = StringProperty('0.0')
    home_alt = StringProperty('0.0')
    drone_lat = StringProperty('0.0')
    drone_lon = StringProperty('0.0')
    drone_alt = StringProperty('0.0')

    gyro_r = StringProperty('0.0')
    gyro_p = StringProperty('0.0')
    gyro_y = StringProperty('0.0')
    mag = StringProperty('0')

    acc_x = StringProperty('0.0')
    acc_y = StringProperty('0.0')
    acc_z = StringProperty('0.0')

    _acc_meter = NumericProperty(150)
    _kmh_str = StringProperty('0.0')
    _ms_str = StringProperty('0.0')
    targetSpd = StringProperty('0.0')
    settingSpd = StringProperty('0.0')

    temp = StringProperty('0.0')
    pres = StringProperty('0.0')

    ruler = Scale()
    bow = Arrow()
    scale = ObjectProperty(None)
    max = ObjectProperty(None)
    min = ObjectProperty(None)
    middle = ObjectProperty(None)
    home = ObjectProperty(None)
    arrow = ObjectProperty(None)
    position = ObjectProperty(None)

    fltInfo = FlightInfo()
    heightVal = HeightValue()
    errInfo = ErrInfo()
    gpsInfo = GPSInfo()
    spdInfo = SpeedInfo()
    baroInfo = BaroInfo()

    # viewer execution
    def __init__(self):
        super(RootWidget, self).__init__()
        event = Clock.schedule_interval(self.view_update, 0.1)

    # view update
    def view_update(self, dt):
        self._fltInfo_update()
        self._errInfo_update()
        self._gpsInfo_update()
        self._spdInfo_update()
        self._baroInfo_update()
        self.scale_update()
        self.arrow_update()

        # ===== RPY_Viewer_RTC start =====
        global ROLL, PITCH, YAW, YAW_Line, PITCH_Line
        gyro = GyroInfo()
        r, p, y = gyro.get_gyro()
        ROLL = math.degrees(r)
        PITCH = math.degrees(p)
        YAW = math.degrees(y)
        self.gyro_r = str(r)
        self.gyro_p = str(p)
        self.gyro_y = str(y)
        mag = gyro.get_mag()
        self.mag = str(int(mag))

        if self.tabPanel.current_tab.text == 'Quick':

            HUD_R = 0.2
            HUD_G = 1.0
            HUD_B = 0.80

            RAD_ROLL = math.radians(ROLL)
            Roll_C = math.cos(RAD_ROLL)
            Roll_S = math.sin(RAD_ROLL)
            RAD_ROLL2 = math.radians(ROLL + 180)
            Roll_C2 = math.cos(RAD_ROLL2)
            Roll_S2 = math.sin(RAD_ROLL2)

            # ####################################################################################
            # YAW
            # ####################################################################################
            Y_light_K = 1.0
            Y_light_LK = 1
            YAW_W = 300
            YAW_H = 375
            with self.canvas.after:
                self.canvas.after.clear()

                Color(1, 1, 0)
                ER_ = 5
                Rotate_YAW = 0
                for i in xrange(0, 36):
                    if math.cos(math.radians(YAW + Rotate_YAW)) >= -0.01:
                        K = math.cos(math.radians(YAW + Rotate_YAW) * Y_light_K)
                        LK = math.cos(math.radians(YAW + Rotate_YAW) * Y_light_LK)
                        if LK < 0.6:
                            LK = 0.6
                        YAW_X = YAW_W + 100 * math.sin(math.radians((YAW + Rotate_YAW)))
                        YAW_Y = YAW_H

                        if i == 0:  # N
                            YAW_Label[0] = Label(group="pitch_Line", text=str(" N "), pos=(
                                YAW_X - 50, YAW_Y - 25), halign='left', text_size=[None, 0], font_size=15, color=[LK, LK, LK, 1])
                            Color(1 * K, 0, 0)
                            YAW_Line[i] = Line(group="yaw_Line", points=[
                                            YAW_X, YAW_Y, YAW_X, YAW_Y + 10], width=2)

                        if i == 9:  # N
                            YAW_Label[1] = Label(group="pitch_Line", text=str(" W "), pos=(
                                YAW_X - 50, YAW_Y - 25), halign='left', text_size=[None, 0], font_size=15, color=[LK, LK, LK, 1])
                            Color(1 * K, 0, 0)
                            YAW_Line[i] = Line(group="yaw_Line", points=[
                                            YAW_X, YAW_Y, YAW_X, YAW_Y + 10], width=2)

                        if i == 18:  # N
                            YAW_Label[2] = Label(group="pitch_Line", text=str(" S "), pos=(
                                YAW_X - 50, YAW_Y - 25), halign='left',  text_size=[None, 0], font_size=15, color=[LK, LK, LK, 1])
                            Color(1 * K, 0, 0)
                            YAW_Line[i] = Line(group="yaw_Line", points=[
                                            YAW_X, YAW_Y, YAW_X, YAW_Y + 10], width=2)

                        if i == 27:  # N
                            YAW_Label[3] = Label(group="pitch_Line", text=str(" E "), pos=(
                                YAW_X - 50, YAW_Y - 25), halign='left', text_size=[None, 0], font_size=15, color=[LK, LK, LK, 1])
                            Color(1 * K + 0.2, 0, 0)
                            YAW_Line[i] = Line(group="yaw_Line", points=[
                                            YAW_X, YAW_Y, YAW_X, YAW_Y + 10], width=2)

                        if i != 0 and i != 9 and i != 18 and i != 27:
                            Color(HUD_R, HUD_G, HUD_B)
                            Color(HUD_R * K, HUD_G * K, HUD_B * K)
                            YAW_Line[i] = Line(group="yaw_Line", points=[
                                            YAW_X, YAW_Y, YAW_X, YAW_Y + 10], width=1)

                    Rotate_YAW = Rotate_YAW + 10

                if YAW < 0:
                    YAW += 360
                YAW_format_ = '%d' % YAW
                YAW_Label[4] = Label(group="yaw_Line", text=str(YAW_format_), pos=(
                    YAW_W - 50, YAW_H - 75), halign='left', font_size=15, markup=False)

            with self.canvas.after:
                Color(1, 1, 1)
                TRI_H = 5
                TRI_W = 4.5
                YAW_HEIGHT = 365
                TRI_X = 300
                TRI_Y = YAW_HEIGHT + TRI_H
                TRI_X_0 = TRI_X
                TRI_Y_0 = TRI_Y

                TRI_X = 300 + TRI_W
                TRI_Y = YAW_HEIGHT
                TRI_X_1 = TRI_X
                TRI_Y_1 = TRI_Y
                TRI_X = 300 - TRI_W
                TRI_Y = YAW_HEIGHT
                TRI_X_2 = TRI_X
                TRI_Y_2 = TRI_Y
                Triangle(points=[TRI_X_2, TRI_Y_2, TRI_X_1, TRI_Y_1, TRI_X_0, TRI_Y_0])

            # ####################################################################################
            # Pitch
            # ####################################################################################
            PITCH_R_POS_ALIGN_X = 105
            PITCH_L_POS_ALIGN_X = 85
            PITCH_POS_ALIGN_Y = 30

            P_light_K = 1.4
            P_light_LK = 2
            PITCH_W = 95  # 300
            PITCH_H = 0  # 200
            PITCH_Circle = 175
            LK = 0
            LK_FLG = 0

            with self.canvas.after:
                Color(1, 1, 0)
                Rotate_PITCH = -180
                RAD_ROLL = math.radians(ROLL)
                for i in xrange(0, 72):
                    if i == 36:
                        Color(1, 0, 0)
                    if 0.93 < math.cos(math.radians(PITCH + Rotate_PITCH)) < 0.99:
                        K = math.cos(math.radians((PITCH + Rotate_PITCH) * P_light_K))
                        if K < 0 and Rotate_PITCH < 0:
                            K = math.cos(math.radians((PITCH + Rotate_PITCH + 360) * P_light_K))
                            LK_FLG = 1
                        if K < 0 and Rotate_PITCH > 0:
                            K = math.cos(math.radians((PITCH + Rotate_PITCH - 360) * P_light_K))
                            LK_FLG = 2
                        if K < 0.3:
                            K = 0.3

                        # right
                        Color(HUD_R * K, HUD_G * K, HUD_B * K)

                        PITCH_X = PITCH_W
                        PITCH_Y = PITCH_H + PITCH_Circle * math.sin(-math.radians(PITCH + Rotate_PITCH))

                        PITCH_X_ = PITCH_X * Roll_C + PITCH_Y * Roll_S + 200 + PITCH_R_POS_ALIGN_X
                        PITCH_Y_ = - PITCH_X * Roll_S + PITCH_Y * Roll_C + 225 - 10 - PITCH_POS_ALIGN_Y

                        PITCH_X = PITCH_W + 25

                        PITCH_X_L = PITCH_X * Roll_C + PITCH_Y * Roll_S + 200 - 50 + PITCH_R_POS_ALIGN_X
                        PITCH_Y_L = - PITCH_X * Roll_S + PITCH_Y * Roll_C + 225 - 50 - 10 - PITCH_POS_ALIGN_Y
                        LK = math.cos(math.radians(PITCH + Rotate_PITCH) * P_light_LK)

                        if LK_FLG == 1:
                            LK = math.cos(math.radians(PITCH + Rotate_PITCH + 360) * P_light_LK)
                        if LK_FLG == 2:
                            LK = math.cos(math.radians(PITCH + Rotate_PITCH - 360) * P_light_LK)
                        if LK < 0.3:
                            LK = 0.3

                        if Rotate_PITCH % 10 == 0:
                            PITCH_Line1[i] = Line(group="pitch_Line", points=[
                                                PITCH_X_, PITCH_Y_, PITCH_X_ + 10 * math.cos(-RAD_ROLL), PITCH_Y_ + 10 * math.sin(-RAD_ROLL)], width=1.1)
                            PITCH_Label1[i] = Label(group="pitch_Line", text=str(-Rotate_PITCH), pos=(
                                PITCH_X_L, PITCH_Y_L), halign='left', text_size=[None, 0], font_size=15, color=[LK, LK, LK, 1])
                        else:
                            PITCH_Line1[i] = Line(group="pitch_Line", points=[
                                                PITCH_X_, PITCH_Y_, PITCH_X_ + 7 * math.cos(-RAD_ROLL), PITCH_Y_ + 7 * math.sin(-RAD_ROLL)], width=1)
                        # left
                        Color(HUD_R * K, HUD_G * K, HUD_B * K)

                        PITCH_X = PITCH_W
                        PITCH_Y = PITCH_H + PITCH_Circle * math.sin(math.radians(PITCH + Rotate_PITCH))

                        PITCH_X_ = PITCH_X * Roll_C2 + PITCH_Y * Roll_S2 + 200 + PITCH_L_POS_ALIGN_X
                        PITCH_Y_ = - PITCH_X * Roll_S2 + PITCH_Y * Roll_C2 + 225 - 10 - PITCH_POS_ALIGN_Y

                        PITCH_X = PITCH_W + 15

                        PITCH_X_L = PITCH_X * Roll_C2 + PITCH_Y * Roll_S2 + 200 - 50 + PITCH_L_POS_ALIGN_X
                        PITCH_Y_L = - PITCH_X * Roll_S2 + PITCH_Y * Roll_C2 + 225 - 50 - 10 - PITCH_POS_ALIGN_Y
                        LK = math.cos(math.radians(PITCH + Rotate_PITCH) * P_light_LK)

                        if LK_FLG == 1:
                            LK = math.cos(math.radians(PITCH + Rotate_PITCH + 360) * P_light_LK)
                        if LK_FLG == 2:
                            LK = math.cos(math.radians(PITCH + Rotate_PITCH - 360) * P_light_LK)
                        if LK < 0.3:
                            LK = 0.3

                        if Rotate_PITCH % 10 == 0:
                            PITCH_Line2[i] = Line(group="pitch_Line", points=[
                                                PITCH_X_, PITCH_Y_, PITCH_X_ + 10 * math.cos(-RAD_ROLL), PITCH_Y_ + 10 * math.sin(-RAD_ROLL)], width=1.1)
                            PITCH_Label2[i] = Label(group="pitch_Line", text=str(-Rotate_PITCH), pos=(
                                PITCH_X_L, PITCH_Y_L), halign='left', text_size=[None, 0], font_size=15, color=[LK, LK, LK, 1])
                        else:
                            PITCH_X = PITCH_W - 4
                            PITCH_Y = PITCH_H + PITCH_Circle * math.sin(math.radians(PITCH + Rotate_PITCH))

                            PITCH_X_ = PITCH_X * Roll_C2 + PITCH_Y * Roll_S2 + 200 + PITCH_L_POS_ALIGN_X
                            PITCH_Y_ = - PITCH_X * Roll_S2 + PITCH_Y * Roll_C2 + 225 - 10 - PITCH_POS_ALIGN_Y
                            PITCH_Line2[i] = Line(group="pitch_Line", points=[
                                                PITCH_X_, PITCH_Y_, PITCH_X_ + 7 * math.cos(-RAD_ROLL), PITCH_Y_ + 7 * math.sin(-RAD_ROLL)], width=1)

                    Rotate_PITCH = Rotate_PITCH + 5

                ##
                # 0label-1
                ##
                PITCH_X = PITCH_W
                PITCH_Y = PITCH_H + PITCH_Circle * math.sin(-math.radians(0))

                PITCH_X_ = PITCH_X * Roll_C + PITCH_Y * Roll_S + 200 + PITCH_R_POS_ALIGN_X
                PITCH_Y_ = - PITCH_X * Roll_S + PITCH_Y * Roll_C + 225 - 10 - PITCH_POS_ALIGN_Y

                PITCH_X = PITCH_W + 30

                PITCH_X_L = PITCH_X * Roll_C + PITCH_Y * Roll_S + 200 - 50 + PITCH_R_POS_ALIGN_X
                PITCH_Y_L = - PITCH_X * Roll_S + PITCH_Y * Roll_C + 225 - 50 - 10 - PITCH_POS_ALIGN_Y
                Pitch_format_ = '%.1f' % PITCH
                PITCH_Label1[i] = Label(group="pitch_Line", text='[b]' + str(Pitch_format_) + '[/b]', pos=(
                    PITCH_X_L, PITCH_Y_L), halign='left', text_size=[None, 0], font_size=17, markup=True)

                TRI_H = 10
                TRI_W = 3.5
                P1_OFF_X = 9
                PITCH_X = PITCH_W + P1_OFF_X
                PITCH_Y = PITCH_H + PITCH_Circle * math.sin(0)
                PITCH_X_ = PITCH_X * Roll_C + PITCH_Y * Roll_S + 200 + PITCH_R_POS_ALIGN_X
                PITCH_Y_ = - PITCH_X * Roll_S + PITCH_Y * Roll_C + 225 - 10 - PITCH_POS_ALIGN_Y

                TRI_X_1 = PITCH_X * Roll_C + PITCH_Y * Roll_S + 200 + PITCH_R_POS_ALIGN_X
                TRI_Y_1 = - PITCH_X * Roll_S + PITCH_Y * Roll_C + 225 - 10 - PITCH_POS_ALIGN_Y
                TRI_X = PITCH_W - TRI_H + P1_OFF_X
                TRI_Y = PITCH_H + PITCH_Circle * math.sin(-math.radians(0)) + TRI_W
                TRI_X_0 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + PITCH_R_POS_ALIGN_X
                TRI_Y_0 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10 - PITCH_POS_ALIGN_Y
                TRI_X = PITCH_W - TRI_H + P1_OFF_X
                TRI_Y = PITCH_H + PITCH_Circle * math.sin(-math.radians(0)) - TRI_W
                TRI_X_2 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + PITCH_R_POS_ALIGN_X
                TRI_Y_2 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10 - PITCH_POS_ALIGN_Y
                Triangle(points=[TRI_X_2, TRI_Y_2, TRI_X_1, TRI_Y_1, TRI_X_0, TRI_Y_0])

                PITCH_W = 95  # 300
                PITCH_H = 0  # 200
                Rotate_PITCH = -180
                LK_FLG = 0

                # #
                # 0 label-2
                # #
                PITCH_X = PITCH_W
                PITCH_Y = PITCH_H + PITCH_Circle * math.sin(-math.radians(0))

                PITCH_X_ = PITCH_X * Roll_C2 + PITCH_Y * Roll_S2 + 200 + PITCH_L_POS_ALIGN_X
                PITCH_Y_ = - PITCH_X * Roll_S2 + PITCH_Y * Roll_C2 + 225 - 10 - PITCH_POS_ALIGN_Y

                PITCH_X = PITCH_W + 18

                PITCH_X_L = PITCH_X * Roll_C2 + PITCH_Y * Roll_S2 + 200 - 50 + PITCH_L_POS_ALIGN_X
                PITCH_Y_L = - PITCH_X * Roll_S2 + PITCH_Y * Roll_C2 + 225 - 50 - 10 - PITCH_POS_ALIGN_Y
                Pitch_format_ = '%.1f' % PITCH
                PITCH_Label2[i] = Label(group="pitch_Line", text='[b]' + str(Pitch_format_) + '[/b]', pos=(
                    PITCH_X_L, PITCH_Y_L), halign='left', text_size=[None, 0], font_size=17, markup=True)

                PITCH_X = PITCH_W
                PITCH_Y = PITCH_H + PITCH_Circle * math.sin(0)
                PITCH_X_ = PITCH_X * Roll_C2 + PITCH_Y * Roll_S2 + 200 + PITCH_L_POS_ALIGN_X
                PITCH_Y_ = - PITCH_X * Roll_S2 + PITCH_Y * Roll_C2 + 225 - 10 - PITCH_POS_ALIGN_Y
                TRI_X_1 = PITCH_X * Roll_C2 + PITCH_Y * Roll_S2 + 200 + PITCH_L_POS_ALIGN_X
                TRI_Y_1 = - PITCH_X * Roll_S2 + PITCH_Y * Roll_C2 + 225 - 10 - PITCH_POS_ALIGN_Y
                TRI_X = PITCH_W - TRI_H
                TRI_Y = PITCH_H + PITCH_Circle * math.sin(-math.radians(0)) + TRI_W
                TRI_X_0 = TRI_X * Roll_C2 + TRI_Y * Roll_S2 + 200 + PITCH_L_POS_ALIGN_X
                TRI_Y_0 = - TRI_X * Roll_S2 + TRI_Y * Roll_C2 + 225 - 10 - PITCH_POS_ALIGN_Y
                TRI_X = PITCH_W - TRI_H
                TRI_Y = PITCH_H + PITCH_Circle * math.sin(-math.radians(0)) - TRI_W
                TRI_X_2 = TRI_X * Roll_C2 + TRI_Y * Roll_S2 + 200 + PITCH_L_POS_ALIGN_X
                TRI_Y_2 = - TRI_X * Roll_S2 + TRI_Y * Roll_C2 + 225 - 10 - PITCH_POS_ALIGN_Y
                Triangle(points=[TRI_X_2, TRI_Y_2, TRI_X_1, TRI_Y_1, TRI_X_0, TRI_Y_0])

            # ####################################################################################
            # Roll
            # ####################################################################################
            ROLL_POS_ALIGN_X = 100
            ROLL_POS_ALIGN_Y = 30

            ROLL_W = 100 + ROLL_POS_ALIGN_X
            ROLL_H = 0 - ROLL_POS_ALIGN_Y
            ROLL_Circle = 45
            ROLL_L = 3
            with self.canvas.after:
                Color(HUD_R, HUD_G, HUD_B)
                Rotate_ROLL = 0

                PITCH_HEIGHT = 280
                TRI_H = 5
                TRI_W = 4.5

                TRI_X = 200 - 200
                TRI_Y = PITCH_HEIGHT - (225 - 10) + TRI_H
                TRI_X_1 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                TRI_Y_1 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                TRI_X = 200 - 200 + TRI_W
                TRI_Y = PITCH_HEIGHT - (225 - 10)
                TRI_X_0 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                TRI_Y_0 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                TRI_X = 200 - 200 - TRI_W
                TRI_Y = PITCH_HEIGHT - (225 - 10)
                TRI_X_2 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                TRI_Y_2 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y

                Color(1, 1, 1)
                Triangle(points=[TRI_X_2, TRI_Y_2, TRI_X_1, TRI_Y_1, TRI_X_0, TRI_Y_0])

                LK = math.cos(math.radians(PITCH + Rotate_PITCH) * P_light_LK)
                Color(HUD_R, HUD_G, HUD_B)
                for i in xrange(0, 36):
                    ROLL_X = ROLL_W + ROLL_Circle * math.cos(math.radians(Rotate_ROLL)) + 100
                    ROLL_Y = ROLL_H + ROLL_Circle * math.sin(math.radians(Rotate_ROLL)) + 225 - 10
                    ROLL_Line[i] = Line(group="roll_Line",
                                        points=[ROLL_X, ROLL_Y, ROLL_X + ROLL_L * math.cos(math.radians(Rotate_ROLL)),  ROLL_Y + ROLL_L * math.sin(math.radians(Rotate_ROLL))], width=1)
                    Rotate_ROLL = Rotate_ROLL + 10

                Rotate_ROLL = -90
                ROLL_Circle = 70

                for i in xrange(0, 72):
                    if i == 36:
                        Color(1, 0, 0)
                    else:
                        Color(HUD_R, HUD_G, HUD_B)

                    if 0.71 < math.sin(math.radians(ROLL + Rotate_ROLL)) < 0.99:
                        K = math.sin(math.radians((ROLL + Rotate_ROLL - 25) * P_light_K))
                        if K < 0 and Rotate_ROLL < 0:
                            K = math.sin(math.radians((ROLL + Rotate_ROLL - 25 + 360) * P_light_K))
                            LK_FLG = 1
                        if K < 0 and Rotate_ROLL > 0:
                            K = math.sin(math.radians((ROLL + Rotate_ROLL - 25 - 360) * P_light_K))
                            LK_FLG = 2
                        LK = K
                        if K < 0.3:
                            K = 0.3
                        Color(HUD_R * K, HUD_G * K, HUD_B * K)

                        ROLL_X = PITCH_W
                        ROLL_X = PITCH_H

                        ROLL_X = ROLL_W + ROLL_Circle * math.cos(math.radians(Rotate_ROLL)) + 100
                        ROLL_Y = ROLL_H + ROLL_Circle * math.sin(math.radians(Rotate_ROLL)) + 225 - 10

                        if Rotate_ROLL % 10 == 0:
                            ROLL_L = 15
                            ROLL_Line[i] = Line(group="roll_Line", points=[ROLL_X, ROLL_Y, ROLL_X + ROLL_L * math.cos(math.radians(Rotate_ROLL)),
                                                                        ROLL_Y + ROLL_L * math.sin(math.radians(Rotate_ROLL))], width=1)

                            ROLL_X = PITCH_W
                            ROLL_X = PITCH_H
                            ROLL_Circle_L = 95

                            ROLL_X_L = ROLL_W + ROLL_Circle_L * math.cos(math.radians(Rotate_ROLL)) + 100 - 50
                            ROLL_Y_L = ROLL_H + ROLL_Circle_L * math.sin(math.radians(Rotate_ROLL)) + 225 - 50 - 10
                            ROLL_Label[i] = Label(group="roll_Line", text=str(Rotate_ROLL - 90), pos=(
                                ROLL_X_L, ROLL_Y_L), halign='left', text_size=[None, 0], font_size=13, color=[LK, LK, LK, 1])
                        else:
                            ROLL_L = 7.5
                            ROLL_Line[i] = Line(group="roll_Line",
                                                points=[ROLL_X, ROLL_Y, ROLL_X + ROLL_L * math.cos(math.radians(Rotate_ROLL)),  ROLL_Y + ROLL_L * math.sin(math.radians(Rotate_ROLL))], width=1)

                    Rotate_ROLL = Rotate_ROLL + 5

                ROLL_L = 24
                ROLL_X = ROLL_W + ROLL_Circle * math.cos(math.radians(-ROLL + 90)) + 100
                ROLL_Y = ROLL_H + ROLL_Circle * math.sin(math.radians(-ROLL + 90)) + 225 - 10

                ROLL_Line_ = Line(group="roll_Line", points=[ROLL_X, ROLL_Y, ROLL_X + ROLL_L * math.cos(math.radians(-ROLL + 90)),
                                                            ROLL_Y + ROLL_L * math.sin(math.radians(-ROLL + 90))], width=1)
                ROLL_Circle_L = 105

                ROLL_format_ = '%.1f' % -ROLL
                ROLL_X_L = ROLL_W + ROLL_Circle_L * math.cos(math.radians(-ROLL + 90)) + 100 - 50
                ROLL_Y_L = ROLL_H + ROLL_Circle_L * math.sin(math.radians(-ROLL + 90)) + 225 - 50 - 10
                ROLL_Label[i] = Label(group="roll_Line", text='[b]' + str(ROLL_format_) + '[/b]', pos=(
                    ROLL_X_L, ROLL_Y_L), halign='left',  text_size=[None, 0], font_size=15, markup=True)

            #######################################################################
            # Center Line?
            #######################################################################

            with self.canvas.after:
                ##
                # ROLL
                ##
                Color(HUD_R, HUD_G, HUD_B)
                Line(group="Central_Line", points=[200 + ROLL_POS_ALIGN_X, 245 - ROLL_POS_ALIGN_Y, 200 + ROLL_POS_ALIGN_X, 248 - ROLL_POS_ALIGN_Y], width=1)
                Line(group="Central_Line", points=[200 + ROLL_POS_ALIGN_X, 185 - ROLL_POS_ALIGN_Y, 200 + ROLL_POS_ALIGN_X, 182 - ROLL_POS_ALIGN_Y], width=1)
                Line(group="Central_Line", points=[167 + ROLL_POS_ALIGN_X, 215 - ROLL_POS_ALIGN_Y, 170 + ROLL_POS_ALIGN_X, 215 - ROLL_POS_ALIGN_Y], width=1)
                Line(group="Central_Line", points=[230 + ROLL_POS_ALIGN_X, 215 - ROLL_POS_ALIGN_Y, 233 + ROLL_POS_ALIGN_X, 215 - ROLL_POS_ALIGN_Y], width=1)
                ##
                # Rolling Center line
                ##
                PITCH_PLUS = 25 * math.sin(math.radians(PITCH))

                ROLL_CS_LINE_X = 180 - 200
                ROLL_CS_LINE_Y = 215 - (225 - 10) + PITCH_PLUS
                ROLL_CS_LINE_X_ = ROLL_CS_LINE_X * Roll_C + ROLL_CS_LINE_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                ROLL_CS_LINE_Y_ = - ROLL_CS_LINE_X * Roll_S + ROLL_CS_LINE_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                ROLL_CE_LINE_X = 220 - 200
                ROLL_CE_LINE_Y = 215 - (225 - 10) + PITCH_PLUS
                ROLL_CE_LINE_X_ = ROLL_CE_LINE_X * Roll_C + ROLL_CE_LINE_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                ROLL_CE_LINE_Y_ = - ROLL_CE_LINE_X * Roll_S + ROLL_CE_LINE_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                Line(group="Central_Line", points=[ROLL_CS_LINE_X_, ROLL_CS_LINE_Y_, ROLL_CE_LINE_X_, ROLL_CE_LINE_Y_], width=1)

                ROLL_CS_LINE_X = 130 - 200
                ROLL_CS_LINE_Y = 215 - (225 - 10)
                ROLL_CS_LINE_X_ = ROLL_CS_LINE_X * Roll_C + ROLL_CS_LINE_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                ROLL_CS_LINE_Y_ = - ROLL_CS_LINE_X * Roll_S + ROLL_CS_LINE_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                ROLL_CE_LINE_X = 160 - 200
                ROLL_CE_LINE_Y = 215 - (225 - 10)
                ROLL_CE_LINE_X_ = ROLL_CE_LINE_X * Roll_C + ROLL_CE_LINE_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                ROLL_CE_LINE_Y_ = - ROLL_CE_LINE_X * Roll_S + ROLL_CE_LINE_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                Line(group="Central_Line", points=[ROLL_CS_LINE_X_, ROLL_CS_LINE_Y_, ROLL_CE_LINE_X_, ROLL_CE_LINE_Y_], width=1)
                ROLL_CS_LINE_X = 240 - 200
                ROLL_CS_LINE_Y = 215 - (225 - 10)
                ROLL_CS_LINE_X_ = ROLL_CS_LINE_X * Roll_C + ROLL_CS_LINE_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                ROLL_CS_LINE_Y_ = - ROLL_CS_LINE_X * Roll_S + ROLL_CS_LINE_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                ROLL_CE_LINE_X = 270 - 200
                ROLL_CE_LINE_Y = 215 - (225 - 10)
                ROLL_CE_LINE_X_ = ROLL_CE_LINE_X * Roll_C + ROLL_CE_LINE_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                ROLL_CE_LINE_Y_ = - ROLL_CE_LINE_X * Roll_S + ROLL_CE_LINE_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                Line(group="Central_Line", points=[ROLL_CS_LINE_X_, ROLL_CS_LINE_Y_, ROLL_CE_LINE_X_, ROLL_CE_LINE_Y_], width=1)

                Pitch_MEM = -25
                for i in xrange(0, 11):
                    ROLL_CS_LINE_X = 199 - 200
                    if i == 0 or i == 5 or i == 10:
                        ROLL_CS_LINE_X = 197 - 200

                    ROLL_CS_LINE_Y = 215 - (225 - 10) + Pitch_MEM
                    ROLL_CS_LINE_X_ = ROLL_CS_LINE_X * Roll_C + ROLL_CS_LINE_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                    ROLL_CS_LINE_Y_ = - ROLL_CS_LINE_X * Roll_S + ROLL_CS_LINE_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y

                    ROLL_CE_LINE_X = 201 - 200

                    if i == 0 or i == 5 or i == 10:
                        ROLL_CE_LINE_X = 203 - 200

                    ROLL_CE_LINE_Y = 215 - (225 - 10) + Pitch_MEM
                    ROLL_CE_LINE_X_ = ROLL_CE_LINE_X * Roll_C + ROLL_CE_LINE_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                    ROLL_CE_LINE_Y_ = - ROLL_CE_LINE_X * Roll_S + ROLL_CE_LINE_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                    Line(group="Central_Line", points=[ROLL_CS_LINE_X_, ROLL_CS_LINE_Y_, ROLL_CE_LINE_X_, ROLL_CE_LINE_Y_], width=1)
                    Pitch_MEM = Pitch_MEM + 5

                ##
                # Rolling center triangle
                ##
                Color(HUD_R, HUD_G, HUD_B)
                TRI_H = 5
                TRI_W = 4.5
                PITCH_HEIGHT = 390
                TRI_X = 200 - 200
                TRI_Y = 215 - (225 - 10) + TRI_H + PITCH_PLUS
                TRI_X_1 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                TRI_Y_1 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y

                TRI_X = 200 - 200 + TRI_W
                TRI_Y = 215 - (225 - 10) + PITCH_PLUS
                TRI_X_0 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                TRI_Y_0 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                TRI_X = 200 - 200 - TRI_W
                TRI_Y = 215 - (225 - 10) + PITCH_PLUS
                TRI_X_2 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + ROLL_POS_ALIGN_X
                TRI_Y_2 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10 - ROLL_POS_ALIGN_Y
                Triangle(points=[TRI_X_2, TRI_Y_2, TRI_X_1, TRI_Y_1, TRI_X_0, TRI_Y_0])

                ##
                # Rolling pitch's triangle
                ##
                if PITCH >= 0:
                    PITCH_HEIGHT = 320
                    TRI_H = 5.5
                    TRI_W = 20
                    TRI_X = 200 - 200
                    TRI_Y = PITCH_HEIGHT - 10 - (225 - 10)
                    TRI_X_2 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 - 50 + 100
                    TRI_Y_2 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10 - 50
                if PITCH < 0:
                    PITCH_HEIGHT = 125
                    TRI_H = -5.5
                    TRI_W = 20
                TRI_X = 200 - 200
                TRI_Y = PITCH_HEIGHT - (225 - 10) + TRI_H
                TRI_X_1 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + 100
                TRI_Y_1 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10

                TRI_X = 200 - 200 + TRI_W
                TRI_Y = PITCH_HEIGHT - (225 - 10)
                TRI_X_0 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + 100
                TRI_Y_0 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10
                TRI_X = 200 - 200 - TRI_W
                TRI_Y = PITCH_HEIGHT - (225 - 10)
                TRI_X_2 = TRI_X * Roll_C + TRI_Y * Roll_S + 200 + 100
                TRI_Y_2 = - TRI_X * Roll_S + TRI_Y * Roll_C + 225 - 10
                Triangle(points=[TRI_X_2, TRI_Y_2, TRI_X_1, TRI_Y_1, TRI_X_0, TRI_Y_0])

                ##
                # Not rolling center triangle
                ##
                """
                Color( 1, 1, 1)
                TRI_H=5
                TRI_W=4.5
                YAW_HEIGHT=390
                TRI_X=200       
                TRI_Y=YAW_HEIGHT  +TRI_H
                TRI_X_0 =TRI_X
                TRI_Y_0 =TRI_Y 

                TRI_X=200        +TRI_W 
                TRI_Y=YAW_HEIGHT 
                TRI_X_1 =TRI_X
                TRI_Y_1 =TRI_Y 
                TRI_X=200        -TRI_W 
                TRI_Y=YAW_HEIGHT  
                TRI_X_2 =TRI_X
                TRI_Y_2 =TRI_Y 
                Triangle(points=[TRI_X_2 ,TRI_Y_2,    TRI_X_1 ,TRI_Y_1,   TRI_X_0,TRI_Y_0])
                """
            # ===== RPY_Viewer_RTC end =====
        else:
            with self.canvas.after:
                self.canvas.after.clear()

    def _fltInfo_update(self):
        self.fltMode = self.fltInfo.get_fltMode()
        self.mvMode = self.fltInfo.get_mvMode()
        self.state = self.fltInfo.get_state()
        volt, curr = self.fltInfo.get_batt()
        self.battV = str(round(volt, 2))
        self.battA = str(round(curr, 2))
        self.fltTime = str(self.fltInfo.get_fltTime())

    def _errInfo_update(self):
        self.sysErr = self.errInfo.get_sysErr()

    def _gpsInfo_update(self):
        self.dist = str(round(self.gpsInfo.get_dist(), 2))
        hm_lat, hm_lon, hm_alt = self.gpsInfo.get_home_pos()
        dr_lat, dr_lon, dr_alt = self.gpsInfo.get_drone_pos()
        self.home_lat = str(round(hm_lat, 10))
        self.home_lon = str(round(hm_lon, 10))
        self.home_alt = str(round(hm_alt, 2))
        self.drone_lat = str(round(dr_lat, 10))
        self.drone_lon = str(round(dr_lon, 10))
        self.drone_alt = str(round(dr_alt, 2))

    def _spdInfo_update(self):
        x, y, z = self.spdInfo.get_accel()
        self.acc_x = str(round(x, 2))
        self.acc_y = str(round(y, 2))
        self.acc_z = str(round(z, 2))
        # ===== E2_Viewer_RTC start =====
        accel = math.sqrt(x**2 + y**2 + z**2)
        acc_km = (accel * 60 * 60) / 1000
        self._acc_meter = 150 - math.fabs(acc_km) / 72 * 300 
        self._ms_str = str(round(accel, 2))
        self._kmh_str = str(round(((accel * 60 * 60) / 1000), 2))
        # ===== E2_Viewer_RTC end =====

        self.targetSpd = str(round(self.spdInfo.get_tgtSpd(), 2))
        self.settingSpd = str(round(self.spdInfo.get_settingSpd(), 2))

    def _baroInfo_update(self):
        temp, pres = self.baroInfo.get_baro()
        self.temp = str(round(temp, 2))
        self.pres = str(round(pres, 2))

    # scale value update
    def scale_update(self):
        maxVal = self.ruler.max_value(self.heightVal.get_max_val())
        minVal = self.ruler.min_value(maxVal)
        middleVal = self.ruler.middle_value(maxVal)
        self.max.text = str(maxVal)
        self.min.text = str(minVal)
        self.middle.text = str(middleVal)

    # current position update
    def arrow_update(self):
        posVal = self.bow.position_value(self.heightVal.get_pos())
        self.arrow.center_y = self.arrow_center_y()
        self.position.text = str(posVal)
        self.position.center_y = self.arrow.center_y

    # arrow position change
    def arrow_center_y(self):
        maxVal = self.ruler.max_value(self.heightVal.get_max_val())
        minVal = self.ruler.min_value(maxVal)
        posVal = self.bow.position_value(self.heightVal.get_pos())
        ret = VIEW_Y * \
            self.bow.arrow_move(
                maxVal, minVal, posVal) + self.bow.margin_bottom()
        return ret

class DroneInfoWidget(Widget):
    pass


class drone_view(App):

    def build(self):
        self.title = "Drone Viewer"
        Window.size = (WIN_X, 510)
        return RootWidget()

if __name__ == "__main__":
    drone_view().run()
