#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file drone_view_test.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import unittest
import RTC
import OpenRTM_aist
import drone_view
import drone_distance_cal as dr_cal

from mock import Mock

class Testdrone_view(unittest.TestCase):

    def setUp(self):

        self.m = drone_view.RootWidget()

    def test_fltInfo_update(self):

        self.m._fltInfo = drone_view.FlightInfo()
        self.m._fltInfo.set_fltMode("GUIDED")
        self.m._fltInfo.set_mvMode("Absolute")
        self.m._fltInfo.set_state("In_Move")
        self.m._fltInfo.set_batt(15.75, 0.0)
        min = 19 / 60
        sec = 19 % 60
        time_str = "{0:d}:{1:02d}".format(min, sec)
        self.m._fltInfo.set_fltTime(time_str)

        self.m._fltInfo_update()

        self.assertEqual(self.m.fltMode, "GUIDED")
        self.assertEqual(self.m.mvMode, "Absolute")
        self.assertEqual(self.m.state, "In_Move")
        self.assertEqual(self.m.battV, "15.75")
        self.assertEqual(self.m.battA, "0.0")
        self.assertEqual(self.m.fltTime, "0:19")


    def test_errInfo_update(self):
    
        self.m._errInfo = drone_view.ErrInfo()
        self.m._errInfo.set_sysErr("Low-battery detected.")

        self.m._errInfo_update()

        self.assertEqual(self.m.sysErr, "Low-battery detected.")

    def test_gpsInfo_update(self):
    
        h_lat = 37.5223693848
        h_lon = 139.9386749268
        h_alt = 208.09
        dr_lat = 37.5223941
        dr_lon = 139.9386752
        dr_alt = 10.0
        self.m._gpsInfo = drone_view.GPSInfo()
        d = dr_cal.DroneMathmetics()
        dist = d.distance_math(h_lon, dr_lon, h_lat, dr_lat, 0, dr_alt)
        self.m._gpsInfo.set_home_pos(h_lat, h_lon, h_alt)
        self.m._gpsInfo.set_drone_pos(dr_lat, dr_lon, dr_alt)
        self.m._gpsInfo.set_dist(dist)

        self.m._gpsInfo_update()

        self.assertEqual(self.m.home_lat, "37.5223693848")
        self.assertEqual(self.m.home_lon, "139.938674927")
        self.assertEqual(self.m.home_alt, "208.09")
        self.assertEqual(self.m.drone_lat, "37.5223941")
        self.assertEqual(self.m.drone_lon, "139.9386752")
        self.assertEqual(self.m.drone_alt, "10.0")
        self.assertEqual(self.m.dist, "10.37")

    def test_speedInfo_update(self):
        
        self.m._spdInfo = drone_view.SpeedInfo()
        self.m._spdInfo.set_accel(0.61, -0.03, 0.0)
        self.m._spdInfo.set_tgtSpd(3.0)
        self.m._spdInfo.set_settingSpd(3.0)

        self.m._spdInfo_update()

        self.assertEqual(self.m.acc_x, "0.61")
        self.assertEqual(self.m.acc_y, "-0.03")
        self.assertEqual(self.m.acc_z, "0.0")
        self.assertEqual(self.m.targetSpd, "3.0")
        self.assertEqual(self.m.settingSpd, "3.0")

    def test_gyroInfo_update(self):
        
        gyro = drone_view.GyroInfo()
        gyro.set_gyro(-0.000740, -0.048703, -0.19275)
        gyro.set_mag(358.0)

        self.m._spdInfo_update()
        r, p, y = gyro.get_gyro()
        mag = gyro.get_mag()
        self.m.gyro_r = str(r)
        self.m.gyro_p = str(p)
        self.m.gyro_y = str(y)
        self.m.mag = str(int(mag))

        self.assertEqual(self.m.gyro_r, "-0.00074")
        self.assertEqual(self.m.gyro_p, "-0.048703")
        self.assertEqual(self.m.gyro_y, "-0.19275")
        self.assertEqual(self.m.mag, "358")

    def test_baroInfo_update(self):
        
        self.m._baroInfo = drone_view.BaroInfo()
        self.m._baroInfo.set_baro(28.51, 987.38)

        self.m._baroInfo_update()

        self.assertEqual(self.m.temp, "28.51")
        self.assertEqual(self.m.pres, "987.38")


if __name__ == '__main__':
    unittest.main()
