#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file RobotImageUploader.py
 @brief Robot picture and additional-data uploader
 @date $Date$

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import sys
import time

# Import RTM module
import RTC
import OpenRTM_aist

import os
import glob
import re
import csv
import subprocess
import shlex
import pipes
from logging import getLogger, StreamHandler, DEBUG,INFO

sys.path.append(".")

logger = getLogger(__name__)
handler = StreamHandler()
handler.setLevel(INFO)
logger.setLevel(DEBUG)
logger.addHandler(handler)

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
robotimageuploader_spec = ["implementation_id", "RobotImageUploader",
                           "type_name",         "RobotImageUploader",
                           "description",       "Robot picture and additional-data uploader",
                           "version",           "1.0.0",
                           "vendor",            "EAC",
                           "category",          "Category",
                           "activity_type",     "STATIC",
                           "max_instance",      "1",
                           "language",          "Python",
                           "lang_type",         "SCRIPT",
                           "conf.default.UploadPath", "/home/",
                           "conf.default.AddDataFilePath", "/home/pi/",
                           "conf.default.PictureFilePath", "/home/pi/",
                           "conf.default.UserID", "user",
                           "conf.default.HostName", "localhost",
                           "conf.__widget__.UploadPath", "text",
                           "conf.__widget__.AddDataFilePath", "text",
                           "conf.__widget__.PictureFilePath", "text",
                           "conf.__widget__.UserID", "text",
                           "conf.__widget__.HostName", "text",
                           ""]
# </rtc-template>

##
# @class RobotImageUploader
# @brief Robot picture and additional-data uploader
#
# Robot picture and additional-data uploader
#
#


class RobotImageUploader(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_outImgAddData = RTC.RobotImgAddData(RTC.Time(0, 0),
                                                    RTC.TimedString(RTC.Time(0, 0), 0),
                                                    RTC.TimedString(RTC.Time(0, 0), 0),
                                                    RTC.TimedString(RTC.Time(0, 0), 0),
                                                    RTC.TimedString(RTC.Time(0, 0), 0),
                                                    RTC.TimedShortSeq(RTC.Time(0, 0), [0, 0]),
                                                    RTC.TimedString(RTC.Time(0, 0), 0),
                                                    RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0), 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0, RTC.GPS_FIX_NORMAL),
                                                    RTC.TimedOrientation3D(RTC.Time(0, 0), RTC.Orientation3D(0.0, 0.0, 0.0)),
                                                    RTC.TimedString(RTC.Time(0, 0), 0),
                                                    RTC.TimedString(RTC.Time(0, 0), 0),
                                                    RTC.TimedString(RTC.Time(0, 0), 0))
        """
		Images and image additional data captured by the robot.
		"""
        self._outImgAddDataOut = OpenRTM_aist.OutPort("outImgAddData", self._d_outImgAddData)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
        """
		Image upload path
		 - Name:  UploadPath
		 - DefaultValue: /home/
		"""
        self._UploadPath = ['/home/']
        """
		additional-data root path
		 - Name:  AddDataFilePath
		 - DefaultValue: /home/pi/
		"""
        self._AddDataFilePath = ['/home/pi/']
        """
		Image root path
		 - Name:  PictureFilePath
		 - DefaultValue: /home/pi/
		"""
        self._PictureFilePath = ['/home/pi/']
        """
		For image upload location
		 - Name:  UserID
		 - DefaultValue: user
		"""
        self._UserID = ['user']
        """
		For image upload location
		 - Name:  HostName
		 - DefaultValue: localhost
		"""
        self._HostName = ['localhost']

        # </rtc-template>

    ##
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onInitialize(self):
        # Bind variables and configuration variable
        self.bindParameter("UploadPath", self._UploadPath, "/home/")
        self.bindParameter("AddDataFilePath", self._AddDataFilePath, "/home/pi/")
        self.bindParameter("PictureFilePath", self._PictureFilePath, "/home/pi/")
        self.bindParameter("UserID", self._UserID, "user")
        self.bindParameter("HostName", self._HostName, "localhost")

        # Set InPort buffers

        # Set OutPort buffers
        self.addOutPort("outImgAddData", self._outImgAddDataOut)

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        return RTC.RTC_OK

    #	##
    #	#
    #	# The finalize action (on ALIVE->END transition)
    #	# formaer rtc_exiting_entry()
    #	#
    #	# @return RTC::ReturnCode_t
    #
    #	#
    # def onFinalize(self):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The startup action when ExecutionContext startup
    #	# former rtc_starting_entry()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onStartup(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The shutdown action when ExecutionContext stop
    #	# former rtc_stopping_entry()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onShutdown(self, ec_id):
    #
    #	return RTC.RTC_OK

        ##
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onActivated(self, ec_id):

        self._timeBfr = time.time()
        # onExecute counter(data write counter)
        self._exeCount = 0

        # read file list-up
        self._files = sorted(glob.glob(self._AddDataFilePath[0] + "Additional data"+os.sep+"*"+os.sep+"*.csv"), key=self._numAscSort)
        self._file_idx = 0

        return RTC.RTC_OK

    #	##
    #	#
    #	# The deactivated action (Active state exit action)
    #	# former rtc_active_exit()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onDeactivated(self, ec_id):
    #
    #	return RTC.RTC_OK

        ##
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onExecute(self, ec_id):

        if self._exeCount < len(self._files):
            # read additional-data (1 record per file)
            with open(self._files[self._file_idx], 'r') as f:
                reader = csv.reader(f)
                for row in reader:
                    self._row = []
                    self._row.append(row)
            # set and write
            self._setOutPorts(self._row[0])
            self._writeOutPorts()
            self._exeCount += 1
            self._file_idx += 1

            logger.debug("-----act" + " : "+ str(self._exeCount))
            logger.debug(self._d_outImgAddData.fileName.data)
            logger.debug(self._d_outImgAddData.folderPath.data)
            logger.debug(self._d_outImgAddData.size.data)
            logger.debug(self._d_outImgAddData.robotID.data)
            logger.debug(self._d_outImgAddData.camNo.data)
            logger.debug(str(self._d_outImgAddData.gps.latitude)+":"+str(self._d_outImgAddData.gps.longitude)+":"+str(self._d_outImgAddData.gps.altitude))
            logger.debug(self._d_outImgAddData.gyro.data)
            logger.debug(self._d_outImgAddData.shootDate.data)
            logger.debug(self._d_outImgAddData.shootTime.data)
            logger.debug(self._d_outImgAddData.shootPlanName.data)
            logger.debug(self._d_outImgAddData.path.data)
        # additional data write end
        elif self._exeCount == len(self._files):
            # Image transfer start
            self._linSftp()

            timeAft = time.time()
            logger.debug( str(timeAft-self._timeBfr))
            self.deactivate(ec_id)

        return RTC.RTC_OK

    #	##
    #	#
    #	# The aborting action when main logic error occurred.
    #	# former rtc_aborting_entry()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onAborting(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The error action in ERROR state
    #	# former rtc_error_do()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onError(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The reset action that is invoked resetting
    #	# This is same but different the former rtc_init_entry()
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onReset(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The state update action that is invoked after onExecute() action
    #	# no corresponding operation exists in OpenRTm-aist-0.2.0
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#

    #	#
    # def onStateUpdate(self, ec_id):
    #
    #	return RTC.RTC_OK

    #	##
    #	#
    #	# The action that is invoked when execution context's rate is changed
    #	# no corresponding operation exists in OpenRTm-aist-0.2.0
    #	#
    #	# @param ec_id target ExecutionContext Id
    #	#
    #	# @return RTC::ReturnCode_t
    #	#
    #	#
    # def onRateChanged(self, ec_id):
    #
    #	return RTC.RTC_OK
 
    # set out port
    def _setOutPorts(self, row):

        self._d_outImgAddData.fileName.data = row[0]
        self._d_outImgAddData.folderPath.data = row[1]
        size = []
        size.append(int(float(row[2])))
        size.append(int(float(row[3])))
        self._d_outImgAddData.size.data = size
        self._d_outImgAddData.robotID.data = row[4]
        self._d_outImgAddData.camNo.data = row[5]
        self._d_outImgAddData.gps.latitude = float(row[6])
        self._d_outImgAddData.gps.longitude = float(row[7])
        self._d_outImgAddData.gps.altitude = float(row[8])
        self._d_outImgAddData.gyro.data.y = float(row[9])
        self._d_outImgAddData.gyro.data.p = float(row[10])
        self._d_outImgAddData.gyro.data.r = float(row[11])
        self._d_outImgAddData.shootDate.data = row[12]
        self._d_outImgAddData.shootTime.data = row[13]
        self._d_outImgAddData.shootPlanName.data = row[14]
        self._d_outImgAddData.path.data = self._UploadPath[0]
    
    # write out port
    def _writeOutPorts(self):

        OpenRTM_aist.setTimestamp(self._d_outImgAddData)
        self._d_outImgAddData.robotID.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.path.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.fileName.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.folderPath.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.size.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.camNo.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.gps.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.gyro.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.shootDate.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.shootTime.tm = self._d_outImgAddData.tm
        self._d_outImgAddData.shootPlanName.tm = self._d_outImgAddData.tm
        
        self._outImgAddDataOut.write()

    # Image transfer for SCP
    # def linScp(self):
    #     try:
    #         cmd = "scp {0}*.jpg {1}@{2}:{3}".format(pipes.quote(self._PictureFilePath[0]), pipes.quote(self._UserID[0]), pipes.quote(self._HostName[0]), pipes.quote(self._UploadPath[0]))
    #         logger.debug(cmd)
    #         # subprocess.check_call(["sh","-c",cmd])
    #         subprocess.Popen(cmd,shell=True)
    #         logger.debug("success")
    #     except:
    #         logger.debug("error")
    
    # -r付きPicture以下再帰的SCP
    # def linScp(self):
    #     try:
    #         cmd = "scp -r {0}Picture {1}@{2}:{3}".format(pipes.quote(self._PictureFilePath[0]), pipes.quote(self._UserID[0]), pipes.quote(self._HostName[0]), pipes.quote(self._UploadPath[0]))
    #         logger.debug( cmd
    #         # subprocess.check_call(["sh","-c",cmd])
    #         subprocess.Popen(cmd,shell=True)
    #         logger.debug("success")
    #     except:
    #         logger.debug("error")
    
    # Image transfer for SFTP
    def _linSftp(self):
        try:
            cmd = "bash sftp.sh {0} {1} {2} {3}".format(pipes.quote(self._PictureFilePath[0]), pipes.quote(self._UserID[0]), pipes.quote(self._HostName[0]), pipes.quote(self._UploadPath[0]))
            logger.debug(cmd)
            # subprocess.Popen(cmd,shell=True)
            subprocess.check_call(cmd, shell=True)
            logger.debug("success")
        except:
            logger.debug("error")

    # glob sort
    def _numAscSort(self, value):
        nums = re.compile(r'(\d+)')
        num = nums.split(value)
        num[1::2] = map(int, num[1::2])
        return num


def RobotImageUploaderInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=robotimageuploader_spec)
    manager.registerFactory(profile,
                            RobotImageUploader,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    RobotImageUploaderInit(manager)

    # Create a component
    comp = manager.createComponent("RobotImageUploader")


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()

if __name__ == "__main__":
    main()
