#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file VLAltitudeViewer.py
 @brief Altitude viewer for virtual leader
 @date $Date$

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist

# Import viewer
import altitude_view as view

# log output
from logging import getLogger, StreamHandler, DEBUG
logger = getLogger(__name__)
handler = StreamHandler()
handler.setLevel(DEBUG)
logger.setLevel(DEBUG)
logger.addHandler(handler)

# error message
ERR_MSG_01 = "WARNING: It exceeds the display area"


# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
vlaltitudeviewer_spec = ["implementation_id", "VLAltitudeViewer",
                         "type_name",         "VLAltitudeViewer",
                         "description",       "Altitude viewer for virtual leader",
                         "version",           "1.0.0",
                         "vendor",            "EAC",
                         "category",          "Drone",
                         "activity_type",     "STATIC",
                         "max_instance",      "1",
                         "language",          "Python",
                         "lang_type",         "SCRIPT",
                         "conf.default.MaxValue", "50",
                         "conf.__widget__.MaxValue", "spin",
                         "conf.__constraints__.MaxValue", "0<x<=1000",
                         ""]
# </rtc-template>

##
# @class VLAltitudeViewer
# @brief Altitude viewer for virtual leader
#
# Refer to the "機能仕様書_VLAltitudeViewer" for details
#
# Refer to the "機能仕様書_VLAltitudeViewer" for details
#
# Refer to the "機能仕様書_VLAltitudeViewer" for details
#
# Refer to the "機能仕様書_VLAltitudeViewer" for details
#
#


class VLAltitudeViewer(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_inVirtualLeaderGPS = RTC.DroneMap(RTC.Time(0, 0),
                                                  RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                              100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                              0.0, 0, RTC.GPS_FIX_NORMAL),
                                                  RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                              100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                              0.0, 0, RTC.GPS_FIX_NORMAL),
                                                  RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                                  RTC.TimedString(RTC.Time(0, 0), ""))
        """
        Position of virtual leader
         - Semantics: Altitude of virtual leader
         - Unit: m
        """
        self._inVirtualLeaderGPSIn = OpenRTM_aist.InPort(
            "inVirtualLeaderGPS", self._d_inVirtualLeaderGPS)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
        """
        Maximum value of scale
         - Name: MaxValue MaxValue
         - DefaultValue: 50
         - Unit: m
         - Range: 0<x1000
        """
        self._MaxValue = [50]

        # </rtc-template>

    ##
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onInitialize(self):
        # Bind variables and configuration variable
        self.bindParameter("MaxValue", self._MaxValue, "50")

        # Set InPort buffers
        self.addInPort("inVirtualLeaderGPS", self._inVirtualLeaderGPSIn)

        # Set OutPort buffers

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    #
    #    # @return RTC::ReturnCode_t
    #
    #    #
    # def onFinalize(self):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The activated action (Active state entry action)
    #    # former rtc_active_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onActivated(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The deactivated action (Active state exit action)
    #    # former rtc_active_exit()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onDeactivated(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onExecute(self, ec_id):

        self.read_altitude()

        self.set_altitude()

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The aborting action when main logic error occurred.
    #    # former rtc_aborting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onAborting(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onError(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The reset action that is invoked resetting
    #    # This is same but different the former rtc_init_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onReset(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    def read_altitude(self):

        self._altitude = False

        if self._inVirtualLeaderGPSIn.isNew():
            self._d_inVirtualLeaderGPS = self._inVirtualLeaderGPSIn.read()
            self._altitude = True

    def set_altitude(self):
        if self._altitude:
            altitude = self.round_altitude(self._d_inVirtualLeaderGPS.droneGPS.altitude)
            self.set_max()
            self.area_check(altitude)
            self.set_pos(altitude)

    def set_max(self):
        send = view.HeightValue()
        send.set_max_val(self._MaxValue[0])

    def set_pos(self, altitude):
        send = view.HeightValue()
        send.set_pos(altitude)

    def area_check(self, altitude):
        ruler = view.ScaleWidget()
        min = ruler.min_value(self._MaxValue[0])
        if altitude > self._MaxValue[0] or altitude < min:
            logger.warning(ERR_MSG_01)

    def round_altitude(self, altitude):
        ret = round(altitude, 6)

        return ret


def VLAltitudeViewerInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=vlaltitudeviewer_spec)
    manager.registerFactory(profile,
                            VLAltitudeViewer,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    VLAltitudeViewerInit(manager)

    # Create a component
    comp = manager.createComponent("VLAltitudeViewer")


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager(True)
    viewer = view.altitude_view().run()

if __name__ == "__main__":
    main()
