#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file VLAltitudeViewer_test.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import unittest
import RTC
import OpenRTM_aist
import VLAltitudeViewer

from mock import Mock

class TestVLAltitudeViewer(unittest.TestCase):

    def setUp(self):

        self.m = VLAltitudeViewer.VLAltitudeViewer(0)

        self.xx_logger = Mock()
        VLAltitudeViewer.logger.warning = self.xx_logger.warning
        VLAltitudeViewer.logger.error = self.xx_logger.error
        VLAltitudeViewer.logger.info = self.xx_logger.info
        VLAltitudeViewer.logger.debug = self.xx_logger.debug

    def test_read_altitude(self):

        xx = Mock()
        self.m._inVirtualLeaderGPSIn.isNew = xx.isNew
        self.m._inVirtualLeaderGPSIn.read = xx.read

        init_vlalt = RTC.DroneMap(RTC.Time(0, 0),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                RTC.TimedString(RTC.Time(0, 0), ""))
 
        vlalt_val = RTC.DroneMap(RTC.Time(0, 0),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522369, 139.938675, 10.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                RTC.TimedString(RTC.Time(0, 0), ""))

        in_is_new = [False, True]
        in_vlalt = [init_vlalt, vlalt_val]
        out_vlalt = [0.0, 10.0]        
        result = [False, True]

        for cnt in range(len(in_is_new)):
            xx.isNew.return_value = in_is_new[cnt]
            xx.read.return_value = in_vlalt[cnt]
            self.m.read_altitude()
            self.m.set_altitude()

            self.assertEqual(self.m._d_inVirtualLeaderGPS.droneGPS.altitude, out_vlalt[cnt])
            self.assertEqual(self.m._altitude, result[cnt])


if __name__ == '__main__':
    unittest.main()
