"""
 @file altitude_view.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
from kivy.app import App
from kivy.uix.widget import Widget
from kivy.properties import ObjectProperty
from kivy.core.window import Window
from kivy.clock import Clock
import math

# window size
WIN_X = 200
WIN_Y = 260

# view size
VIEW_Y = 230


# global variable change class
class HeightValue:

    # max value&position value
    max_val = 50
    pos = 0

    # max value setter getter
    @classmethod
    def set_max_val(cls, max_val):
        cls.max_val = max_val

    @classmethod
    def get_max_val(cls):
        return cls.max_val

    # position value setter getter
    @classmethod
    def set_pos(cls, pos):
        cls.pos = pos

    @classmethod
    def get_pos(cls):
        return cls.pos


# view class
class ViewWidget(Widget):

    # parts
    ruler = ObjectProperty(None)
    scale = ObjectProperty(None)
    max = ObjectProperty(None)
    min = ObjectProperty(None)
    middle = ObjectProperty(None)
    home = ObjectProperty(None)
    bow = ObjectProperty(None)
    arrow = ObjectProperty(None)
    position = ObjectProperty(None)

    heightVal = HeightValue()

    # viewer execution
    def __init__(self):
        super(ViewWidget, self).__init__()
        event = Clock.schedule_interval(self.view_update, 0.1)

    # view update
    def view_update(self, dt):
        self.scale_update()
        self.arrow_update()

    # scale value update
    def scale_update(self):
        maxVal = self.ruler.max_value(self.heightVal.get_max_val())
        minVal = self.ruler.min_value(maxVal)
        middleVal = self.ruler.middle_value(maxVal)
        self.max.text = str(maxVal)
        self.min.text = str(minVal)
        self.middle.text = str(middleVal)

    # current position update
    def arrow_update(self):
        posVal = self.bow.position_value(self.heightVal.get_pos())
        self.arrow.center_y = self.arrow_center_y()
        self.position.text = str(posVal)
        self.position.center_y = self.arrow.center_y

    # arrow position change
    def arrow_center_y(self):
        maxVal = self.ruler.max_value(self.heightVal.get_max_val())
        minVal = self.ruler.min_value(maxVal)
        posVal = self.bow.position_value(self.heightVal.get_pos())
        ret = VIEW_Y * \
            self.bow.arrow_move(
                maxVal, minVal, posVal) + self.bow.margin_bottom()
        return ret


# arrow class
class ArrowWidget(Widget):

    # arrow move
    def arrow_move(self, max, min, pos):
        ret = math.fabs(pos + math.fabs(min)) / \
            (math.fabs(max) + math.fabs(min))
        return ret

    # arrow margin bottom
    def margin_bottom(self):
        ret = (WIN_Y - VIEW_Y) / 2
        return ret

    # get position value
    def position_value(self, pos):
        ret = pos
        return ret


# scale class
class ScaleWidget(Widget):

    # get max value
    def max_value(self, max_val):
        ret = max_val
        return ret

    # get min value
    def min_value(self, max_val):
        ret = max_val / -2
        return ret

    # get middle value
    def middle_value(self, max_val):
        ret = max_val / 2
        return ret


class altitude_view(App):

    def build(self):
        self.title = "Virtual-leader Alt"
        Window.size = (WIN_X, WIN_Y)
        return ViewWidget()

if __name__ == "__main__":
    altitude_view().run()
